#
# Universal USB Driver
# --------------------
#
# Distributed under the terms of the GNU Public License Version 2
# See files COPYRIGHT and COPYING at the root distribution directory
#
# PPC Stuff for iMac by Paul Mackerras <paulus@cs.anu.edu.au>
#
# Common make rules for stand-alone compilation 


# no user-serviceable parts below this line

ifndef IN_LINUX_KERNEL
.PHONY: TAGS tags dep clean distclean realclean doc docs tar cvstar love

%.i: %.c
	$(CPP) $(CPPFLAGS) $(CFLAGS) $(EXTRA_CFLAGS) -c $^ -o $@

.PRECIOUS: %.i
%.ci: %.i
	-grep -v -e "^#" -e "^ *$$" $^ | indent > $@
	mv -f $@ $^

%.s: %.c
	$(CC) -S $(CFLAGS) $(EXTRA_CFLAGS) -c $^ -o $@

%.io: %.i
	$(CC) $(CFLAGS) $(EXTRA_CFLAGS) -c $^ -o $@

dep:
ifneq "$(strip $(SUBDIRS))" ""
	set -e; for i in $(SUBDIRS); do $(MAKE) -C $$i $@; done
endif	
ifneq "$(strip $(SUB_DIRS))" ""
	set -e; for i in $(SUB_DIRS); do $(MAKE) -C $$i $@; done
endif	
ifneq "$(strip $(MOD_SUB_DIRS))" ""
	set -e; for i in $(MOD_SUB_DIRS); do $(MAKE) -C $$i $@; done
endif	
	-$(CPP) -M $(CPPFLAGS) $(CFLAGS) $(EXTRA_CFLAGS) *.c > .depend

CLEANSTUFF += *~ *.[aios] *.bak *.ci *.io
DISTCLEANSTUFF += .depend \\\#*\\\# .*.flags
REALCLEANSTUFF += TAGS 

clean:
	rm -f $(CLEANSTUFF)
ifneq "$(strip $(SUBDIRS))" ""
	set -e; for i in $(SUBDIRS); do $(MAKE) -C $$i $@; done
endif

distclean:
	rm -f $(DISTCLEANSTUFF) $(CLEANSTUFF)
ifneq "$(strip $(SUBDIRS))" ""
	set -e; for i in $(SUBDIRS); do $(MAKE) -C $$i $@; done
endif

realclean:
	rm -f $(REALCLEANSTUFF) $(DISTCLEANSTUFF) $(CLEANSTUFF)
ifneq "$(strip $(SUBDIRS))" ""
	set -e; for i in $(SUBDIRS); do $(MAKE) -C $$i $@; done
endif

doc docs:
ifneq "$(strip $(SUBDIRS))" ""
	set -e; for i in $(SUBDIRS); do $(MAKE) -C $$i $@; done
endif

TAGS: *.c *.h
TAGS tags:
	find . -name "*.[ch]" | etags -

# archive

ifeq "$(TARNAME)" ""
TARNAME=uusbd
TARDIR=uusbd
endif

tar:
	tar c -C ~/src/usb \
		--exclude '*~' --exclude '*.bak' \
		--exclude '*.[aios]' --exclude '*.ci' --exclude '*.io' \
		--exclude '*/System.map' --exclude '.*flags' \
		--exclude '*./TAGS' --exclude '*/.depend' \
		--exclude '#*#' \
		-f - $(TARDIR) \
	| bzip2 -9f \
	> ../$(TARNAME)-$$(date +%Y%m%d%H%M).tar.bz2

cvstar: 
	tar c -C /home/CVS \
		--exclude '*~' --exclude '*.bak' \
		--exclude '*.[aios]' --exclude '*.ci' --exclude '*.io' \
		--exclude '*/System.map' --exclude '.*flags' \
		--exclude '*./TAGS' --exclude '*/.depend' \
		--exclude '#*#' \
		-f - $(TARDIR) \
	| bzip2 -9f \
	> ../$(TARNAME)-$$(date +%Y%m%d%H%M).CVS.tar.bz2

# Flower Power :)

love:
	@echo "not war!"

# Symbols
KVER=$(shell uname -r)
sysmap System.map: /boot/System.map-$(KVER)
	patches/make_sysmap \
				/boot/System.map-$(KVER) /proc/ksyms \
				/lib/modules/$(KVER)/*/*.o \
				usb.o */usb-*.o \
				> System.map

endif # !IN_LINUX_KERNEL
