
/* USB Audio Devices Driver v$Revision: 1.7 $ 
 *
 * (C) 1997 by Iaky Prez Gonzlez <inaky@peloncho.fis.ucm.es>
 *
 * Distributed under the terms of the GNU Public License Version 2
 * See files COPYRIGHT and COPYING at the root distribution directory
 *
 * $Id: cfg.c,v 1.7 1998/11/14 17:17:10 inaky Exp $
 *
 * Changelog at the bottom 
 *
 *     USB Audio Devices - Configuration
 *     ---------------------------------
 *
 *  This is the embrion of the USB AUDD.
 *
 */

#include <uusbd/usbd.h>

#define GDLF_IDENT          "USB-AUDD"
#define GDLF_LOCAL          /* Define if want local settings */
#define GDLF_DEF_LOGLEVEL   9
#define GDLF_ASSERTIONS     /* want assertions (#undef if not) */
#define GDLF_FNTRACE        /* want function traces */
#define GDLF_LOCATIONS      /* want locations in log messages */
#include <uusbd/gdlf.h>

#include <uusbd/usb-audd.h>


    /* FIXME: Detailed desc
     *
     *  First time we are called (*paudd == NULL), we just create an
     * audd structure, fill it's dtor, register it, fill in the data
     * from the descriptors and assign it to cfel's data,
     * referencing. Then we finish by setting the cfel's configured
     * flag to 1.
     *
     * Compliance:      USB Spec 1.0 xxxx
     *
     * Parameters:  
     *
     *  type            descr
     *
     * Global:
     *
     *  type name       descr
     *
     * Returns:
     *
     *  int             Number of cfels configured, errno code < 0 on
     *                  error. 
     *
     * Global:
     *
     *  type name       descr
     */

int audd_config_iface (ts_audd** paudd, ts_usb_cfel* cfel,
                       ts_usb_device* device, ts_usb_cfg* cfg, 
                       ts_usb_iface* iface)
{
    int errcode;
    ts_audd *audd;
    
    gdlf_fntrace ("audd_config_iface (0x%p, 0x%p, 0x%p, 0x%p, 0x%p)", 
                  paudd, cfel, device, cfg, iface);
    gdlf_aassert (paudd != NULL, return -EINVAL);
    gdlf_aassert (device != NULL, return -EINVAL);
    gdlf_aassert (cfel != NULL, return -EINVAL);
    gdlf_aassert (cfg != NULL, return -EINVAL);
    gdlf_aassert (iface != NULL, return -EINVAL);
    gdlf_aassert (iface->active != NULL, return -EINVAL);
    gdlf_aassert (&iface->active->cfel == cfel, return -EINVAL);
    gdlf_aassert (cfel->class == AUDD_CLASS, return -EINVAL);

        /* Create new audio structure? */

    if (*paudd == NULL)             /* got to create a new one */
    {
        audd = ts_audd_init();
        if (audd == NULL)
        {
            gdlf_err ("No memory to configure interface %d "
                      "of device (%d %d)\n",    
                      iface->addr,
                      usb_host_addr (device), usb_dev_addr (device));
            audd_errno = -USBE_NO_MEMORY;
            usb_errno = -USBE_NO_MEMORY;
            return -ENOMEM;
        }
        ts_3snode_ref (&device->msnode);
        audd->device = device;
        *paudd = audd;
        cfel->data = &audd->snode;      /* reference later */
        errcode = audd_register (audd); /* register ... */
        if (errcode < 0)                /* ok? ... */
            return errcode;             /* ... no, audd_register() complained */
    }
    else
        audd = *paudd;
    ts_snode_ref (&audd->snode);
    cfel->configured = (unsigned long)audd;

        /*
         *  Ok, initialize the interface data accordingly to it's
         * subclass. 
         */

#if 0
    switch (iface->active->cfel.subclass)
    {
    }
#endif
        
    return 1;
}

    /* Configure a device class interface
     *
     *  This function may be used to be called from audd_config() to
     * configure the class part of a configurable element.
     *
     *  Audio devices have many interfaces which are independent, but
     * for the audio purposes, we need them all, so the first time we
     * are called on a device, we search for all the interfaces we
     * could be interested in (from the active USB configuration) and
     * configure them.
     *
     * Returns:
     *
     *  int             Number of cfels configured, errno code < 0 on
     *                  error. 
     *
     * Global:
     *
     *  int audd_errno On error, error code.  
     *  int usb_errno   On error, error code.
     */

           
int audd_config (ts_usb_cfel* cfel, ts_usb_device* device, 
                 ts_usb_cfg* cfg, t_usb_cfg cfg_idx, 
                 ts_usb_iface* iface, t_usb_iface iface_idx, 
                 ts_usb_aset* aset, t_usb_aset aset_idx, __u32 flags)
{
    int errcode, cfels = 0;
    ts_audd *audd = NULL;
    ts_usb_iface *ifaceitr, *iface_top;
    
    gdlf_fntrace ("audd_config (0x%p, 0x%p, 0x%p, "
                  "0x%p, 0x%p, %u)", cfel, device, cfg, 
                  iface, aset, flags);
    gdlf_aassert (cfel != NULL, return -EINVAL);
    gdlf_aassert (device != NULL, return -EINVAL);
    
    if (cfel->class != AUDD_CLASS)
    {
        gdlf_err ("Pretended to configure as audio non-audio "
                  "class interface %d of device (%d %d)\n",
                  iface->addr, 
                  usb_host_addr (device), usb_dev_addr (device));
        return 0;
    }

        /* Ok, configure this cfel */

    errcode = audd_config_iface (&audd, cfel, device, cfg, iface);
    if (errcode <= 0)
    {
        gdlf_err ("Couldn't configure interface %d of device (%d %d)\n",
                  iface->addr, 
                  usb_host_addr (device), usb_dev_addr (device));
        return errcode;
    }
    else if (errcode > 0)
        cfels++;
    
        /*
         * Walk all the device iface's, searching for more audd ifaces
         */

    for (ifaceitr = cfg->iface, iface_top = ifaceitr + cfg->ifaces; 
         ifaceitr < iface_top; ifaceitr++)
    {
        gdlf_aassert (ifaceitr->active != NULL, continue);
        
        if (ifaceitr->active->cfel.class == AUDD_CLASS)
        {
            gdlf_debug ("Found interface %d of device (%d %d)\n",
                        ifaceitr->addr, 
                        usb_host_addr (device), usb_dev_addr (device));
            errcode = audd_config_iface (&audd, &ifaceitr->active->cfel, 
                                         device, cfg, ifaceitr);
            if (errcode <= 0)
                gdlf_debug ("Couldn't configure interface???\n");
            else if (errcode > 0)
                cfels++;
        }
    }
    return cfels;
}


    /* Unconfigure all cfels configured by this driver
     *
     *  This function may be used to unconfigure a driver's
     * class-configured configurable elements.
     *
     * Parameters:  
     *
     *  ts_usb_device*  Pointer to the device structure to
     *                  unconfigure. 
     *
     *  unsigned        USB_CFG_* flags indicating why is this
     *                  function called.  
     *
     * Returns:
     *
     *  int             0 if ok, errno code < 0 on error.
     *
     * Global:
     *
     *  int audd_errno On error, error code.  
     *  int usb_errno   On error, error code.  
     */

int audd_unconfig (ts_usb_device* device, unsigned flags)
{
    t_usb_cfg cfg_cnt;
    ts_usb_cfg *cfg;
    ts_usb_iface *iface, *iface_top;
    ts_usb_aset *aset, *aset_top;
    
    gdlf_fntrace ("audd_unconfig (0x%p, %u)", device, flags);
    gdlf_aassert (device != NULL, return -EINVAL);

    for (cfg_cnt = 0; cfg_cnt < device->cfgs; cfg_cnt++)
    {
        cfg = device->cfg[cfg_cnt];
        if (cfg == NULL)        /* unloaded? skip */
            continue;
        
            /* Walk cfg's interfaces */

        for (iface = cfg->iface, iface_top = iface + cfg->ifaces; 
             iface < iface_top; iface++)
        {
                /* Walk the iface's asets */

            for (aset = iface->aset, aset_top = aset + iface->asets; 
                 aset < aset_top; aset++)

                      /* cfged by us? */

                if (aset->cfel.configured
                    && aset->cfel.data
                    && aset->cfel.data->dtor.fn
                       == (tf_dtor)audd_destroy)
                {
                    ts_snode_unref (aset->cfel.data, NULL);
                    aset->cfel.data = NULL;
                    aset->cfel.configured = 0;      /* uncfged */
                }
        }         
    }
    return 0;
}

/* Log:
 *
 * $Log: cfg.c,v $
 * Revision 1.7  1998/11/14 17:17:10  inaky
 * Moved the include files to uusbd/include/uusbd, to ease integration
 * into the Linux kernel.
 *
 * Revision 1.6  1998/10/31 02:15:06  inaky
 * Removed all the d_* pointers from ts_usb_cfel and collapsed them into
 * a single pointer; only one was really used.
 *
 * Revision 1.5  1998/08/30 13:53:15  inaky
 * Comment fixes.
 *
 * Revision 1.4  1998/08/28 23:57:53  inaky
 * Unscrewed down ..
 *
 * Revision 1.2  1998/08/25 04:05:50  inaky
 * Added the basic configuration code.
 *
 * Revision 1.1  1998/08/24 22:16:57  inaky
 * Added first revision of embrionic audio driver
 *
 *
 */



