
/* USB Audio Devices Driver v$Revision: 1.6 $ 
 *
 * (C) 1997 by Iaky Prez Gonzlez <inaky@peloncho.fis.ucm.es>
 *
 * Distributed under the terms of the GNU Public License Version 2
 * See files COPYRIGHT and COPYING at the root distribution directory
 *
 * $Id: driver.c,v 1.6 1998/11/14 17:17:11 inaky Exp $
 *
 * Changelog at the bottom 
 *
 *     USB Audio Devices - Driver management
 *     -------------------------------------
 *
 *  This is the embrion of the USB AUDD.
 *
 */


#include <linux/init.h>     /* __init */
#define __NO_VERSION__
#include <linux/module.h>   /* MOD_*_USE_COUNT */
#include <asm/spinlock.h>   /* locks */

#include <uusbd/usbd.h>

#define GDLF_IDENT          "USB-AUDD"
#define GDLF_LOCAL          /* Define if want local settings */
#define GDLF_DEF_LOGLEVEL   9
#define GDLF_ASSERTIONS     /* want assertions (#undef if not) */
#define GDLF_FNTRACE        /* want function traces */
#define GDLF_LOCATIONS      /* want locations in log messages */
#include <uusbd/gdlf.h>

#include <uusbd/usb-audd.h>


atomic_t usecnt;
volatile int audd_errno = 0;

#define NAME   /* short!! */                    \
    "USB Audio Devices Driver v$Revision: 1.6 $"

#define COPYRIGHT                               \
    "(c) 1998 Inaky Perez Gonzalez "            \
    "<inaky@peloncho.fis.ucm.es>"

#define VENDOR 0
#define VENDORNAME                              \
    NULL

#define PRODUCT 0
#define DEVICENAME                              \
    NULL

#define CLASS AUDD_CLASS
#define CLASSNAME                               \
    "USB Audio Devices"

#define SUBCLASS 0
#define SUBCLASSNAME                            \
    NULL

#define PROTOCOL 0
#define PROTOCOLNAME                            \
    NULL

#define SNODE_NULL { {NULL}, NULL, 0}

ts_usb_driver* audd_driver = NULL;


    /* 
     * Linux module support
     */

#ifdef MODULE

int init_module (void)
{
    return audd_init();
}

void cleanup_module (void)
{
    audd_release();
    audd_shutdown();
    return;
}

#endif


    /* Initialize the audd driver
     *
     * Returns:
     *
     *  int             0 if ok, errno code < 0 on error.
     *
     * Global:
     *
     *  int audd_errno On error, error code.  
     *  int usb_errno   On error, error code.  
     */

int __init audd_init (void)
{
    int errcode;
    
    gdlf_fntrace ("audd_init()");
    gdlf_info (NAME "\n");
#ifdef MODULE
    gdlf_info (COPYRIGHT "\n");
#endif

    audd_driver = ts_usb_driver_init();
    if (audd_driver == NULL)
    {
        usb_errno = audd_errno = -USBE_NO_MEMORY;
        return -ENOMEM;
    }

        /* Set driver's attributes */

    audd_driver->vendor = VENDOR;
    audd_driver->product = PRODUCT;
    audd_driver->class = CLASS;
    audd_driver->subclass = SUBCLASS;
    audd_driver->protocol = PROTOCOL;

        /* Set info */

    audd_driver->name = NAME;                   /* driver name */
    audd_driver->copyright = COPYRIGHT;         /* driver copyright */
    audd_driver->vendorname = VENDORNAME;       /* device's vendor name */
    audd_driver->productname = DEVICENAME;      /* device's name */
    audd_driver->classname = CLASSNAME;         /* class' name */
    audd_driver->subclassname = SUBCLASSNAME;   /* subclass' name */
    audd_driver->protocolname = PROTOCOLNAME;   /* protocol name */

    audd_driver->config = audd_config;          /* config method */
    audd_driver->unconfig = audd_unconfig;      /* unconfig method */
    audd_driver->reference = audd_reference;    /* reference method */
    ts_snode_ref (&audd_driver->snode);         /* we hold a reference to it */
    
    ts_slist_clean (&audd_list);                /* initialize the list */
    errcode = usb_drv_register (audd_driver);   /* register driver */
    if (errcode)
    {
        gdlf_err ("Cannot register driver: "
                  "errno/usb %d/%d\n",
                  errcode, usb_errno);
        if (audd_driver->snode.next == NULL)
        {
            ts_usb_driver_clean (audd_driver);
            usb_kfree (audd_driver);
        }
        audd_errno = -USBE_DRIVER_CANT_REG;
        return errcode;
    }
        
        /* FIXME: do you class' stuf */
#warning FIXME: do class stuff!

    return 0;
}


    /* Release the audd driver
     *
     *  Do what is needed to ensure the use count of the driver is
     * zero and can be unloaded from the kernel.
     *
     * Returns:
     *
     *  int             0 if ok, errno code < 0 on error.
     *
     * Global:
     *
     *  int audd_errno On error, error code.  
     *  int usb_errno   On error, error code.  
     */

int audd_release (void)
{
    gdlf_fntrace ("audd_release()");

#warning FIXME: audd_release() not implemented!

    return -ENOSYS;
}


    /* Shutdown the skel driver
     *
     * Returns:
     *
     *  int             0 if ok, errno code < 0 on error.
     *
     * Global:
     *
     *  int audd_errno On error, error code.  
     *  int usb_errno   On error, error code.  
     */

int audd_shutdown (void)
{
    gdlf_fntrace ("audd_shutdown()");

        /* FIXME: do you class' stuf */

    ts_snode_unref (&audd_driver->snode, NULL);
    return usb_drv_unregister (audd_driver);
}


    /* Driver reference
     *
     *  The 'reference' method is used when we take any of the driver's
     * code or data structures to use it. If so, we call it with a
     * positive argument (one reference, regardless the value of the
     * _positive_ number). When we are done, we release the reference
     * calling it with a negative number. It always returns the number
     * of references or < 0 on error. Calling it with an argument of
     * zero will just return the number of references. 
     * Compliance:      USB Spec 1.0 xxxx
     *
     * Parameters:  
     *
     *  type            descr
     *
     * Global:
     *
     *  type name       descr
     *
     * Returns:
     *
     *  type            descr
     *
     * Global:
     *
     *  type name       descr
     */

int audd_reference (ts_usb_device* device, int n)
{
    gdlf_fntrace ("audd_reference (0x%p, %d)", device, n);
    gdlf_aassert (device != NULL, return -EINVAL);

    if (n > 0)
    {
        atomic_inc (&usecnt);
        MOD_INC_USE_COUNT;
    }
    else if (n < 0)
    {
        atomic_dec (&usecnt);
        MOD_DEC_USE_COUNT;
    }
    return atomic_read (&usecnt);
}

/* Log:
 *
 * $Log: driver.c,v $
 * Revision 1.6  1998/11/14 17:17:11  inaky
 * Moved the include files to uusbd/include/uusbd, to ease integration
 * into the Linux kernel.
 *
 * Revision 1.5  1998/08/30 13:53:27  inaky
 * Comment fixes.
 *
 * Revision 1.4  1998/08/28 23:58:23  inaky
 * Unscrewed off.
 *
 * Revision 1.2  1998/08/25 04:06:02  inaky
 * Removed module stuff.
 *
 * Revision 1.1  1998/08/24 22:16:58  inaky
 * Added first revision of embrionic audio driver
 *
 */

