
/* USB Audio Devices Driver v$Revision: 1.6 $ 
 *
 * (C) 1997 by Iaky Prez Gonzlez <inaky@peloncho.fis.ucm.es>
 *
 * Distributed under the terms of the GNU Public License Version 2
 * See files COPYRIGHT and COPYING at the root distribution directory
 *
 * $Id: generic.c,v 1.6 1998/11/14 17:17:12 inaky Exp $
 *
 * Changelog at the bottom 
 *
 *     USB Audio Devices - Generic stuff
 *     ---------------------------------
 *
 *  This is the embrion of the USB AUDD.
 *
 */

#include <uusbd/usbd.h>

#define GDLF_IDENT          "USB-AUDD"
#define GDLF_LOCAL          /* Define if want local settings */
#define GDLF_DEF_LOGLEVEL   9
#define GDLF_ASSERTIONS     /* want assertions (#undef if not) */
#define GDLF_FNTRACE        /* want function traces */
#define GDLF_LOCATIONS      /* want locations in log messages */
#include <uusbd/gdlf.h>

#include <uusbd/usb-audd.h>


    /* Allocates and initializes to a known state a ts_audd*
     * structure.
     *
     * Returns:
     *  ts_audd*    Pointer to newly allocated hub structure. Must
     *              be freed with usb_kfree().
     */

ts_audd* ts_audd_init (void)
{
    ts_audd* audd;
    
    gdlf_fntrace ("ts_audd_init()");
    audd = usb_kmalloc (sizeof (ts_audd), GFP_KERNEL);
    if (audd == NULL)
    {
        gdlf_err ("Cannot allocate space for a "
                  "ts_audd structure\n");
        return NULL;
    }
    ts_audd_clean (audd);
    return audd;              
}


    /* Free a ts_audd structure's contents. 
     *
     * Checks for the magic number being ok. Remember that the
     * structure iself is not freed, just it's contents.
     *
     * Parameters:  
     *
     *  ts_hubd*        Pointer to the hub structure to free.
     *
     * Returns:
     *
     *  int             0 if ok, errno code < 0 on error.
     *
     * Global:
     *
     *  int usb_errno   Modified on error with an USB error code.  
     */

void ts_audd_release (ts_audd* audd)
{
    gdlf_fntrace ("ts_audd_free (0x%p)", audd);
    gdlf_aassert (audd != NULL, return);
    gdlf_assert (audd->snode.dtor.fn == (tf_dtor)audd_destroy);
    gdlf_assert (ts_slist_next (&audd->snode) == NULL);
    
    if (audd->device)
        ts_3snode_unref (&audd->device->msnode, NULL);

    if (audd->aic)
    {   
#warning FIXME: finish this!!!
#if 0
        ts_audd_aic_a_release (audd->aic, audd->aics);
#endif
        usb_kfree (audd->aic);
    }
    return;
}


/* Log:
 *
 * $Log: generic.c,v $
 * Revision 1.6  1998/11/14 17:17:12  inaky
 * Moved the include files to uusbd/include/uusbd, to ease integration
 * into the Linux kernel.
 *
 * Revision 1.5  1998/08/30 13:53:35  inaky
 * Comment fixes.
 *
 * Revision 1.4  1998/08/29 00:02:25  inaky
 * unscrewed off
 *
 * Revision 1.2  1998/08/25 04:06:24  inaky
 * Added warning comments on unfinished stuff.
 *
 * Revision 1.1  1998/08/24 22:16:58  inaky
 * Added first revision of embrionic audio driver
 *
 */

