
/* USB Audio Devices Driver v$Revision: 1.5 $ 
 *
 * (C) 1997 by Iaky Prez Gonzlez <inaky@peloncho.fis.ucm.es>
 *
 * Distributed under the terms of the GNU Public License Version 2
 * See files COPYRIGHT and COPYING at the root distribution directory
 *
 * $Id: hkeep.c,v 1.5 1998/11/14 17:17:13 inaky Exp $
 *
 * Changelog at the bottom 
 *
 *     USB Audio Devices - 
 *     ---------------------------------
 *
 *  This is the embrion of the USB AUDD.
 *
 */

#include <asm/spinlock.h>   /* locks */
#include <linux/module.h>   /* MOD_*_USE_COUNT */

#include <uusbd/usbd.h>

#define GDLF_IDENT          "USB-AUDD"
#define GDLF_LOCAL          /* Define if want local settings */
#define GDLF_DEF_LOGLEVEL   9
#define GDLF_ASSERTIONS     /* want assertions (#undef if not) */
#define GDLF_FNTRACE        /* want function traces */
#define GDLF_LOCATIONS      /* want locations in log messages */
#include <uusbd/gdlf.h>

#include <uusbd/usb-audd.h>


ts_slist audd_list;  /* list of registered class structures */


    /* Unregister an audd element
     *
     * This is the destructor function.
     *
     * Parameters:  
     *
     *  ts_audd* Class element to unregister
     *
     * Returns:
     *
     *  int             0 if ok, errno code < 0 on error.
     *
     * Global:
     *
     *  int usb_errno   On error, error code.  
     *  int audd_errno On error, error code.  
     */

int audd_unregister (ts_audd* audd)
{
    gdlf_fntrace ("audd_unregister (0x%p)", audd);
    gdlf_aassert (audd != NULL, return -EINVAL);
    gdlf_assert (audd->snode.dtor.fn == (tf_dtor)audd_destroy);

        /* Let's get it removed from the list */

    ts_slist_del (&audd_list, &audd->snode);
#warning FIXME: remove this, it makes modules stay for ever
    MOD_DEC_USE_COUNT;
    return 0;                   /* bye */
}


    /* FIXME: Detailed desc
     *
     * Compliance:      USB Spec 1.0 xxxx
     *
     * Parameters:  
     *
     *  type            descr
     *
     * Global:
     *
     *  type name       descr
     *
     * Returns:
     *
     *  type            descr
     *
     * Global:
     *
     *  type name       descr
     */

int audd_destroy (ts_audd* audd)
{
    t_usb_cfg cfg_cnt;
    ts_usb_cfg *cfg;
    ts_usb_iface *iface, *iface_top;
    ts_usb_aset *aset, *aset_top;
    
    gdlf_fntrace ("audd_destroy (0x%p)", audd);
    gdlf_aassert (audd != NULL, return -EINVAL);
    gdlf_assert (audd->snode.dtor.fn == (tf_dtor)audd_destroy);
    gdlf_aassert (audd->device, return -EINVAL);
    gdlf_aassert (audd->device->cfg, return -EINVAL);
    
    ts_snode_ref (&audd->snode);

        /* 
         *  Ok, walk the cfgs, ifaces and aset, searching for an
         * element configured for this audd. If found, unref audd and
         * keep on. 
         */

    for (cfg_cnt = 0; cfg_cnt < audd->device->cfgs; cfg_cnt++)
    {
        cfg = audd->device->cfg[cfg_cnt];
        if (cfg == NULL)        /* unloaded? skip */
            continue;
        
            /* Walk cfg's interfaces */

        for (iface = cfg->iface, iface_top = iface + cfg->ifaces; 
             iface < iface_top; iface++)
        {
                /* Walk the iface's asets */

            for (aset = iface->aset, aset_top = aset + iface->asets; 
                 aset < aset_top; aset++)

                      /* cfged by us? */

                if (aset->cfel.configured == (unsigned long)&audd->snode)                    
                {
                    ts_snode_unref (&audd->snode, NULL);
                    aset->cfel.configured = 0;      /* uncfged */
                }
        }         
    }

    audd_unregister (audd);     /* unregister */
    ts_snode_unref_nd (&audd->snode);
    ts_audd_release (audd);     /* clean the struct */
    return 0;                   /* bye */
}


    /* Register an audd element
     *
     * Parameters:  
     *
     *  ts_audd* Class element to register
     *
     * Returns:
     *
     *  int             0 if ok, errno code < 0 on error.
     *
     * Global:
     *
     *  int usb_errno   On error, error code.  
     *  int audd_errno On error, error code.  
     */

int audd_register (ts_audd* audd)
{
    int errcode;
    
    gdlf_fntrace ("audd_register (0x%p)", audd);
    gdlf_aassert (audd != NULL, return -EINVAL);

    ts_dtor_fill (&audd->snode.dtor, (tf_dtor)audd_destroy);

        /* get into the list */

    errcode = ts_slist_prep (&audd_list, &audd->snode);

    if (errcode)
        return errcode;
#warning FIXME: remove this, it makes modules stay for ever
    MOD_INC_USE_COUNT;
    return 0;                   /* bye */
}

/* Log:
 *
 * $Log: hkeep.c,v $
 * Revision 1.5  1998/11/14 17:17:13  inaky
 * Moved the include files to uusbd/include/uusbd, to ease integration
 * into the Linux kernel.
 *
 * Revision 1.4  1998/08/29 00:05:30  inaky
 * Unscrewed off.
 *
 * Revision 1.2  1998/08/25 04:06:57  inaky
 * Added destructor.
 *
 * Revision 1.1  1998/08/24 22:16:58  inaky
 * Added first revision of embrionic audio driver
 *
 */


