
/* USB Communications Class Devices Driver v$Revision: 1.3 $ 
 *
 * (C) 1998 by Iaky Prez Gonzlez <inaky@peloncho.fis.ucm.es>
 *
 * Distributed under the terms of the GNU Public License Version 2
 * See files COPYRIGHT and COPYING at the root distribution directory
 *
 * $Id: driver.c,v 1.3 1998/11/14 17:18:06 inaky Exp $
 *
 * Changelog at the bottom 
 *
 *
 *     Communications Class Driver - Driver housekeeping
 *     -------------------------------------------------
 */

#include <linux/init.h>     /* __init */
#define __NO_VERSION__
#include <linux/module.h>   /* MOD_*_USE_COUNT */
#include <asm/spinlock.h>   /* locks */

#include <uusbd/usbd.h>

#define GDLF_IDENT          "USB-CDCD"
#define GDLF_LOCAL          /* Define if want local settings */
#define GDLF_DEF_LOGLEVEL   9
#define GDLF_ASSERTIONS     /* want assertions (#undef if not) */
#define GDLF_FNTRACE        /* want function traces */
#define GDLF_LOCATIONS      /* want locations in log messages */
#include <uusbd/gdlf.h>

#include <uusbd/usb-cdcd.h>


atomic_t usecnt;

#define NAME   /* short!! */                    \
    "Communication Devices Class Driver v$Revision: 1.3 $ "

#define COPYRIGHT                               \
    "(c) 1998 Inaky Perez Gonzalez "            \
    "<inaky@peloncho.fis.ucm.es>"

#define VENDOR 0
#define VENDORNAME NULL

#define PRODUCT 0
#define DEVICENAME NULL

#define CLASS 0x2
#define CLASSNAME                               \
    "Communication Class Devices"

#define SUBCLASS 0
#define SUBCLASSNAME NULL

#define PROTOCOL 0
#define PROTOCOLNAME NULL

#define SNODE_NULL { {NULL}, NULL, 0}

ts_usb_driver* cdcd_driver = NULL;


    /* 
     * Linux module support
     */

#ifdef MODULE

int init_module (void)
{
    return cdcd_init();
}

void cleanup_module (void)
{
    cdcd_release();
    cdcd_shutdown();
    return;
}

#endif


    /* Initialize the cdcd driver
     *
     * Returns:
     *
     *  int             0 if ok, errno code < 0 on error.
     */

int __init cdcd_init (void)
{
    int errcode;
    
    gdlf_fntrace ("cdcd_init()");
    gdlf_info (NAME "\n");
#ifdef MODULE
    gdlf_info (COPYRIGHT "\n");
#endif

    cdcd_driver = ts_usb_driver_init();
    if (cdcd_driver == NULL)
    {
        usb_errno = -USBE_NO_MEMORY;
        return -ENOMEM;
    }

        /* Set driver's attributes */

    cdcd_driver->vendor = VENDOR;
    cdcd_driver->product = PRODUCT;
    cdcd_driver->class = CLASS;
    cdcd_driver->subclass = SUBCLASS;
    cdcd_driver->protocol = PROTOCOL;

        /* Set info */

    cdcd_driver->name = NAME;                   /* driver name */
    cdcd_driver->copyright = COPYRIGHT;         /* driver copyright */
    cdcd_driver->vendorname = VENDORNAME;       /* device's vendor name */
    cdcd_driver->productname = DEVICENAME;      /* device's name */
    cdcd_driver->classname = CLASSNAME;         /* class' name */
    cdcd_driver->subclassname = SUBCLASSNAME;   /* subclass' name */
    cdcd_driver->protocolname = PROTOCOLNAME;   /* protocol name */

    cdcd_driver->config = cdcd_config;          /* config method */
    cdcd_driver->unconfig = cdcd_unconfig;      /* unconfig method */
    cdcd_driver->reference = cdcd_reference;    /* unconfig method */
    ts_snode_ref (&cdcd_driver->snode);         /* we hold a reference to it */
    
    ts_slist_clean (&cdcd_list);                /* initialize the list */
    errcode = usb_drv_register (cdcd_driver);   /* register driver */
    if (errcode)
    {
        gdlf_err ("Cannot register driver "
                  "(errno/usb = %d/%d)\n",
                  errcode, usb_errno);
        if (cdcd_driver->snode.next == NULL)
        {
            ts_usb_driver_clean (cdcd_driver);
            usb_kfree (cdcd_driver);
        }
        return errcode;
    }
        
        /* FIXME: do you class' stuf */
#warning FIXME: do class stuff!

    return 0;
}


    /* Release the cdcd driver
     *
     *  Do what is needed to ensure the use count of the driver is
     * zero and can be unloaded from the kernel.
     *
     * Returns:
     *
     *  int             0 if ok, errno code < 0 on error.
     */

int cdcd_release (void)
{
    gdlf_fntrace ("cdcd_release()");

#warning FIXME: cdcd_release() not implemented!

    return -ENOSYS;
}


    /* Shutdown the skel driver
     *
     * Returns:
     *
     *  int             0 if ok, errno code < 0 on error.
     */

int cdcd_shutdown (void)
{
    gdlf_fntrace ("cdcd_shutdown()");

        /* FIXME: do you class' stuf */

    ts_snode_unref (&cdcd_driver->snode, NULL);
    return usb_drv_unregister (cdcd_driver);
}


    /* FIXME: Detailed desc
     *
     *  The 'reference' method is used when we take any of the driver's
     * code or data structures to use it. If so, we call it with a
     * positive argument (one reference, regardless the value of the
     * _positive_ number). When we are done, we release the reference
     * calling it with a negative number. It always returns the number
     * of references or < 0 on error. Calling it with an argument of
     * zero will just return the number of references. 
     * Compliance:      USB Spec 1.0 xxxx
     *
     * Parameters:  
     *
     *  type            descr
     *
     * Global:
     *
     *  type name       descr
     *
     * Returns:
     *
     *  type            descr
     *
     * Global:
     *
     *  type name       descr
     */

int cdcd_reference (ts_usb_device* device, int n)
{
    gdlf_fntrace ("cdcd_reference (0x%p, %d)", device, n);
    gdlf_aassert (device != NULL, return -EINVAL);

#warning FIXME: add more stuff, keep the device ptr ...

    if (n > 0)
    {
        atomic_inc (&usecnt);
        MOD_INC_USE_COUNT;
    }
    else if (n < 0)
    {
        atomic_dec (&usecnt);
        MOD_DEC_USE_COUNT;
    }
    return atomic_read (&usecnt);
}


/* Log:
 *
 * $Log: driver.c,v $
 * Revision 1.3  1998/11/14 17:18:06  inaky
 * Linkage into the kernel made possible. Moved the include files to
 * uusbd/include/uusbd, to ease integration into the Linux kernel.
 *
 * Revision 1.2  1998/10/14 21:46:51  inaky
 * Removed errno and _class stuff.
 *
 * Revision 1.1  1998/10/13 22:58:38  inaky
 * First embrionical CDCD driver
 *
 */

