
/* USB Human Interface Devices Driver v$Revision: 1.5 $ 
 *
 * (C) 1997 by Iaky Prez Gonzlez <inaky@peloncho.fis.ucm.es>
 *
 * Distributed under the terms of the GNU Public License Version 2
 * See files COPYRIGHT and COPYING at the root distribution directory
 *
 * $Id: hkeep.c,v 1.5 1998/11/14 17:18:57 inaky Exp $
 *
 * Changelog at the bottom 
 *
 *     USB HID Boot Protocol
 *     ---------------------
 *
 *  This is an smaller driver then HID, but is capable of reading from
 * keyboards and mice which understand the boot protocol. This makes
 * it nice to have it compiled for boot disks and that stuff with size
 * constraints.
 */

#define __NO_VERSION__
#include <linux/module.h>
#include <asm/uaccess.h>
#include <asm/spinlock.h>   /* locks */
#include <linux/miscdevice.h>

#include <uusbd/usbd.h>

#define GDLF_IDENT          "USB-HIDBP"
#define GDLF_LOCAL          /* Define if want local settings */
#define GDLF_DEF_LOGLEVEL   9
#define GDLF_ASSERTIONS     /* want assertions (#undef if not) */
#define GDLF_FNTRACE        /* want function traces */
#define GDLF_LOCATIONS      /* want locations in log messages */
#include <uusbd/gdlf.h>

#include <uusbd/usb-hidbp.h>


    /* Unregister an hidbp element
     *
     * This is the destructor function.
     *
     * Parameters:  
     *
     *  ts_hidbp* Class element to unregister
     *
     * Returns:
     *
     *  int             0 if ok, errno code < 0 on error.
     */

int hidbp_unregister (ts_hidbp* hidbp)
{
    gdlf_fntrace ("hidbp_unregister (0x%p)", hidbp);
    gdlf_aassert (hidbp != NULL, return -EINVAL);
    gdlf_aassert (hidbp->snode.dtor.fn == (tf_dtor)hidbp_destroy,
                  return -EINVAL);

        /* Let's get it removed from the list */

    ts_slist_del (&hidbp_list, &hidbp->snode);
#warning FIXME: remove this, it makes modules stay for ever
    MOD_DEC_USE_COUNT;
    return 0;                   /* bye */
}


    /* FIXME: Detailed desc
     *
     * Compliance:      USB Spec 1.0 xxxx
     *
     * Parameters:  
     *
     *  type            descr
     *
     * Global:
     *
     *  type name       descr
     *
     * Returns:
     *
     *  type            descr
     *
     * Global:
     *
     *  type name       descr
     */

int hidbp_destroy (ts_hidbp* hidbp)
{
    gdlf_fntrace ("hidbp_destroy (0x%p)", hidbp);
    gdlf_aassert (hidbp != NULL, return -EINVAL);
    gdlf_assert (hidbp->open.dev != NULL);
        /* hidbp->file may be null, if no file is opened */

    ts_snode_ref (&hidbp->snode);
    if (hidbp->file && hidbp->file->private_data) /* tell file to go away */
    {
        gdlf_assert (hidbp->file->private_data == hidbp);
        hidbp->file->private_data = NULL;
        ts_snode_unref (&hidbp->snode, NULL);
        MOD_DEC_USE_COUNT;
    }
    if (hidbp->tr.data)
        usb_kfree (hidbp->tr.data);
    usb_close (&hidbp->open);
    hidbp_unregister (hidbp);  /* unregister */
        /* remove the reference for cfel.data */
    ts_snode_unref (&hidbp->snode, NULL);
    ts_snode_unref_nd (&hidbp->snode);
    ts_hidbp_release (hidbp);
    return 0;
}


    /* Register an hidbp element
     *
     * Parameters:  
     *
     *  ts_hidbp* Class element to register
     *
     * Returns:
     *
     *  int             0 if ok, errno code < 0 on error.
     */

int hidbp_register (ts_hidbp* hidbp)
{
    int errcode;
    
    gdlf_fntrace ("hidbp_register (0x%p)", hidbp);
    gdlf_aassert (hidbp != NULL, return -EINVAL);
    gdlf_aassert (hidbp->snode.dtor.fn == (tf_dtor)hidbp_destroy,
                  return -EINVAL);

        /* get into the list */

    errcode = ts_slist_prep (&hidbp_list, &hidbp->snode);
    if (errcode)
        return errcode;
#warning FIXME: remove this, it makes modules stay for ever
    MOD_INC_USE_COUNT;
    return 0;                   /* bye */
}


/* Log:
 *
 * $Log: hkeep.c,v $
 * Revision 1.5  1998/11/14 17:18:57  inaky
 * Moved the include files to uusbd/include/uusbd, to ease integration
 * into the Linux kernel.
 *
 * Revision 1.4  1998/10/31 02:16:43  inaky
 * Removed all the d_* pointers from ts_usb_cfel and collapsed them into
 * a single pointer; only one was really used.
 *
 * Revision 1.3  1998/10/22 00:07:22  inaky
 * Made it free the buffer space used by the tr to get data from the kbd
 * or mouse.
 *
 * Revision 1.2  1998/09/14 02:04:37  inaky
 * Removed global variables comment.
 *
 * Revision 1.1  1998/09/11 00:50:58  inaky
 * Moved the HID Boot Protocol to a single driver, so it can be isolated
 * more easily. It supports keyboards and mice, settable at compile time.
 *
 * Revision 1.1  1998/08/30 13:55:46  inaky
 * Added boot protocol support and a simple test program.
 *
 * Revision 1.1  1998/08/28 23:53:14  inaky
 * New driver
 *
 * Revision 1.3  1998/08/28 00:14:22  inaky
 * New driver ...
 *
 */



