
/* USB Human Interface Devices Driver Boot Protocol v$Revision: 1.6 $ 
 *
 * (C) 1997 by Iaky Prez Gonzlez <inaky@peloncho.fis.ucm.es>
 *
 * Distributed under the terms of the GNU Public License Version 2
 * See files COPYRIGHT and COPYING at the root distribution directory
 *
 * $Id: keyboard.c,v 1.6 1998/11/14 17:18:58 inaky Exp $
 *
 * Changelog at the bottom 
 *
 *     USB HID Boot Protocol
 *     ---------------------
 *
 *  This is an smaller driver then HID, but is capable of reading from
 * keyboards and mice which understand the boot protocol. This makes
 * it nice to have it compiled for boot disks and that stuff with size
 * constraints.
 */

#define __NO_VERSION__
#include <linux/module.h>
#include <asm/uaccess.h>
#include <asm/spinlock.h>   /* locks */
#include <linux/miscdevice.h>

#include <uusbd/usbd.h>

#define GDLF_IDENT          "USB-HIDBP"
#define GDLF_LOCAL          /* Define if want local settings */
#define GDLF_DEF_LOGLEVEL   9
#define GDLF_ASSERTIONS     /* want assertions (#undef if not) */
#define GDLF_FNTRACE        /* want function traces */
#define GDLF_LOCATIONS      /* want locations in log messages */
#include <uusbd/gdlf.h>

#include <uusbd/usb-hidbp.h>


struct file_operations bkbd_fops = 
{
    NULL,           /* llseek */
    bkbd_read,      /* read */
    NULL,           /* write */
    NULL,           /* readdir */
    NULL,           /* select */
    bkbd_ioctl,     /* ioctl */
    NULL,           /* mmap */
    hidbp_open,     /* open */
    NULL,           /* flush */
    hidbp_close,    /* release */
    NULL,           /* fsync */
    NULL,           /* fasync */
    NULL,           /* check_media_change */
    NULL            /* revalidate */
};


    /* FIXME: Detailed desc
     *
     * Compliance:      USB Spec 1.0 xxxx
     *
     * Parameters:  
     *
     *  type            descr
     *
     * Global:
     *
     *  type name       descr
     *
     * Returns:
     *
     *  type            descr
     *
     * Global:
     *
     *  type name       descr
     */

ssize_t bkbd_read (struct file* file, char* data, size_t size, 
                   loff_t* offset)
{
    ts_hidbp *bkbd;
    ssize_t errcode;
    
    gdlf_fntrace ("bkbd_read (0x%p, 0x%p, %u, 0x%p)", 
                  file, data, size, offset);
    gdlf_aassert (file != NULL, return -EINVAL);
    gdlf_aassert (data != NULL, return -EINVAL);
    gdlf_aassert (file != NULL, return -EINVAL);

    if (size < sizeof (ts_bkbd_report))
        return -EINVAL;
    
    errcode = verify_area (VERIFY_WRITE, data, size);
    if (errcode)
        return errcode;
    
    bkbd = file->private_data;          /* get the private data */
    if (bkbd == NULL)                   /* device removed? */
        return -EBADFD;
    gdlf_aassert (bkbd->snode.dtor.fn == (tf_dtor)hidbp_destroy, 
                  return -EBADFD);

    errcode = usb_tr_run (&bkbd->tr);
    if (errcode >= 0)
        errcode = bkbd->tr.errno < 0? bkbd->tr.errno : bkbd->tr.xfered;
    if (errcode >= 0)
    {
        copy_to_user (data, bkbd->tr.data, errcode);
        clear_user (data + errcode, size - errcode);
    }
    else
    {
        if (errcode == -EREMOTEIO)
            usb_endp_stall_clear (bkbd->tr.open);
        clear_user (data, size);
    }
    usb_tr_reset (&bkbd->tr);
    return errcode;
}


    /* FIXME: Detailed desc
     *
     * Compliance:      USB Spec 1.0 xxxx
     *
     * Parameters:  
     *
     *  type            descr
     *
     * Global:
     *
     *  type name       descr
     *
     * Returns:
     *
     *  type            descr
     *
     * Global:
     *
     *  type name       descr
     */

int bkbd_ioctl (struct inode* inode, struct file* file, 
                unsigned int cmd, unsigned long arg)
{
    ts_hidbp* bkbd;
    
    gdlf_fntrace ("bkbd_ioctl (0x%p, 0x%p, %u, %lu)",
                  inode, file, cmd, arg);
    gdlf_aassert (inode != NULL, return -EINVAL);
    gdlf_aassert (file != NULL, return -EINVAL);

    bkbd = file->private_data;          /* get the private data */
    if (bkbd == NULL)                   /* device removed? */
        return -EBADFD;

#warning FIXME: ioctls?
    return 0;
}


/* Log:
 *
 * $Log: keyboard.c,v $
 * Revision 1.6  1998/11/14 17:18:58  inaky
 * Moved the include files to uusbd/include/uusbd, to ease integration
 * into the Linux kernel.
 *
 * Revision 1.5  1998/10/14 21:47:16  inaky
 * Fixed _blatant_ mistake!
 *
 * Revision 1.4  1998/09/24 16:06:15  inaky
 * Fixed all the writing code, just to be sure and make it cleaner.
 *
 * Revision 1.3  1998/09/24 16:00:50  inaky
 * Added area verification before write and clearing stalled endpoints on
 * error.
 *
 * Revision 1.2  1998/09/14 02:05:11  inaky
 * Removed unneeded specific routines. Made the tr be inserted at open
 * time and only ran at read time.
 *
 * Revision 1.1  1998/09/11 00:50:58  inaky
 * Moved the HID Boot Protocol to a single driver, so it can be isolated
 * more easily. It supports keyboards and mice, settable at compile time.
 *
 * Revision 1.1  1998/08/30 13:55:46  inaky
 * Added boot protocol support and a simple test program.
 *
 * Revision 1.1  1998/08/28 23:53:14  inaky
 * New driver
 *
 * Revision 1.3  1998/08/28 00:14:22  inaky
 * New driver ...
 *
 */



