
/* USB Human Interface Devices Driver v$Revision: 1.6 $ 
 *
 * (C) 1997 by Iaky Prez Gonzlez <inaky@peloncho.fis.ucm.es>
 *
 * Distributed under the terms of the GNU Public License Version 2
 * See files COPYRIGHT and COPYING at the root distribution directory
 *
 * $Id: mouse.c,v 1.6 1998/11/17 09:51:30 inaky Exp $
 *
 * Changelog at the bottom 
 *
 *     USB HID Boot Protocol
 *     ---------------------
 *
 *  This is an smaller driver then HID, but is capable of reading from
 * keyboards and mice which understand the boot protocol. This makes
 * it nice to have it compiled for boot disks and that stuff with size
 * constraints.
 */

#define __NO_VERSION__
#include <linux/module.h>
#include <asm/uaccess.h>
#include <asm/spinlock.h>   /* locks */
#include <linux/miscdevice.h>

#include <uusbd/usbd.h>

#define GDLF_IDENT          "USB-HIDBP"
#define GDLF_LOCAL          /* Define if want local settings */
#define GDLF_DEF_LOGLEVEL   9
#define GDLF_ASSERTIONS     /* want assertions (#undef if not) */
#undef GDLF_FNTRACE        /* want function traces */
#define GDLF_LOCATIONS      /* want locations in log messages */
#include <uusbd/gdlf.h>

#include <uusbd/usb-hidbp.h>

struct file_operations bmse_fops = 
{
    NULL,           /* llseek */
    bmse_read,      /* read */
    NULL,           /* write */
    NULL,           /* readdir */
    NULL,           /* select */
    bmse_ioctl,     /* ioctl */
    NULL,           /* mmap */
    hidbp_open,     /* open */
    NULL,           /* flush */
    hidbp_close,    /* release */
    NULL,           /* fsync */
    NULL,           /* fasync */
    NULL,           /* check_media_change */
    NULL            /* revalidate */
};


    /* FIXME: Detailed desc
     *
     * Compliance:      USB Spec 1.0 xxxx
     *
     * Parameters:  
     *
     *  type            descr
     *
     * Global:
     *
     *  type name       descr
     *
     * Returns:
     *
     *  type            descr
     *
     * Global:
     *
     *  type name       descr
     */

ssize_t bmse_read (struct file* file, char* data, size_t size, 
                   loff_t* offset)
{
    ts_hidbp *bmse;
    ssize_t errcode;
    
    gdlf_fntrace ("bmse_read (0x%p, 0x%p, %u, 0x%p)", 
                  file, data, size, offset);
    gdlf_aassert (file != NULL, return -EINVAL);
    gdlf_aassert (data != NULL, return -EINVAL);
    gdlf_aassert (file != NULL, return -EINVAL);

    if (size < sizeof (ts_bmse_report))
        return -EINVAL;

    errcode = verify_area (VERIFY_WRITE, data, size);
    if (errcode)
        return errcode;
    
    bmse = file->private_data;          /* get the private data */
    if (bmse == NULL)                   /* device removed? */
        return -EBADFD;
    gdlf_aassert (bmse->snode.dtor.fn == (tf_dtor)hidbp_destroy, 
                  return -EBADFD);
    
    errcode = usb_tr_run (&bmse->tr);
    if (errcode >= 0)
        errcode = bmse->tr.errno < 0? bmse->tr.errno : bmse->tr.xfered;
    if (errcode >= 0)
    {
        __u8 buttons;
        buttons = ((__u8*)bmse->tr.data)[0];
        buttons = ((buttons & 1)? 4 : 0)  /* left? */
            | ((buttons & 2)? 1 : 0)   /* right? */
            | ((buttons & 4)? 2 : 0);  /* middle? */
        ((__u8*)bmse->tr.data)[0] = (~buttons & 0x07) | 0x80;
        ((__u8*)bmse->tr.data)[2] = - (int)(((__u8*)bmse->tr.data)[2]);
        copy_to_user (data, bmse->tr.data, errcode);
        clear_user (data + errcode, size - errcode);
    }
    else
    {
        if (errcode == -EREMOTEIO)
            usb_endp_stall_clear (bmse->tr.open);
        clear_user (data, size);
    }
    usb_tr_reset (&bmse->tr);
    return errcode;
}


    /* FIXME: Detailed desc
     *
     * Compliance:      USB Spec 1.0 xxxx
     *
     * Parameters:  
     *
     *  type            descr
     *
     * Global:
     *
     *  type name       descr
     *
     * Returns:
     *
     *  type            descr
     *
     * Global:
     *
     *  type name       descr
     */

int bmse_ioctl (struct inode* inode, struct file* file, 
                unsigned int cmd, unsigned long arg)
{
    ts_hidbp* bmse;
    
    gdlf_fntrace ("bmse_ioctl (0x%p, 0x%p, %u, %lu)",
                  inode, file, cmd, arg);
    gdlf_aassert (inode != NULL, return -EINVAL);
    gdlf_aassert (file != NULL, return -EINVAL);

    bmse = file->private_data;          /* get the private data */
    if (bmse == NULL)                   /* device removed? */
        return -EBADFD;

#warning FIXME: bmse ioctls?
    return 0;
}


/* Log:
 *
 * $Log: mouse.c,v $
 * Revision 1.6  1998/11/17 09:51:30  inaky
 * Fixed the protocol to imitate bus mouse and added more options to the
 * test utilities to be able not to print anything not wait [to do load
 * tests].
 *
 * Revision 1.5  1998/11/14 17:18:59  inaky
 * Moved the include files to uusbd/include/uusbd, to ease integration
 * into the Linux kernel.
 *
 * Revision 1.4  1998/10/14 21:47:34  inaky
 * Fixed _blatant_ mistake.
 *
 * Revision 1.3  1998/09/24 16:05:58  inaky
 * Fixed error handling on read, added verifications and checks.
 *
 * Revision 1.2  1998/09/14 02:05:18  inaky
 * Removed unneeded specific routines. Made the tr be inserted at open
 * time and only ran at read time.
 *
 * Revision 1.1  1998/09/11 00:50:59  inaky
 * Moved the HID Boot Protocol to a single driver, so it can be isolated
 * more easily. It supports keyboards and mice, settable at compile time.
 *
 * Revision 1.1  1998/08/30 13:55:46  inaky
 * Added boot protocol support and a simple test program.
 *
 * Revision 1.1  1998/08/28 23:53:14  inaky
 * New driver
 *
 * Revision 1.3  1998/08/28 00:14:22  inaky
 * New driver ...
 *
 */



