
/* USB Human Interface Devices Driver v$Revision: 1.4 $ 
 *
 * (C) 1997 by Iaky Prez Gonzlez <inaky@peloncho.fis.ucm.es>
 *
 * Distributed under the terms of the GNU Public License Version 2
 * See files COPYRIGHT and COPYING at the root distribution directory
 *
 * $Id: driver.c,v 1.4 1998/11/14 17:19:47 inaky Exp $
 *
 * Changelog at the bottom 
 *
 *     USB Human Interface Devices - Configuration
 *     -------------------------------------------
 *
 *  This is the embrion of the USB HID Driver.
 *
 */

#include <linux/init.h>     /* __init */
#define __NO_VERSION__
#include <linux/module.h>   /* MOD_*_USE_COUNT */
#include <asm/spinlock.h>   /* locks */

#include <uusbd/usbd.h>

#define GDLF_IDENT          "USB-HIDD"
#define GDLF_LOCAL          /* Define if want local settings */
#define GDLF_DEF_LOGLEVEL   9
#define GDLF_ASSERTIONS     /* want assertions (#undef if not) */
#define GDLF_FNTRACE        /* want function traces */
#define GDLF_LOCATIONS      /* want locations in log messages */
#include <uusbd/gdlf.h>

#include <uusbd/usb-hidbp.h>


atomic_t usecnt;
volatile int hidd_errno = 0;

#define NAME   /* short!! */                    \
    "USB Human Interface Devices Driver v$Revision: 1.4 $"

#define COPYRIGHT                               \
    "(c) 1998 Inaky Perez Gonzalez "            \
    "<inaky@peloncho.fis.ucm.es>"

#define VENDOR 0
#define VENDORNAME                              \
    NULL

#define PRODUCT 0
#define DEVICENAME                              \
    NULL

#define CLASS HIDD_CLASS
#define CLASSNAME                               \
    "USB Human Inteface Devices"

#define SUBCLASS 0
#define SUBCLASSNAME                            \
    NULL

#define PROTOCOL 0
#define PROTOCOLNAME                            \
    NULL

#define SNODE_NULL { {NULL}, NULL, 0}

ts_usb_driver* hidd_driver = NULL;

    /* 
     * Linux module support
     */

#ifdef MODULE

int init_module (void)
{
    return hidd_init();
}

void cleanup_module (void)
{
    hidd_release();
    hidd_shutdown();
    return;
}

#endif


    /* Initialize the hidd driver
     *
     * Returns:
     *
     *  int             0 if ok, errno code < 0 on error.
     *
     * Global:
     *
     *  int hidd_errno On error, error code.  
     *  int usb_errno   On error, error code.  
     */

int __init hidd_init (void)
{
    int errcode;
    
    gdlf_fntrace ("hidd_init()");
    gdlf_info (NAME "\n");
#ifdef MODULE
    gdlf_info (COPYRIGHT "\n");
#endif

    hidd_driver = ts_usb_driver_init();
    if (hidd_driver == NULL)
    {
        usb_errno = hidd_errno = -USBE_NO_MEMORY;
        return -ENOMEM;
    }

        /* Set driver's attributes */

    hidd_driver->vendor = VENDOR;
    hidd_driver->product = PRODUCT;
    hidd_driver->class = CLASS;
    hidd_driver->subclass = SUBCLASS;
    hidd_driver->protocol = PROTOCOL;

        /* Set info */

    hidd_driver->name = NAME;                   /* driver name */
    hidd_driver->copyright = COPYRIGHT;         /* driver copyright */
    hidd_driver->vendorname = VENDORNAME;       /* device's vendor name */
    hidd_driver->productname = DEVICENAME;      /* device's name */
    hidd_driver->classname = CLASSNAME;         /* class' name */
    hidd_driver->subclassname = SUBCLASSNAME;   /* subclass' name */
    hidd_driver->protocolname = PROTOCOLNAME;   /* protocol name */

    hidd_driver->config = hidd_config;          /* config method */
    hidd_driver->unconfig = hidd_unconfig;      /* unconfig method */
    hidd_driver->reference = hidd_reference;    /* reference method */
    ts_snode_ref (&hidd_driver->snode);         /* we hold a reference to it */
    
    ts_slist_clean (&hidd_list);                /* initialize the list */
    errcode = usb_drv_register (hidd_driver);   /* register driver */
    if (errcode)
    {
        gdlf_err ("Cannot register driver: "
                  "errno/usb %d/%d\n",
                  errcode, usb_errno);
        usb_drv_destroy (hidd_driver);
        usb_kfree (hidd_driver);
        hidd_errno = -USBE_DRIVER_CANT_REG;
        return errcode;
    }
        
        /* FIXME: do you class' stuf */
#warning FIXME: do class stuff!

    return 0;
}


    /* Release the hidd driver
     *
     *  Do what is needed to ensure the use count of the driver is
     * zero and can be unloaded from the kernel.
     *
     * Returns:
     *
     *  int             0 if ok, errno code < 0 on error.
     *
     * Global:
     *
     *  int hidd_errno On error, error code.  
     *  int usb_errno   On error, error code.  
     */

int hidd_release (void)
{
    gdlf_fntrace ("hidd_release()");

#warning FIXME: hidd_release() not implemented!

    return -ENOSYS;
}


    /* Shutdown the skel driver
     *
     * Returns:
     *
     *  int             0 if ok, errno code < 0 on error.
     *
     * Global:
     *
     *  int hidd_errno On error, error code.  
     *  int usb_errno   On error, error code.  
     */

int hidd_shutdown (void)
{
    gdlf_fntrace ("hidd_shutdown()");
    ts_snode_unref (&hidd_driver->snode, NULL);
    return usb_drv_unregister (hidd_driver);
}


    /* Driver reference
     *
     *  The 'reference' method is used when we take any of the driver's
     * code or data structures to use it. If so, we call it with a
     * positive argument (one reference, regardless the value of the
     * _positive_ number). When we are done, we release the reference
     * calling it with a negative number. It always returns the number
     * of references or < 0 on error. Calling it with an argument of
     * zero will just return the number of references. 
     * Compliance:      USB Spec 1.0 xxxx
     *
     * Parameters:  
     *
     *  type            descr
     *
     * Global:
     *
     *  type name       descr
     *
     * Returns:
     *
     *  type            descr
     *
     * Global:
     *
     *  type name       descr
     */

int hidd_reference (ts_usb_device* device, int n)
{
    gdlf_fntrace ("hidd_reference (0x%p, %d)", device, n);
    gdlf_aassert (device != NULL, return -EINVAL);

    if (n > 0)
    {
        atomic_inc (&usecnt);
        MOD_INC_USE_COUNT;
    }
    else if (n < 0)
    {
        atomic_dec (&usecnt);
        MOD_DEC_USE_COUNT;
    }
    return atomic_read (&usecnt);
}

/* Log:
 *
 * $Log: driver.c,v $
 * Revision 1.4  1998/11/14 17:19:47  inaky
 * Moved the include files to uusbd/include/uusbd, to ease integration
 * into the Linux kernel.
 *
 * Revision 1.3  1998/09/11 00:51:51  inaky
 * Moved the boot protocol to a single driver, to isolate it more easily.
 *
 * Revision 1.2  1998/08/30 13:56:54  inaky
 * Fixed destroying driver stryctures on error, added char device
 * registration.
 *
 * Revision 1.1  1998/08/28 23:53:14  inaky
 * New driver
 *
 * Revision 1.3  1998/08/28 00:14:23  inaky
 * New driver ...
 *
 */

