
/* USB Human Interface Devices Driver v$Revision: 1.4 $ 
 *
 * (C) 1997 by Iaky Prez Gonzlez <inaky@peloncho.fis.ucm.es>
 *
 * Distributed under the terms of the GNU Public License Version 2
 * See files COPYRIGHT and COPYING at the root distribution directory
 *
 * $Id: hkeep.c,v 1.4 1998/11/14 17:19:50 inaky Exp $
 *
 * Changelog at the bottom 
 *
 *     USB Human Interface Devices - Configuration
 *     -------------------------------------------
 *
 *  This is the embrion of the USB HID Driver.
 *
 */

#include <asm/spinlock.h>   /* locks */
#include <linux/module.h>   /* MOD_*_USE_COUNT */

#include <uusbd/usbd.h>

#define GDLF_IDENT          "USB-HIDD"
#define GDLF_LOCAL          /* Define if want local settings */
#define GDLF_DEF_LOGLEVEL   9
#define GDLF_ASSERTIONS     /* want assertions (#undef if not) */
#define GDLF_FNTRACE        /* want function traces */
#define GDLF_LOCATIONS      /* want locations in log messages */
#include <uusbd/gdlf.h>

#include <uusbd/usb-hidbp.h>


ts_slist hidd_list;  /* list of registered class structures */


    /* Unregister an hidd element
     *
     * This is the destructor function.
     *
     * Parameters:  
     *
     *  ts_hidd* Class element to unregister
     *
     * Returns:
     *
     *  int             0 if ok, errno code < 0 on error.
     *
     * Global:
     *
     *  int usb_errno   On error, error code.  
     *  int hidd_errno On error, error code.  
     */

int hidd_unregister (ts_hidd* hidd)
{
    gdlf_fntrace ("hidd_unregister (0x%p)", hidd);
    gdlf_aassert (hidd != NULL, return -EINVAL);
    gdlf_aassert (hidd->snode.dtor.fn == (tf_dtor)hidd_destroy,
                  return -EINVAL);

        /* Let's get it removed from the list */

    ts_slist_del (&hidd_list, &hidd->snode);
#warning FIXME: remove this, it makes modules stay for ever
    MOD_DEC_USE_COUNT;
    return 0;                   /* bye */
}


    /* FIXME: Detailed desc
     *
     * Compliance:      USB Spec 1.0 xxxx
     *
     * Parameters:  
     *
     *  type            descr
     *
     * Global:
     *
     *  type name       descr
     *
     * Returns:
     *
     *  type            descr
     *
     * Global:
     *
     *  type name       descr
     */

int hidd_destroy (ts_hidd* hidd)
{
    gdlf_fntrace ("hidd_destroy (0x%p)", hidd);
    gdlf_aassert (hidd != NULL, return -EINVAL);
    gdlf_aassert (hidd->snode.dtor.fn == (tf_dtor)hidd_destroy,
                  return -EINVAL);
    
    ts_snode_ref (&hidd->snode);

#warning DEBUG: finish me!!
    hidd_unregister (hidd);     /* unregister */
    ts_snode_unref_nd (&hidd->snode);
    ts_hidd_release (hidd);     /* clean the struct */
    return 0;                   /* bye */
}


    /* Register an hidd element
     *
     * Parameters:  
     *
     *  ts_hidd* Class element to register
     *
     * Returns:
     *
     *  int             0 if ok, errno code < 0 on error.
     *
     * Global:
     *
     *  int usb_errno   On error, error code.  
     *  int hidd_errno On error, error code.  
     */

int hidd_register (ts_hidd* hidd)
{
    int errcode;
    
    gdlf_fntrace ("hidd_register (0x%p)", hidd);
    gdlf_aassert (hidd != NULL, return -EINVAL);
    gdlf_aassert (hidd->snode.dtor.fn == (tf_dtor)hidd_destroy,
                  return -EINVAL);

        /* get into the list */

    errcode = ts_slist_prep (&hidd_list, &hidd->snode);
    if (errcode)
        return errcode;
#warning FIXME: remove this, it makes modules stay for ever
    MOD_INC_USE_COUNT;
    return 0;                   /* bye */
}

/* Log:
 *
 * $Log: hkeep.c,v $
 * Revision 1.4  1998/11/14 17:19:50  inaky
 * Moved the include files to uusbd/include/uusbd, to ease integration
 * into the Linux kernel.
 *
 * Revision 1.3  1998/09/11 00:51:52  inaky
 * Moved the boot protocol to a single driver, to isolate it more easily.
 *
 * Revision 1.2  1998/08/30 13:58:02  inaky
 * Added checks and fixed the destructor.
 *
 * Revision 1.1  1998/08/28 23:53:15  inaky
 * New driver
 *
 * Revision 1.3  1998/08/28 00:14:23  inaky
 * New driver ...
 *
 */


