
/* USB Driver Stack v$Revision: 1.19 $ 
 *
 * (C) 1997 by Iaky Prez Gonzlez <inaky@peloncho.fis.ucm.es>
 *
 * Distributed under the terms of the GNU Public License Version 2
 * See files COPYRIGHT and COPYING at the root distribution directory
 *
 * $Id: hkeep.c,v 1.19 1998/11/14 17:20:33 inaky Exp $
 *
 * Changelog at the bottom 
 *
 *     Hub driver - Class structures housekeeping
 *     ------------------------------------------
 */

#define __NO_VERSION__      /* We just want MOD_*_USE_COUNT() */
#include <linux/module.h>   /* MOD_*_USE_COUNT() */
#include <asm/spinlock.h>   /* locks */

#include <uusbd/usbd.h>
#include <uusbd/usb-hubd.h>

#define GDLF_IDENT          "USB-HUBD"
#define GDLF_LOCAL          /* Define if want local settings */
#define GDLF_DEF_LOGLEVEL   9
#define GDLF_ASSERTIONS     /* want assertions (#undef if not) */
#define GDLF_FNTRACE        /* want function traces */
#define GDLF_LOCATIONS      /* want locations in log messages */
#include <uusbd/gdlf.h>


    /* Release a devices's usage of the hub driver
     *
     * Parameters:  
     *
     *  ts_usb_hub*     Pointer to the hub struct to free.
     *
     * Returns:
     *
     *  int             0 if ok, < 0 errno code on error.
     *
     * Global:
     *
     *  int usb_errno       on error, set to -USBE_*, else unmodified.
     *
     *  int usb_hub_errno   on error, set to -USBE_*, else unmodified.
     */

int hubd_free (ts_usb_hub* hub)
{
    gdlf_fntrace ("hubd_free (0x%p)", hub);
    gdlf_aassert (hub != NULL, return -EINVAL);
    hubd_reference (hub, -1);
    return 0;
}


/* Log:
 *
 * $Log: hkeep.c,v $
 * Revision 1.19  1998/11/14 17:20:33  inaky
 * Moved the include files to uusbd/include/uusbd, to ease integration
 * into the Linux kernel.
 *
 * Revision 1.18  1998/09/14 20:29:05  inaky
 * Severed assertions.
 *
 * Revision 1.17  1998/06/21 20:16:23  inaky
 * Added hubd_free(), to release an usage of the hub driver.
 *
 * Revision 1.16  1998/06/20 21:55:17  inaky
 * Massive changes to split the old monolithic hubd driver into a hub
 * manager (hub wrappers and daemon) and a dynamic driver, which can be
 * replaced and its different parts reused.
 *
 * Revision 1.15  1998/05/01 19:17:13  inaky
 * Fixed device address reporting in all messages. Made it power off all
 * ports on hub destruction. First shy intents to make the root hub
 * detection and setup code.
 *
 * Revision 1.14  1998/04/26 03:40:24  inaky
 * Fixed to use the new configurations array.
 *
 * Revision 1.13  1998/04/13 22:18:39  inaky
 * Finished most of the hub driver; some updates still needed.
 *
 * Revision 1.12  1998/02/12 14:38:26  inaky
 * Fixed to set both usb_errno and hubd_errno if no root hub is found.
 *
 * Revision 1.11  1998/02/10 17:44:36  inaky
 * Moved to use ts_[N]snode_[un]ref().
 *
 * Revision 1.10  1998/02/09 13:44:23  inaky
 * Moved all use counts to atomic
 *
 * Revision 1.9  1998/01/11 15:44:36  inaky
 * Added docs?
 *
 * Revision 1.8  1998/01/08 12:19:18  inaky
 * Added copyright strings to all important files
 *
 * Revision 1.7  1998/01/02 00:07:29  inaky
 * Moved to the new driver registration scheme
 *
 * Revision 1.6  1998/01/01 16:55:26  inaky
 * Added a *FIXED* check against hosts without root hub
 *
 * Revision 1.5  1997/12/31 17:06:38  inaky
 * Removed some unneeded locks and fixed some others.
 *
 * Revision 1.4  1997/12/31 05:37:41  inaky
 * Fixed some craps.
 *
 * Revision 1.3  1997/12/31 04:15:33  inaky
 * Added hubd_root_unset() and specified some of hubd_root_search()
 * (yeah, also changed it's name).
 *
 * Revision 1.2  1997/12/30 22:30:29  inaky
 * Fixed some bugs.
 *
 * Revision 1.1  1997/12/30 20:17:52  inaky
 * House keeping stuff for HUBD.
 *
 */

