/* 
 * Generic Debugging and Logging Framework
 *
 * (c) Nov 1997 Iaky Prez Gonzlez <inaky@peloncho.fis.ucm.es>
 *
 * Distributed under the terms of the GNU Public License Version 2
 * See files COPYRIGHT and COPYING at the root distribution directory
 *
 * $Id: gdlf.h,v 1.1 1998/11/14 00:38:26 inaky Exp $
 */

#ifndef __gdlf_h_   /* We want multiple inclusions to change */
#define __gdlf_h_   /* behaviours in different parts of the */
#endif              /* files! */

#include <linux/kernel.h>

#undef gdlf_level_emerg
#undef gdlf_level_emerg_s
#undef gdlf_level_alert
#undef gdlf_level_alert_s
#undef gdlf_level_crit
#undef gdlf_level_crit_s
#undef gdlf_level_err
#undef gdlf_level_err_s
#undef gdlf_level_warning
#undef gdlf_level_warning_s
#undef gdlf_level_notice
#undef gdlf_level_notice_s
#undef gdlf_level_info
#undef gdlf_level_info_s
#undef gdlf_level_debug
#undef gdlf_level_debug_s
#undef gdlf_level_top
#undef gdlf_level_top_s

#undef gdlf__stringfy
#undef gdlf_stringfy
#undef gdlf_location
#undef gdlf_locations
#undef gdlf_locations
#undef gdlf_loglvl
#undef gdlf_log
#undef gdlf_internal_error
#undef gdlf_emerg
#undef gdlf_alert
#undef gdlf_crit
#undef gdlf_err
#undef gdlf_warning
#undef gdlf_notice
#undef gdlf_info
#undef gdlf_debug
#undef gdlf_assert
#undef gdlf_aassert
#undef gdlf_passert
#undef gdlf_apassert
#undef gdlf_fntrace


    /*
     * Set the defines GDLF_MAINTAINER, GDLF_IDENT at the gdlf.h or
     * before #including it. You can also set it at the gdlf.h header
     * file if they're going to be constant for all the submodules.
     *
     *  If you don't #define GDLF_LOCAL, the gdlf will assume you
     * don't want local settings for the marked GDLF_ variables and
     * will assign the defaults [maximum verbosity and paranoia].
     *
     *  The GDLF_DEG_LOGLEVEL controls which is the default log level
     * at compilation time, and thus, anything verboser than that
     * won't be compiled in. 
     *
     *  The GDLF_ASSERTIONS forces the assertions (gdlf_*assert()) to
     * be compiled in.
     *
     *  The GDLF_FNTRACE forces the function traces (gdlf_fntrace())
     * to be compiled in.
     *
     *  The GDLF_LOCATIONS forces the locations of the messages to be
     * compiled in, and thus showed on the messages.
     *
     *  This means, to control all that, insert one of the blocks like
     * the #ifndef GDLF_LOCAL ... before including gdlf.h and
     * configure there.
     */


    /* Log level type */

    /* Log level type */


#define gdlf_level_emerg        0
#define gdlf_level_emerg_s      KERN_EMERG  /* system is unusable */
#define gdlf_level_alert        1
#define gdlf_level_alert_s      KERN_ALERT  /* action must be taken immediately */
#define gdlf_level_crit         2
#define gdlf_level_crit_s       KERN_CRIT   /* critical conditions */
#define gdlf_level_err          3
#define gdlf_level_err_s        KERN_ERR    /* error conditions */
#define gdlf_level_warning      4
#define gdlf_level_warning_s    KERN_WARNING/* warning conditions */
#define gdlf_level_notice       5
#define gdlf_level_notice_s     KERN_NOTICE /* normal, but significant, condition */
#define gdlf_level_info         6
#define gdlf_level_info_s       KERN_INFO   /* informational message */
#define gdlf_level_debug        7
#define gdlf_level_debug_s      KERN_DEBUG  /* debug-level message */
#define gdlf_level_top          9
#define gdlf_level_top_s        KERN_DEBUG  /* top */


    /* Compilation time settings */

#ifndef GDLF_MAINTAINER
#define GDLF_MAINTAINER     "Inaky Perez Gonzalez <inaky@peloncho.fis.ucm.es>"
#endif

#ifndef GDLF_LOCAL              /* Default settings if undefined */

#define GDLF_IDENT          "USB"   /* One short word to identify the procedence of messages */
#define GDLF_LOCAL                  /* Define if want local settings */
#define GDLF_DEF_LOGLEVEL   9
#define GDLF_ASSERTIONS             /* want assertions (#undef if not) */
#define GDLF_FNTRACE                /* want function traces */
#define GDLF_LOCATIONS              /* want locations in log messages */

    /* 1 to override global settings */

#if 1
#define GDLF_LOCAL
#undef GDLF_FNTRACE
#define GDLF_LOCAL
#define GDLF_ASSERTIONS
#define GDLF_LOCATIONS
#endif

#endif  /* GDLF_LOCAL */


    /* Public use functions and macros*/

#define gdlf__stringfy(a) #a
#define gdlf_stringfy(a) gdlf__stringfy(a)

#define gdlf_location   __FILE__ ":" gdlf_stringfy (__LINE__) ": "
#ifdef GDLF_LOCATIONS
#define gdlf_locations  __FILE__ ":" gdlf_stringfy (__LINE__) ": "
#else
#define gdlf_locations ""
#endif

#define gdlf_loglvl(level, format...) \
    printk (level GDLF_IDENT ": " format)

#define gdlf_log(format...) \
    printk (format);

#define gdlf_internal_error()                           \
    gdlf_loglvl (gdlf_level_emerg_s,                    \
                 gdlf_location "Internal error"         \
                 "; please report to " GDLF_MAINTAINER "\n")
    

#if GDLF_DEF_LOGLEVEL >= gdlf_level_emerg
#define gdlf_emerg(format...) \
    gdlf_loglvl (gdlf_level_emerg_s, gdlf_locations format)
#else
#define gdlf_emerg(format...) do { } while (0)
#endif


#if GDLF_DEF_LOGLEVEL >= gdlf_level_alert
#define gdlf_alert(format...) \
    gdlf_loglvl (gdlf_level_alert_s, gdlf_locations format)
#else
#define gdlf_alert(format...) do { } while (0)
#endif


#if GDLF_DEF_LOGLEVEL >= gdlf_level_crit
#define gdlf_crit(format...) \
    gdlf_loglvl (gdlf_level_crit_s, gdlf_locations format)
#else
#define gdlf_crit(format...) do { } while (0)
#endif


#if GDLF_DEF_LOGLEVEL >= gdlf_level_err
#define gdlf_err(format...) \
    gdlf_loglvl (gdlf_level_err_s, gdlf_locations format)
#else
#define gdlf_err(format...) do { } while (0)
#endif


#if GDLF_DEF_LOGLEVEL >= gdlf_level_warning
#define gdlf_warning(format...) \
    gdlf_loglvl (gdlf_level_warning_s, gdlf_locations format)
#else
#define gdlf_warning(format...) do { } while (0)
#endif


#if GDLF_DEF_LOGLEVEL >= gdlf_level_notice
#define gdlf_notice(format...) \
    gdlf_loglvl (gdlf_level_notice_s, gdlf_locations format)
#else
#define gdlf_notice(format...) do { } while (0)
#endif


#if GDLF_DEF_LOGLEVEL >= gdlf_level_info
#define gdlf_info(format...) \
    gdlf_loglvl (gdlf_level_info_s, gdlf_locations format)
#else
#define gdlf_info(format...) do { } while (0)
#endif

#if GDLF_DEF_LOGLEVEL >= gdlf_level_debug
#define gdlf_debug(format...) \
    gdlf_loglvl (gdlf_level_debug_s, gdlf_locations format)
#else
#define gdlf_debug(format...) do { } while (0)
#endif


#ifdef GDLF_ASSERTIONS
#define gdlf_assert_dec(x) x
#define gdlf_assert_act(x)                      \
do {                                            \
    x;                                          \
} while (0)

#define gdlf_c1assert(x) (x)
#define gdlf_c0assert(x) (x)

#define gdlf_assert(x)                                      \
do {                                                        \
    if (!(x))                                               \
        gdlf_loglvl (gdlf_level_emerg_s,                    \
                     gdlf_location "Assertion failed: "     \
                     gdlf_stringfy (x) "\n");               \
} while (0)

        /* 
         * Use IF directly, because if 'a' is 'break', for example, it
         * won't work as expected.
         */ 

#define gdlf_aassert(x,a)                           \
if (!(x))                                           \
{                                                   \
    gdlf_loglvl (gdlf_level_emerg_s,                \
                 gdlf_location "Assertion failed: " \
                 gdlf_stringfy (x) "\n");           \
    a;                                              \
}


#define gdlf_passert(x, fmt...)                             \
do {                                                        \
    if (!(x))                                               \
    {                                                       \
        gdlf_loglvl (gdlf_level_emerg_s,                    \
                     gdlf_location "Assertion failed: ");   \
        gdlf_log (fmt);                                     \
    }                                                       \
} while (0)

#define gdlf_apassert(x, a, fmt...)                     \
if (!(x))                                               \
{                                                       \
    gdlf_loglvl (gdlf_level_emerg_s,                    \
                 gdlf_location "Assertion failed: ");   \
    gdlf_log (fmt);                                     \
    a;                                                  \
}

#else

#define gdlf_assert_act(x) /* x */ do { } while (0)
#define gdlf_c1assert(x) /* x */ 1
#define gdlf_c0assert(x) /* x */ 0
#define gdlf_assert(x) /* x */ do { } while (0)
#define gdlf_aassert(x, a) /* x, a */ do { } while (0)
#define gdlf_passert(x, fmt...) /* x, fmt */ do { } while (0)
#define gdlf_apassert(x, a, fmt...) /* x, a, fmt */ do { } while (0)

#endif

#ifdef GDLF_FNTRACE
#define gdlf_fntrace(format...)       \
do                                    \
{                                     \
    gdlf_loglvl (gdlf_level_emerg_s,  \
                 gdlf_locations       \
                 "fntrace: " format); \
    printk ("\n");                    \
} while (0)

#else
#define gdlf_fntrace(format...) do { } while (0)
#endif


/* 
 * $Log: gdlf.h,v $
 * Revision 1.1  1998/11/14 00:38:26  inaky
 * Isolated all the header files
 *
 * Revision 1.17  1998/10/21 23:57:31  inaky
 * Removed traces for snapshots, and moved force-definitions block
 * _after_ locals.
 *
 * Revision 1.16  1998/09/14 02:27:06  inaky
 * More settings ...
 *
 * Revision 1.15  1998/08/30 13:48:37  inaky
 * Removed assertions.
 *
 * Revision 1.14  1998/08/28 23:43:09  inaky
 * *** empty log message ***
 *
 * Revision 1.13  1998/08/27 22:12:56  inaky
 * Removed the forced conditions and added some asserts for putting into
 * logical expressions [one defaults to 1 and other to 0 on undefinition
 * of assertions].
 *
 * Revision 1.12  1998/08/24 02:13:46  inaky
 * Removed fn traces
 *
 * Revision 1.11  1998/08/24 02:10:32  inaky
 * Fixed in gdlf_a?assert() not to use do { } while (0), as if the action
 * was `break' or `continue', it wouldn't work as expected.
 *
 * Revision 1.10  1998/08/19 23:32:35  inaky
 * Debugging changes.
 *
 * Revision 1.9  1998/08/03 13:21:19  inaky
 * Added special clauses to be able to disable everyting, yet keeping
 * lcoal changes for some files (UHCI scheduler and irq-handler)
 *
 * Revision 1.8  1998/06/20 21:54:47  inaky
 * Massive changes to split the old monolithic hubd driver into a hub
 * manager (hub wrappers and daemon) and a dynamic driver, which can be
 * replaced and its different parts reused.
 *
 * Revision 1.7  1998/04/26 03:21:31  inaky
 * Fixed the gdlf_*() macros to use 'do { BODY } while(0)' constructs;
 * added the assertion expresion to the assert message [when it doesn't
 * have an own print message].
 *
 * Revision 1.6  1998/01/08 12:19:06  inaky
 * Added copyright strings to all important files
 *
 * Revision 1.5  1997/12/30 20:15:41  inaky
 * Moved everything to the new lists scheme. Untested yet.
 *
 * Revision 1.4  1997/12/29 00:25:35  inaky
 * Fixed unimplemented functions to return -ENOSYS and moving the khubdd
 * to use signals as commands.
 * Also added function traces on most files that had them disabled (donkey!)
 *
 * Revision 1.3  1997/12/27 02:24:02  inaky
 * Moving everything to use the GDLF for logging and debug checkings.
 * Removed the global array of USB hosts and moved it to a linked
 * list. Added an address field to each USB host controller.
 * Removed lots of cruft that were accumulated and useless. Still some
 * missing, but may be useful someday.
 *
 * Revision 1.2  1997/12/22 02:44:28  inaky
 * Trying to move all the documentation to the source files from the
 * header files, as it was duplicated.
 *
 * Revision 1.1  1997/12/13 20:56:31  inaky
 * Moving all the documentation to the .c files except the ones for data
 * structures and inline functions. Helps making the header files cleaner.
 *
 * Revision 1.1  1997/11/22 01:06:47  inaky
 * *** empty log message ***
 *
 */

