
/* USB Audio Devices Driver v$Revision: 1.1 $ 
 *
 * (C) 1997 by Iaky Prez Gonzlez <inaky@peloncho.fis.ucm.es>
 *
 * Distributed under the terms of the GNU Public License Version 2
 * See files COPYRIGHT and COPYING at the root distribution directory
 *
 * $Id: usb-audd.h,v 1.1 1998/11/14 00:38:26 inaky Exp $
 *
 * Changelog at the bottom 
 *
 *
 *     USB Audio Devices Driver
 *     ------------------------
 *
 *  This is the embrion of the USB Audio Devices Driver.
 *
 */

#ifndef _usb_audd_h_
#define _usb_audd_h_

#include <uusbd/usb.h>            /* UUSBD data structures */
#include <uusbd/usb-slist.h>      /* Sigle Linked Lists */

    /* Class' stuff */

#define AUDD_DEBUG

#define AUDD_CLASS              0x1     /* audio class */
#define AUDD_SUBCLASS_UNDEF     0x0     /* undefined interface */
#define AUDD_SUBCLASS_AUDIO_CTL 0x1     /* audio control interface */
#define AUDD_SUBCLASS_AUDIO_STR 0x2     /* audio streaming interface */
#define AUDD_SUBCLASS_MIDI_STR  0x3     /* midi streaming interface */

    /* Audio descriptor types */

typedef enum
{
    e_audd_desc_undefined = 0x20,   /* Undefined */
    e_audd_desc_device,             /* Device descriptor */
    e_audd_desc_configuration,      /* Configuration descriptor */
    e_audd_desc_string,             /* String descriptor */
    e_audd_desc_interface,          /* Interface descriptor */
    e_audd_desc_endpoint            /* Endpoint descriptor */
} e_audd_desc;

    /* Audio Control descriptor subtypes */

typedef enum
{
    e_audd_ac_desc_undefined = 0x00,/* Undefined */
    e_audd_ac_desc_header,          /* FIXME */
    e_audd_ac_desc_input,           /* FIXME */
    e_audd_ac_desc_output,          /* FIXME */
    e_audd_ac_desc_mixer,           /* FIXME */
    e_audd_ac_desc_selector,        /* FIXME */
    e_audd_ac_desc_feature,         /* FIXME */
    e_audd_ac_desc_processing,      /* FIXME */
    e_audd_ac_desc_extension        /* FIXME */
} e_audd_ac_desc;

    /* Audio Streaming Interface Descriptor Subtypes */

typedef enum
{
    e_audd_as_desc_undefined = 0x00,/* Undefined */
    e_audd_as_desc_general,         /* FIXME */
    e_audd_as_desc_format_type,     /* FIXME */
    e_audd_as_desc_format_specific  /* FIXME */
} e_audd_as_desc;

    /* Processing Units Process Types */

typedef enum
{
    e_audd_puproc_undefined = 0x00, /* Undefined */
    e_audd_puproc_updn_mix,         /* FIXME */
    e_audd_puproc_dolby_pro,        /* Dolby Prologic(tm) */
    e_audd_puproc_3ds_ext,          /* 3D Stereo Extender */
    e_audd_puproc_reverb,           /* Reverberation */
    e_audd_puproc_chorus,           /* FIXME */
    e_audd_puproc_dr_compr,         /* Dynamic Range Compression */
} e_audd_puproc;

    /* Audio Endpoint Descriptor Subtypes */

typedef enum
{
    e_audd_endp_desc_undefined = 0x00,  /* Undefined */
    e_audd_endp_desc_general            /* FIXME */
} e_audd_endp_desc;

    /* Audio Request Codes */

typedef enum
{
    e_audd_req_undefined = 0,
    e_audd_req_set_current = 0x01,
    e_audd_req_get_current = 0x81,
    e_audd_req_set_min = 0x02,
    e_audd_req_get_min = 0x82,
    e_audd_req_set_max = 0x03,
    e_audd_req_get_max = 0x83,
    e_audd_req_set_resollution = 0x04,
    e_audd_req_get_resollution = 0x84,
    e_audd_req_set_memory = 0x05,
    e_audd_req_get_memory = 0x85,
    e_audd_req_get_stat = 0xff
} e_audd_req;

    /* Terminal Control Selectors */

typedef enum
{
    e_audd_tc_sel_undefined = 0x00,
    e_audd_tc_sel_copy_prot
} e_audd_tm_csel;

    /* Feature Unit Control Selectors */

typedef enum
{
    e_audd_fu_csel_undefined = 0x00,
    e_audd_fu_csel_mute,
    e_audd_fu_csel_volume,
    e_audd_fu_csel_bass,
    e_audd_fu_csel_mid,
    e_audd_fu_csel_treble,
    e_audd_fu_csel_graphic_equalizer,
    e_audd_fu_csel_auto_gain,
    e_audd_fu_csel_delay,
    e_audd_fu_csel_bass_boost,
    e_audd_fu_csel_loudness
} e_audd_fu_csel;

    /* Up/Down Processing Unit Control Selector */

typedef enum
{
    e_audd_udpu_csel_undefined = 0x00,
    e_audd_udpu_csel_enable,
    e_audd_udpu_csel_mode_select,
} e_audd_udpu_csel;

    /* Dolby Prologic(tm) Processing Unit Control Selectors */

typedef enum
{
    e_audd_dppu_csel_undefined = 0x00,
    e_audd_dppu_csel_enable,
    e_audd_dppu_csel_mode_select
} e_audd_dppu_csel;

    /* 3D Stereo Extender Processing Unit Control Selectors */

typedef enum
{
    e_audd_3dpu_csel_undefined = 0x00,
    e_audd_3dpu_csel_enable,
    e_audd_3dpu_csel_mode_select,
    e_audd_3dpu_csel_spaciousness = 0x3
} e_audd_3dpu_csel;

    /* Reverberation Processing Unit Control Selectors */

typedef enum
{
    e_audd_rvpu_csel_undefined = 0x00,
    e_audd_rvpu_csel_enable,
    e_audd_rvpu_csel_level,
    e_audd_rvpu_csel_time,
    e_audd_rvpu_csel_feedback
} e_audd_rvpu_csel;

    /* Chorus Processing Unit Control Selectors */

typedef enum
{
    e_audd_chpu_csel_undefined = 0x00,
    e_audd_chpu_csel_enable,
    e_audd_chpu_csel_level,
    e_audd_chpu_csel_rate,
    e_audd_chpu_csel_depth
} e_audd_chpu_csel;

    /* Dynamic Range Compressor Unit Control Selectors */

typedef enum
{
    e_audd_drcpu_csel_undefined = 0x00,
    e_audd_drcpu_csel_enable,
    e_audd_drcpu_csel_rate,
    e_audd_drcpu_csel_maxampl,
    e_audd_drcpu_csel_threshold,
    e_audd_drcpu_csel_attack,
    e_audd_drcpu_csel_release
} e_audd_drcpu_csel;

    /* Extension Unit Control Selectors */

typedef enum
{
    e_audd_eu_csel_undefined = 0x00,
    e_audd_eu_csel_enable
} e_audd_eu_csel;

    /* Endpoint Control Selectors */

typedef enum
{
    e_audd_endp_csel_undefined = 0x00,
    e_audd_endp_csel_sampling_freq,
    e_audd_endp_csel_pitch,
} e_audd_endp_csel;


    /*
     * Audio Terminal Types
     * --------------------
     *
     *  These are the ultimate sources/sinks of audio data
     */

typedef
{
    e_audd_tt_undefined = 0x100, /* general */
    e_audd_tt_streaming,         /* signal carried on endpoint */
    e_audd_tt_vendor_spec,       /* signal on vendor specific iface */
                                    
    e_audd_tti_undefined = 0x200,/* input terminals */
    e_audd_tti_mike,             /* mike = microphone, shorter :) */
    e_audd_tti_mike_desktop,     /* on desktop / monitor embedded */
    e_audd_tti_mike_personal,    /* head mounted / clip on */
    e_audd_tti_mike_omnidir,     /* onmidirectional */
    e_audd_tti_mike_array,       /* array for directional proc'ing */
    e_audd_tti_mike_proc_array,  /* array with signal processor */

    e_audd_tto_undefined = 0x300,/* output terminals */
    e_audd_tto_spk,              /* generic speakers (spk in short) */
    e_audd_tto_headphones,       /* */
    e_audd_tto_hmda,             /* headphones for VR, with orientation */
    e_audd_tto_spk_desktop,      /* desktop or monitor speakers */
    e_audd_tto_spk_room,         /* big speakers, well heard on a room */
    e_audd_tto_spk_communication,/* spks designed for voice communication */
    e_audd_tto_spk_low_freq,     /* low freq speakers, not for speech/music */
                                    
    e_audd_ttb_undefined = 0x400,/* bidirectional */
    e_audd_ttb_handset,          /* handset (ie walkie-talkie) */
    e_audd_ttb_headset,          /* head mounted set */
    e_audd_ttb_spkphone_ner,     /* hands free, w/o echo reduction */
    e_audd_ttb_spkphone_es,      /* ditto, echo suppression (1/2 dupplex) */
    e_audd_ttb_spkphone_ec,      /* ditto, echo cancellation (full duplex) */
                                    
    e_audd_ttt_undefined = 0x500,/* telephony (telephone links and sets) */
    e_audd_ttt_phone_line,       /* POTS, ISDN, PBX or wireless link */
    e_audd_ttt_telephone,        /* usable as telephone/bidirectional */
    e_audd_ttt_dnline_phone,     /* standard set, usable as bi if line down */
                                    
    e_audd_ttx_undefined = 0x600,/* external [audio connectors] */
    e_audd_ttx_analog,           /* generic analog */
    e_audd_ttx_da_iface,         /* generic digital */
    e_audd_ttx_analog_line,      /* standard leves, 3.5mm jacks */
    e_audd_ttx_legacy,           /* connected to std [old] PC audio */
    e_audd_ttx_SPDIF_iface,      /* S/PDIF digital audio interface */
    e_audd_ttx_1394da,           /* 1394 audio streams */
    e_audd_ttx_1394dv,           /* 1394 A/V stream soundtrack */
                                    
    e_audd_tte_undefined = 0x700,/* embedded */
    e_audd_tte_lvl_cal_noise,    /* Internal noise source for calibration */
    e_audd_tte_equ_noise,        /* Internal noise source for measurement */
    e_audd_tte_CD_player,        /* CD/CDROM audio player */
    e_audd_tte_DAT,              /* Digital Audio Tape */
    e_audd_tte_DCC,              /* Digital Compact Cassette */
    e_audd_tte_minidisk,         /* Minidisk player */
    e_audd_tte_analog_tape,      /* Analog audio tapes */
    e_audd_tte_phonograph,       /* Vynil record player */
    e_audd_tte_VCR_audio,        /* VCR audio track */
    e_audd_tte_VD_audio,         /* VD player audio track */
    e_audd_tte_DVD_audio,        /* DVD player audio track */
    e_audd_tte_TV_tuner_audio,   /* TV tuner audio track */
    e_audd_tte_sat_tuner_audio,  /* Satellite tuner audio track */
    e_audd_tte_cable_tuner_audio,/* Cable tuner audio track */
    e_audd_tte_DSS_audio,        /* DSS receiver audio track */
    e_audd_tte_radio_receiver,   /*  */
    e_audd_tte_radio_transmitter,/*  */
    e_audd_tte_mtrack_recorder,  /* multi-track recorder */
    e_audd_tte_synthesizer       /*  */
} e_audd_tt;


    /*
     * Audio Data Format Types
     * -----------------------
     *
     *  These are all the audio format types supported. They're
     * divided in _three_ main types (I, II and III).
     */

typedef enum
{
    e_audd_ft_undefined = 0x00,
    e_audd_ft_i,
    e_audd_ft_ii,
    e_audd_ft_iii
} e_audd_ft;

    /* Type I */

typedef enum
{
    e_audd_fti_undefined = 0x00,
    e_audd_fti_pcm,
    e_audd_fti_pcm8,
    e_audd_fti_ieee_float,
    e_audd_fti_alaw,
    e_audd_fti_ulaw
} e_audd_fti;

    /* Type II */

typedef enum
{
    e_audd_ftii_undefined = 0x1000,
    e_audd_ftii_mpeg,
    e_audd_ftii_ac3
} e_audd_ftii;

    /* Type III */

typedef enum
{
    e_audd_ftiii_undefined = 0x2000,
    e_audd_ftiii_iec1973_ac3,
    e_audd_ftiii_iec1973_mpeg1_l1,
    e_audd_ftiii_iec1973_mpeg1_l23 = 0x2003,
    e_audd_ftiii_iec1973_mpeg1_noext = 0x2003,
    e_audd_ftiii_iec1937_mpeg2_ext,
    e_audd_ftiii_iec1937_mpeg2_l1_ls,
    e_audd_ftiii_iec1937_mpeg2_l23_ls
} e_audd_ftiii;


    /* 
     * MPEG and AC-3 Control Selectors 
     * -------------------------------
     *
     */

typedef enum
{
    e_audd_mpeg_csel_undefined = 0x00,
    e_audd_mpeg_csel_dual_channel,
    e_audd_mpeg_csel_second_stereo,
    e_audd_mpeg_csel_multilingual,
    e_audd_mpeg_csel_dyn_range,
    e_audd_mpeg_csel_scaling,
    e_audd_mpeg_csel_hilo_scaling
} e_audd_mpeg_csel;

typedef enum
{
    e_audd_ac3_csel_undefined = 0x00,
    e_audd_ac3_csel_mode,
    e_audd_ac3_csel_dyn_range,
    e_audd_ac3_csel_scaling,
    e_audd_ac3_csel_hilo_scaling
} e_audd_ac3_csel;


    /* 
     * Descriptors and associated data structures
     * ------------------------------------------
     *
     * Audio Control Interface
     * -----------------------
     *
     *  'length' includes the length of the ACI descriptor and all
     * associated unit and terminal descriptors. Following this
     * descriptor there are all the descriptors for associated units
     * and terminals. We'll build the Audio Controls from that.
     */

typedef struct
{
    ts_usb_desc_head    head;
    __u8                subtype;    /* e_audd_ac_desc_header */
    __u16               version;    /* ADC release in BCD */
    __u16               length;     /* length of all descriptors */
    __u8                ifaces;     /* number of streaming interfaces */
        /* now it comes a __u8 array of 'ifaces' elements, each one
         * containing the index number of an Audio/MIDI streaming
         * interface */
} ts_audd_ac_descr;


    /* Audio Control Interface */

typedef struct s_audd_ac ts_audd_ac;
struct s_audd_ac
{
    t_usb_cfg   cfg;
    t_usb_iface iface;
};

#ifdef __KERNEL__
#define ts_audd_ac_nozero(ac)                   \
do                                              \
{                                               \
} while (0)
#define ts_audd_ac_clean(ac)                    \
do                                              \
{                                               \
    memset ((ac), 0, sizeof (*ac));             \
    ts_audd_ac_nozero (ac);                     \
} while (0)
extern ts_audd_ac* ts_audd_ac_init (void);
extern void ts_audd_ac_release (ts_audd_ac*);
#endif /* __KERNEL__ */


    /* Audio Interface Collection */

struct s_audd_aic
{
    ts_audd_ac  ac;     /* AudioControl interface */
    ts_audd_as  *as;    /* Audio Streaming Interfaces (NULL if none) */
    __u8        ass;    /* Number of ASs (funny name, anyway) */
    ts_audd_ms  *ms;    /* Midi Streaming Interfaces (NULL if none) */
    __u8        mss;    /* Number of MSs */
};

#ifdef __KERNEL__
#define ts_audd_aic_nozero(aic)                 \
do                                              \
{                                               \
    ts_audd_ac_nozero (&(aic)->ac);             \
} while (0)
#define ts_audd_aic_clean(aic)                  \
do                                              \
{                                               \
    memset ((aic), 0, sizeof (*aic));           \
    ts_audd_aic_nozero (aic);                   \
} while (0)
extern ts_audd_aic* ts_audd_aic_init (void);
extern void ts_audd_aic_release (ts_audd_aic*);
extern ts_audd_aic* ts_audd_aic_a_init (size_t);
extern void ts_audd_aic_a_release (ts_audd_aic*, size_t);
#endif /* __KERNEL__ */


    /*
     * Common descriptor fields 
     * ------------------------
     */

typedef struct
{
    ts_usb_desc_head    head;
    __u8                subtype;    /* e_audd_ac_desc_* */
    __u8                id;         /* terminal/unit ID */
} ts_audd_cd_head;


    /*
     * Cluster descriptor [ADC 1.0 3.7.2.3]
     * ------------------
     *
     *  Channels in a cluster range from 1 to the number of
     * channels. Channel number zero addresses a master control in a
     * unit, affecting all channels at once. Channel 0xff means 'for
     * all available addressed controls'. The 'desc_idx' is the index
     * of the first string descriptor of an array of as many
     * descriptors as channels without predefined locations are.
     */

typedef struct
{
    __u8    channels;   /* [1, channels] */
    __u16   locations;  /* predefined spatial locations */
    __u8    descr_idx;
} ts_audd_desc_cluster;


    /*
     * Input Terminal Descriptor
     * -------------------------
     *
     */

typedef struct
{
    ts_audd_cd_head         head;
    __u16                   type;   /* terminal type e_audd_tt_* */
    __u8                    assoc;  /* associated output's ID */
    ts_audd_desc_cluster    cluster;/* cluster descriptor */
    __u8                    istr;   /* index of describing string */
} ts_audd_input_desc;

typedef struct s_audd_input ts_audd_input;
struct s_audd_input
{
    ts_audd_tid     id;
    e_audd_tt       ttype;
    ts_audd_output  *output;    /* bidirectional */
    ts_audd_cluster *cluster;
    __u8            istr;
};


    /*
     * Output Terminal Descriptor
     * --------------------------
     *
     */

typedef struct
{
    ts_audd_cd_head head;
    __u16           type;   /* terminal type e_audd_tt_* */
    __u8            assoc;  /* associated input's ID */
    __u8            source; /* id of unit/terminal which feeds this one */
    __u8            istr;   /* index of describing string */
} ts_audd_output_desc;

typedef struct s_audd_output ts_audd_output;
struct s_audd_output
{
    ts_audd_tid     id;
    e_audd_tt       ttype;
    ts_audd_input   *input;     /* bidirectional */
    ts_audd_output  *source;
    __u8            istr;
};

#warning FIXME: Type I format descriptor
#warning FIXME: Type II format descriptor
#warning FIXME: Type II MPEG format descriptor
#warning FIXME: Type II MPEG requests
#warning FIXME: Type II AC-3 format descriptor
#warning FIXME: Type II AC-3 requests
#warning FIXME: Type III format descriptor


    /* USB Audio class structure */

typedef struct s_audd ts_audd;
typedef struct s_audd_aic ts_audd_aic;
struct s_audd    /* structure */
{
    ts_snode        snode;

    ts_usb_device   *device;    /* device who owns this struct */
    ts_audd_aic     *aic;       /* Audio Interface Collections */
    __u8            aics;       /* Number of aics */
};

#ifdef __KERNEL__
#define ts_audd_nozero(audd)                    \
do                                              \
{                                               \
    ts_snode_clean (&(audd)->snode);            \
} while (0)
#define ts_audd_clean(audd)                     \
do                                              \
{                                               \
    memset ((audd), 0, sizeof (*audd));         \
    ts_audd_clean (audd);                       \
} while (0)
extern ts_audd* ts_audd_init (void);
extern void ts_audd_release (ts_audd*);
#endif /* __KERNEL__ */


    /* Audio Streaming Interface */

typedef struct s_audd_as ts_audd_as;
struct s_audd_as
{
    t_usb_cfg   cfg;
    t_usb_iface iface;
};

#ifdef __KERNEL__
#define ts_audd_as_clean(as)                    \
do                                              \
{                                               \
    memset ((as), 0, sizeof (*as));             \
} while (0)
extern ts_audd_as* ts_audd_as_init (void);
extern void ts_audd_as_release (ts_audd_as*);
extern ts_audd_as* ts_audd_as_a_init (size_t);
extern void ts_audd_as_a_release (ts_audd_as*, size_t);
#endif /* __KERNEL__ */


    /* MIDI Streaming Interface */

typedef struct s_audd_ms ts_audd_ms;
struct s_audd_ms
{
    t_usb_cfg   cfg;
    t_usb_iface iface;
};

#ifdef __KERNEL__
#define ts_audd_ms_clean(ms)                    \
do                                              \
{                                               \
    memset ((ms), 0, sizeof (*ms));             \
} while (0)
extern ts_audd_ms* ts_audd_ms_init (void);
extern void ts_audd_ms_relemse (ts_audd_ms*);
extern ts_audd_ms* ts_audd_ms_a_init (size_t);
extern void ts_audd_ms_a_release (ts_audd_ms*, size_t);
#endif /* __KERNEL__ */

    /* Class specific variables && housekeeping (class-hkeep.c) */

#ifdef __KERNEL__
extern ts_slist audd_list;
extern int audd_register (ts_audd*);
extern int audd_unregister (ts_audd*);
extern int audd_destroy (ts_audd*);

    /* Class interface to configuration (class-cfg.c) */

extern int audd_config (ts_usb_cfel*, ts_usb_device*, 
                        ts_usb_cfg*, t_usb_cfg, 
                        ts_usb_iface*, t_usb_iface, 
                        ts_usb_aset*, t_usb_aset, __u32);
extern int audd_unconfig (ts_usb_device*, unsigned);

    /* Driver housekeeping (driver.c) */

extern ts_usb_driver *audd_driver;
extern volatile int audd_errno;
extern int audd_init (void);
extern int audd_release (void);
extern int audd_shutdown (void);
extern int audd_reference (ts_usb_device*, int);
#endif /* __KERNEL__ */

#endif /* _usb_audd_h_ */


/* Log:
 *
 * $Log: usb-audd.h,v $
 * Revision 1.1  1998/11/14 00:38:26  inaky
 * Isolated all the header files
 *
 * Revision 1.3  1998/09/14 19:48:38  inaky
 * Added new stuff; still incomplete [it doesn't even compile].
 *
 * Revision 1.2  1998/08/29 00:05:56  inaky
 * Added the reference function.
 *
 * Revision 1.1  1998/08/24 22:16:59  inaky
 * Added first revision of embrionic audio driver
 *
 */

