
/* USB Communications Class Devices Driver v$Revision: 1.1 $ 
 *
 * (C) 1998 by Iaky Prez Gonzlez <inaky@peloncho.fis.ucm.es>
 *
 * Distributed under the terms of the GNU Public License Version 2
 * See files COPYRIGHT and COPYING at the root distribution directory
 *
 * $Id: usb-cdcd.h,v 1.1 1998/11/14 00:38:26 inaky Exp $
 *
 * Changelog at the bottom 
 *
 *
 *     Communications Class Driver
 *     ---------------------------
 */


#ifndef _usb_cdcd_h_
#define _usb_cdcd_h_

#include <uusbd/usb-stddesc.h>    /* Standard USB descriptors */

#define CDCD_CLASS          0x02

    /* Communication interface */

#define CDCD_IFACE_CLASS    0x02

#define CDCD_SUBCLASS_DIRECT        0x01
#define CDCD_SUBCLASS_ABSTRACT      0x02
#define CDCD_SUBCLASS_TELEPHONE     0x03    /* FIXME: ensure these */
#define CDCD_SUBCLASS_MULTI_CHANNEL 0x04
#define CDCD_SUBCLASS_CAPI          0x05
#define CDCD_SUBCLASS_ETHERNET      0x06
#define CDCD_SUBCLASS_ATM           0x07
#define CDCD_SUBCLASS_RNDIS         0x08

#define CDCD_PROTOCOL_AT    0x01

    /* Data interface */

#define CDCD_D_CLASS        0x0a

#define CDCD_D_PROTOCOL_ISDN_BRI    0x30    /* Physical iface for ISDN BRI */
#define CDCD_D_PROTOCOL_HDLC        0x32
#define CDCD_D_PROTOCOL_TRANSPARENT 0x33
#define CDCD_D_PROTOCOL_Q921M       0x50    /* Q921 link protocol managemet */
#define CDCD_D_PROTOCOL_Q921        0x51    /* data link protocol */
#define CDCD_D_PROTOCOL_Q921TM      0x52    /* TEI multiplexor */
#define CDCD_D_PROTOCOL_V42BIS      0x90    /* data compression */
#define CDCD_D_PROTOCOL_EISDN       0x91    /* Euro ISDN */
#define CDCD_D_PROTOCOL_V120        0x92    /* V24 rate adaptation to ISDN */
#define CDCD_D_PROTOCOL_CAPI20      0x93    /* ISDN CAPI 2.0 */
#define CDCD_D_PROTOCOL_FD          0xfe    /* described using
                                               protocol functional
                                               descriptor or
                                               communication class
                                               interface */
#define CDCD_D_PROTOCOL_VENDOR      0xff    /* vendor specific */


    /* 
     * Class' stuff
     *
     *
     * This example is only done for class stuff. It is expandable to
     * subclass, protocol, vendor ... any kind of stuff.
     */


    /* Class structure */

typedef struct s_cdcd ts_cdcd;
struct s_cdcd    /* structure */
{
    ts_snode        snode;
    ts_usb_device   *device;
    
        /* FIXME: your data here */
};

    /* Class structure housekeeping (class-generic.c) */

#ifdef __KERNEL__
#define ts_cdcd_clean(cdcd)                     \
do                                              \
{                                               \
    memset ((cdcd), 0, sizeof (*(cdcd)));       \
    ts_cdcd_nozero (cdcd);                      \
} while (0)

#define ts_cdcd_nozero(cdcd)                    \
do                                              \
{                                               \
    ts_snode_nozero (&(cdcd)->snode);           \
} while (0)

extern ts_cdcd* ts_cdcd_init (void);
extern void ts_cdcd_release (ts_cdcd* cdcd);
#endif /* __KERNEL__ */

    /* Class specific variables && housekeeping (class-hkeep.c) */

#ifdef __KERNEL__
extern ts_slist cdcd_list;
extern int cdcd_register (ts_cdcd*);
extern int cdcd_unregister (ts_cdcd*);
extern int cdcd_destroy (ts_cdcd*);

    /* Class interface to configuration (class-cfg.c) */

extern int cdcd_config (ts_usb_cfel*, ts_usb_device*, 
                        ts_usb_cfg*, t_usb_cfg, 
                        ts_usb_iface*, t_usb_iface, 
                        ts_usb_aset*, t_usb_aset, __u32);
extern int cdcd_unconfig (ts_usb_device*, unsigned);

    /* Driver housekeeping (driver.c) */

extern ts_usb_driver *cdcd_driver;
extern int cdcd_init (void);
extern int cdcd_release (void);
extern int cdcd_shutdown (void);
extern int cdcd_reference (ts_usb_device*, int);

#endif /* __KERNEL__ */

#endif /* _usb_cdcd_h_ */


/* Log:
 *
 * $Log: usb-cdcd.h,v $
 * Revision 1.1  1998/11/14 00:38:26  inaky
 * Isolated all the header files
 *
 * Revision 1.3  1998/10/31 02:15:59  inaky
 * Added the CDC class codes.
 *
 * Revision 1.2  1998/10/14 21:57:05  inaky
 * Removed the structure house-keeping stuff.
 *
 * Revision 1.1  1998/10/13 22:58:38  inaky
 * First embrionical CDCD driver
 *
 */

