
/* USB Human Interface Devices Driver v$Revision: 1.1 $ 
 *
 * (C) 1997 by Iaky Prez Gonzlez <inaky@peloncho.fis.ucm.es>
 *
 * Distributed under the terms of the GNU Public License Version 2
 * See files COPYRIGHT and COPYING at the root distribution directory
 *
 * $Id: usb-hidd-tags.h,v 1.1 1998/11/14 00:38:26 inaky Exp $
 *
 * Changelog at the bottom 
 *
 *     USB Human Interface Devices - Usage Tags
 *     ----------------------------------------
 */

#ifndef _usb_hidd_tags_h_
#define _usb_hidd_tags_h_

    /* NOTE: on most tables, all undefined values are reserved */

    /* HID Usage Pages (HID 1.0, A.1) */

typedef enum
{
    e_hidd_up_undefined = 0,
    e_hidd_up_gdesktop,         /* Generic desktop controls */
    e_hidd_up_kbd_kpad = 0x7,   /* Keyboard/keypad keys */
    e_hidd_up_leds,
    e_hidd_up_buttons,
    e_hidd_up_vendor = 0xff
} e_hidd_up;


    /* Generic Desktop Page (HID 1.0, A.2) */

typedef enum
{
    e_hidd_gdp_undefined = 0,
    e_hidd_gdp_pointer,
    e_hidd_gdp_mouse,
    e_hidd_gdp_joystick = 0x4,
    e_hidd_gdp_gamepad,
    e_hidd_gdp_keyboard,
    e_hidd_gdp_keypad,
    e_hidd_gdp_x = 0x30,        /* translation, inc on lefto to rigth */
    e_hidd_gdp_y,               /* inc on far to near */
    e_hidd_gdp_z,               /* inc on high to low */
    e_hidd_gdp_rx,              /* rotation, right hand rule */
    e_hidd_gdp_ry,
    e_hidd_gdp_rz,
    e_hidd_gdp_slider,
    e_hidd_gdp_dial,
    e_hidd_gdp_wheel,
    e_hidd_gdp_hat_switch
} e_hidd_gdp;


    /* Keyboard/keypad page (HID 1.0 A.3) 
     *
     *  NOTE: different prefixes are used for keyboard (e_hidd_kbd_*)
     *        and keypad (e_hidd_kpd_*) keys, though the sequence is
     *        unique.
     */

typedef enum
{
    e_hidd_kbd_nei = 0,
    e_hidd_kbd_rollover_error,
    e_hidd_kbd_postfail,
    e_hidd_kbd_undefined_error,

    e_hidd_kbd_a,               /* 0x4 */
    e_hidd_kbd_b,
    e_hidd_kbd_c,
    e_hidd_kbd_d,
    e_hidd_kbd_e,
    e_hidd_kbd_f,
    e_hidd_kbd_g,
    e_hidd_kbd_h,
    e_hidd_kbd_i,
    e_hidd_kbd_j,
    e_hidd_kbd_k,
    e_hidd_kbd_l,
    e_hidd_kbd_m,
    e_hidd_kbd_n,
    e_hidd_kbd_o,
    e_hidd_kbd_p,
    e_hidd_kbd_q,
    e_hidd_kbd_r,
    e_hidd_kbd_s,
    e_hidd_kbd_t,
    e_hidd_kbd_u,
    e_hidd_kbd_v,
    e_hidd_kbd_w,
    e_hidd_kbd_x,
    e_hidd_kbd_y,
    e_hidd_kbd_z,

    e_hidd_kbd_1_slash,         /* 0x1e */
    e_hidd_kbd_2_at,
    e_hidd_kbd_3_numbersign,
    e_hidd_kbd_4_dollar,
    e_hidd_kbd_5_percent,
    e_hidd_kbd_6_asciicircum,
    e_hidd_kbd_7_ampersand,
    e_hidd_kbd_8_asterisk,
    e_hidd_kbd_9_parenleft,
    e_hidd_kbd_0_parenright,

    e_hidd_kbd_return,          /* 0x28 */
    e_hidd_kbd_escape,
    e_hidd_kbd_backspace,
    e_hidd_kbd_tab,
    e_hidd_kbd_spacebar,

    e_hidd_kbd_minus_underscore,    /* 0x2d */
    e_hidd_kbd_equal_plus,
    e_hidd_kbd_bracket_brace_left,
    e_hidd_kbd_bracket_brace_right,
    e_hidd_kbd_backslash_pipe,
    e_hidd_kbd_nonus_numbersign_tilde,
    e_hidd_kbd_semicolon_colon,
    e_hidd_kbd_apostrophe_quotebdl, /* 0x34 */
    e_hidd_kbd_grave_accent_tilde,
    e_hidd_kbd_comma_less,
    e_hidd_kbd_period_greater,
    e_hidd_kbd_slash_question,
    e_hidd_kbd_caps_lock,

    e_hidd_kbd_F1,              /* 0x3a */
    e_hidd_kbd_F2,
    e_hidd_kbd_F3,
    e_hidd_kbd_F4,
    e_hidd_kbd_F5,
    e_hidd_kbd_F6,
    e_hidd_kbd_F7,
    e_hidd_kbd_F8,
    e_hidd_kbd_F9,
    e_hidd_kbd_F10,
    e_hidd_kbd_F11,
    e_hidd_kbd_F12,

    e_hidd_kbd_print_screen,    /* 0x46 */
    e_hidd_kbd_scroll_lock,
    e_hidd_kbd_pause,

    e_hidd_kbd_insert,          /* 0x49 */
    e_hidd_kbd_home,
    e_hidd_kbd_pgup,
    e_hidd_kbd_delete,
    e_hidd_kbd_end,
    e_hidd_kbd_pgdn,
    e_hidd_kbd_right_arrow,
    e_hidd_kbd_left_arrow,
    e_hidd_kbd_down_arrow,
    e_hidd_kbd_up_arrow,

    e_hidd_kpd_num_lock_clear,  /* 0x53 */
    e_hidd_kpd_slash,
    e_hidd_kpd_asterisk,
    e_hidd_kpd_minus,
    e_hidd_kpd_plus,
    e_hidd_kpd_enter,
    e_hidd_kpd_1_end,
    e_hidd_kpd_2_down_arrow,
    e_hidd_kpd_3_pgdn,
    e_hidd_kpd_4_left_arrow,
    e_hidd_kpd_5,
    e_hidd_kpd_6_right_arrow,
    e_hidd_kpd_7_home,
    e_hidd_kpd_8_up_arrow,
    e_hidd_kpd_9_pgup,
    e_hidd_kpd_0_insert,
    e_hidd_kpd_period_delete,

    e_hidd_kbd_nonus_backslash_pipe,    /* 0x64 */
    e_hidd_kbd_application,
    e_hidd_kbd_power,
    e_hidd_kpd_equal,

    e_hidd_kbd_F13,             /* 0x68 */
    e_hidd_kbd_F14,
    e_hidd_kbd_F15,
    e_hidd_kbd_F16,
    e_hidd_kbd_F17,
    e_hidd_kbd_F18,
    e_hidd_kbd_F19,
    e_hidd_kbd_F20,
    e_hidd_kbd_F21,
    e_hidd_kbd_F22,
    e_hidd_kbd_F23,
    e_hidd_kbd_F24,

    e_hidd_kbd_execute,         /* 0x74 */
    e_hidd_kbd_help,
    e_hidd_kbd_menu,
    e_hidd_kbd_select,
    e_hidd_kbd_stop,
    e_hidd_kbd_again,
    e_hidd_kbd_undo,
    e_hidd_kbd_cut,
    e_hidd_kbd_copy,
    e_hidd_kbd_paste,
    e_hidd_kbd_find,

    e_hidd_kbd_mute,            /* 0x7f */
    e_hidd_kbd_volume_up,
    e_hidd_kbd_volume_dn,

    e_hidd_kbd_locking_caps_lock,   /* 0x82 */
    e_hidd_kbd_locking_num_lock,
    e_hidd_kbd_locking_scroll_lock,

    e_hidd_kpd_comma,           /* 0x85 */
    e_hidd_kpd_equal_sign,      /* ??? e_hidd_kpd_equal */

    e_hidd_kbd_kanji1,          /* 0x87 */
    e_hidd_kbd_kanji2,
    e_hidd_kbd_kanji3,
    e_hidd_kbd_kanji4,
    e_hidd_kbd_kanji5,
    e_hidd_kbd_kanji6,
    e_hidd_kbd_kanji7,
    e_hidd_kbd_kanji8,
    e_hidd_kbd_kanji9,

    e_hidd_kbd_lang1,           /* 0x90 */
    e_hidd_kbd_lang2,
    e_hidd_kbd_lang3,
    e_hidd_kbd_lang4,
    e_hidd_kbd_lang5,
    e_hidd_kbd_lang6,
    e_hidd_kbd_lang7,
    e_hidd_kbd_lang8,
    e_hidd_kbd_lang9,

    e_hidd_kbd_alternate_erase, /* 0x99 */
    e_hidd_kbd_sysreq,
    e_hidd_kbd_cancel,
    e_hidd_kbd_clear,
    e_hidd_kbd_prior,
    e_hidd_kbd_return2,     /* ??? */
    e_hidd_kbd_separator,
    e_hidd_kbd_out,
    e_hidd_kbd_oper,
    e_hidd_kbd_clear_again,

    e_hidd_kbd_crsel_props,     /* 0xa3 */
    e_hidd_kbd_ex_sel,

    e_hidd_kbd_left_control = 0xe0,
    e_hidd_kbd_left_shift,
    e_hidd_kbd_left_alt,
    e_hidd_kbd_left_gui,
    e_hidd_kbd_right_control,
    e_hidd_kbd_right_shift,
    e_hidd_kbd_right_alt,
    e_hidd_kbd_right_gui,
} e_hidd_kkp;


    /* LED Page (HID 1.0 A.4) */

typedef enum
{
    e_hidd_led_undefined = 0x0,
    e_hidd_led_num_lock,
    e_hidd_led_caps_lock,
    e_hidd_led_scroll_lock,
    e_hidd_led_compose,
    e_hidd_led_kana,
    e_hidd_led_power,
    e_hidd_led_shift,
    e_hidd_led_do_not_disturb,
    e_hidd_led_mute,
    e_hidd_led_tone_enable,
    e_hidd_led_high_cut_filter,
    e_hidd_led_low_cut_filter,
    e_hidd_led_equalizer_enable,
    e_hidd_led_sound_field_on,
    e_hidd_led_speaker,
    e_hidd_led_head_set,
    e_hidd_led_hold,
    e_hidd_led_microphone,
    e_hidd_led_coverage,
    e_hidd_led_night_mode,
    e_hidd_led_send_calls,
    e_hidd_led_call_pickup,
    e_hidd_led_conference,
    e_hidd_led_stand_by,
    e_hidd_led_camera_on,
    e_hidd_led_camera_off,
    e_hidd_led_on_line,
    e_hidd_led_off_line,
    e_hidd_led_busy,
    e_hidd_led_surround_on,
    e_hidd_led_repeat,
    e_hidd_led_stereo,
    e_hidd_led_sampling_rate_detected,
    e_hidd_led_spinning,
    e_hidd_led_CAV,
    e_hidd_led_CLV,
    e_hidd_led_recording_format_detected,
    e_hidd_led_off_hook,
    e_hidd_led_ring,
    e_hidd_led_message_waiting,
    e_hidd_led_data_mode,
    e_hidd_led_battery_op,      /* Battery operation */
    e_hidd_led_battery_ok,
    e_hidd_led_battery_low,
    e_hidd_led_ready,
    e_hidd_led_paper_out,
    e_hidd_led_paper_jam,
    e_hidd_led_remote,
    e_hidd_led_forward,
    e_hidd_led_reverse,
    e_hidd_led_stop,
    e_hidd_led_rewind,
    e_hidd_led_fast_forward,
    e_hidd_led_play,
    e_hidd_led_pause,
    e_hidd_led_record,
    e_hidd_led_error,
} e_hidd_led;


    /* Button page (HID 1.0 A.5) */

typedef enum
{
    e_hidd_but_none = 0x0,  /* no button pressed */
    e_hidd_but_1,           /* primary / trigger */
    e_hidd_but_2,           /* secondary */
    e_hidd_but_3,           /* tertiary */

        /* rest looks useless, but it is in the page, all right? :) */ 

    e_hidd_but_4,
    e_hidd_but_5,
    e_hidd_but_6,
    e_hidd_but_7,
    e_hidd_but_8,
    e_hidd_but_9,
    e_hidd_but_10,
    e_hidd_but_11,
    e_hidd_but_12,
    e_hidd_but_13,
    e_hidd_but_14,
    e_hidd_but_15,
    e_hidd_but_16,
    e_hidd_but_17,
    e_hidd_but_18,
    e_hidd_but_19,
    e_hidd_but_20,
    e_hidd_but_21,
    e_hidd_but_22,
    e_hidd_but_23,
    e_hidd_but_24,
    e_hidd_but_25,
    e_hidd_but_26,
    e_hidd_but_27,
    e_hidd_but_28,
    e_hidd_but_29,
    e_hidd_but_30,
    e_hidd_but_31,
    e_hidd_but_32,
    e_hidd_but_33,
    e_hidd_but_34,
    e_hidd_but_35,
    e_hidd_but_36,
    e_hidd_but_37,
    e_hidd_but_38,
    e_hidd_but_39,
    e_hidd_but_40,
    e_hidd_but_41,
    e_hidd_but_42,
    e_hidd_but_43,
    e_hidd_but_44,
    e_hidd_but_45,
    e_hidd_but_46,
    e_hidd_but_47,
    e_hidd_but_48,
    e_hidd_but_49,
    e_hidd_but_50,
    e_hidd_but_51,
    e_hidd_but_52,
    e_hidd_but_53,
    e_hidd_but_54,
    e_hidd_but_55,
    e_hidd_but_56,
    e_hidd_but_57,
    e_hidd_but_58,
    e_hidd_but_59,
    e_hidd_but_60,
    e_hidd_but_61,
    e_hidd_but_62,
    e_hidd_but_63,
    e_hidd_but_64,
    e_hidd_but_65,
    e_hidd_but_66,
    e_hidd_but_67,
    e_hidd_but_68,
    e_hidd_but_69,
    e_hidd_but_70,
    e_hidd_but_71,
    e_hidd_but_72,
    e_hidd_but_73,
    e_hidd_but_74,
    e_hidd_but_75,
    e_hidd_but_76,
    e_hidd_but_77,
    e_hidd_but_78,
    e_hidd_but_79,
    e_hidd_but_80,
    e_hidd_but_81,
    e_hidd_but_82,
    e_hidd_but_83,
    e_hidd_but_84,
    e_hidd_but_85,
    e_hidd_but_86,
    e_hidd_but_87,
    e_hidd_but_88,
    e_hidd_but_89,
    e_hidd_but_90,
    e_hidd_but_91,
    e_hidd_but_92,
    e_hidd_but_93,
    e_hidd_but_94,
    e_hidd_but_95,
    e_hidd_but_96,
    e_hidd_but_97,
    e_hidd_but_98,
    e_hidd_but_99,
    e_hidd_but_100,
    e_hidd_but_101,
    e_hidd_but_102,
    e_hidd_but_103,
    e_hidd_but_104,
    e_hidd_but_105,
    e_hidd_but_106,
    e_hidd_but_107,
    e_hidd_but_108,
    e_hidd_but_109,
    e_hidd_but_110,
    e_hidd_but_111,
    e_hidd_but_112,
    e_hidd_but_113,
    e_hidd_but_114,
    e_hidd_but_115,
    e_hidd_but_116,
    e_hidd_but_117,
    e_hidd_but_118,
    e_hidd_but_119,
    e_hidd_but_120,
    e_hidd_but_121,
    e_hidd_but_122,
    e_hidd_but_123,
    e_hidd_but_124,
    e_hidd_but_125,
    e_hidd_but_126,
    e_hidd_but_127,
    e_hidd_but_128,
    e_hidd_but_129,
    e_hidd_but_130,
    e_hidd_but_131,
    e_hidd_but_132,
    e_hidd_but_133,
    e_hidd_but_134,
    e_hidd_but_135,
    e_hidd_but_136,
    e_hidd_but_137,
    e_hidd_but_138,
    e_hidd_but_139,
    e_hidd_but_140,
    e_hidd_but_141,
    e_hidd_but_142,
    e_hidd_but_143,
    e_hidd_but_144,
    e_hidd_but_145,
    e_hidd_but_146,
    e_hidd_but_147,
    e_hidd_but_148,
    e_hidd_but_149,
    e_hidd_but_150,
    e_hidd_but_151,
    e_hidd_but_152,
    e_hidd_but_153,
    e_hidd_but_154,
    e_hidd_but_155,
    e_hidd_but_156,
    e_hidd_but_157,
    e_hidd_but_158,
    e_hidd_but_159,
    e_hidd_but_160,
    e_hidd_but_161,
    e_hidd_but_162,
    e_hidd_but_163,
    e_hidd_but_164,
    e_hidd_but_165,
    e_hidd_but_166,
    e_hidd_but_167,
    e_hidd_but_168,
    e_hidd_but_169,
    e_hidd_but_170,
    e_hidd_but_171,
    e_hidd_but_172,
    e_hidd_but_173,
    e_hidd_but_174,
    e_hidd_but_175,
    e_hidd_but_176,
    e_hidd_but_177,
    e_hidd_but_178,
    e_hidd_but_179,
    e_hidd_but_180,
    e_hidd_but_181,
    e_hidd_but_182,
    e_hidd_but_183,
    e_hidd_but_184,
    e_hidd_but_185,
    e_hidd_but_186,
    e_hidd_but_187,
    e_hidd_but_188,
    e_hidd_but_189,
    e_hidd_but_190,
    e_hidd_but_191,
    e_hidd_but_192,
    e_hidd_but_193,
    e_hidd_but_194,
    e_hidd_but_195,
    e_hidd_but_196,
    e_hidd_but_197,
    e_hidd_but_198,
    e_hidd_but_199,
    e_hidd_but_200,
    e_hidd_but_201,
    e_hidd_but_202,
    e_hidd_but_203,
    e_hidd_but_204,
    e_hidd_but_205,
    e_hidd_but_206,
    e_hidd_but_207,
    e_hidd_but_208,
    e_hidd_but_209,
    e_hidd_but_210,
    e_hidd_but_211,
    e_hidd_but_212,
    e_hidd_but_213,
    e_hidd_but_214,
    e_hidd_but_215,
    e_hidd_but_216,
    e_hidd_but_217,
    e_hidd_but_218,
    e_hidd_but_219,
    e_hidd_but_220,
    e_hidd_but_221,
    e_hidd_but_222,
    e_hidd_but_223,
    e_hidd_but_224,
    e_hidd_but_225,
    e_hidd_but_226,
    e_hidd_but_227,
    e_hidd_but_228,
    e_hidd_but_229,
    e_hidd_but_230,
    e_hidd_but_231,
    e_hidd_but_232,
    e_hidd_but_233,
    e_hidd_but_234,
    e_hidd_but_235,
    e_hidd_but_236,
    e_hidd_but_237,
    e_hidd_but_238,
    e_hidd_but_239,
    e_hidd_but_240,
    e_hidd_but_241,
    e_hidd_but_242,
    e_hidd_but_243,
    e_hidd_but_244,
    e_hidd_but_245,
    e_hidd_but_246,
    e_hidd_but_247,
    e_hidd_but_248,
    e_hidd_but_249,
    e_hidd_but_250,
    e_hidd_but_251,
    e_hidd_but_252,
    e_hidd_but_253,
    e_hidd_but_254,
    e_hidd_but_255
} e_hidd_but;

#endif /* _usb_hidd_tags_h_ */


/* Log:
 *
 * $Log: usb-hidd-tags.h,v $
 * Revision 1.1  1998/11/14 00:38:26  inaky
 * Isolated all the header files
 *
 * Revision 1.1  1998/09/24 16:10:24  inaky
 * Added all the known usage tables (monitors missing)
 *
 */

