
/* USB Human Interface Devices Driver v$Revision: 1.1 $ 
 *
 * (C) 1997 by Iaky Prez Gonzlez <inaky@peloncho.fis.ucm.es>
 *
 * Distributed under the terms of the GNU Public License Version 2
 * See files COPYRIGHT and COPYING at the root distribution directory
 *
 * $Id: usb-hidd.h,v 1.1 1998/11/14 00:38:26 inaky Exp $
 *
 * Changelog at the bottom 
 *
 *     USB Human Interface Devices - Configuration
 *     -------------------------------------------
 *
 *  This is the embrion of the USB HID Driver.
 *
 */

#ifndef _usb_hidd_h_
#define _usb_hidd_h_

#include <linux/fs.h>           /* file_operations */
#include <linux/miscdevice.h>   /* struct miscdevice */

#include <uusbd/usb.h>          /* UUSBD data structures */


    /* Class' stuff */

#define HIDD_DEBUG
#define HIDD_CLASS              0x3 /* HID class */
#define HIDD_SUBCLASS_REPORT    0x0 /* Report protocol */
#define HIDD_SUBCLASS_BOOT      0x1 /* Boot protocol */
#define HIDD_CTL_STR USB_CTL_STR "-hid"

enum                                /* HID requests */
{
    e_hidd_req_get_report = 1,
    e_hidd_req_get_idle,
    e_hidd_req_get_protocol,
    e_hidd_req_set_report = 9,
    e_hidd_req_set_idle,
    e_hidd_req_set_protocol
} e_hidd_req;


    /* USB HID class structure */

typedef struct s_hidd ts_hidd;
struct s_hidd  /* structure */
{
    ts_snode snode;
};

#ifdef __KERNEL__
#define ts_hidd_nozero(hidd)                    \
do                                              \
{                                               \
    ts_snode_clean (&(hidd)->snode);            \
} while (0)

#define ts_hidd_clean(hidd)                     \
do                                              \
{                                               \
    memset ((hidd), 0, sizeof (*hidd));         \
    ts_hidd_nozero (hidd);                      \
} while (0)

static inline
ts_hidd* ts_hidd_init (void)
{
    ts_hidd* hidd;
    gdlf_fntrace ("ts_hidd_init (void)");

    hidd = usb_kmalloc (sizeof (*hidd), GFP_KERNEL);
    if (hidd == NULL)
        return NULL;
    ts_hidd_clean (hidd);
    return hidd;
}

static inline
void ts_hidd_release (ts_hidd* hidd)
{
    gdlf_fntrace ("ts_hidd_release (0x%p)", hidd);
    gdlf_aassert (hidd != NULL, return);
    gdlf_assert (hidd);
    ts_snode_release (&hidd->snode);
    return;
}
#endif /* __KERNEL__ */


    /* Class specific variables && housekeeping (class-hkeep.c) */

#ifdef __KERNEL__
extern ts_slist hidd_list;
extern int hidd_register (ts_hidd*);
extern int hidd_unregister (ts_hidd*);
extern int hidd_destroy (ts_hidd*);

    /* Class interface to configuration (class-cfg.c) */

extern int hidd_config (ts_usb_cfel*, ts_usb_device*, 
                        ts_usb_cfg*, t_usb_cfg, 
                        ts_usb_iface*, t_usb_iface, 
                        ts_usb_aset*, t_usb_aset, __u32);
extern int hidd_unconfig (ts_usb_device*, unsigned);

    /* Driver housekeeping (driver.c) */

extern ts_usb_driver *hidd_driver;
extern volatile int hidd_errno;
extern int hidd_init (void);
extern int hidd_release (void);
extern int hidd_shutdown (void);
extern int hidd_reference (ts_usb_device*, int);
#endif /* __KERNEL__ */

#endif /* _usb_hidd_h_ */


/* Log:
 *
 * $Log: usb-hidd.h,v $
 * Revision 1.1  1998/11/14 00:38:26  inaky
 * Isolated all the header files
 *
 * Revision 1.4  1998/09/14 19:49:42  inaky
 * Removed the Boot Protocol stuff to take it out to another header
 * file. This way HIDD and HIDBP are completely separated drivers, though
 * usb-hidbp.h uses some of the definitions in usb-hidd.h
 *
 * Revision 1.3  1998/09/11 00:51:52  inaky
 * Moved the boot protocol to a single driver, to isolate it more easily.
 *
 * Revision 1.2  1998/08/30 13:58:25  inaky
 * Added boot protocol and basic user-mode read support.
 *
 * Revision 1.1  1998/08/28 23:53:16  inaky
 * New driver
 *
 * Revision 1.1  1998/08/28 00:14:23  inaky
 * New driver ...
 *
 */

