
/* USB Driver Stack v$Revision: 1.1 $ 
 *
 * (C) 1997 by Iaky Prez Gonzlez <inaky@peloncho.fis.ucm.es>
 *
 * Distributed under the terms of the GNU Public License Version 2
 * See files COPYRIGHT and COPYING at the root distribution directory
 *
 * $Id: usb-hubd.h,v 1.1 1998/11/14 00:38:27 inaky Exp $
 *
 * Changelog at the bottom 
 *
 *
 *     Generic Hub Driver
 *     ------------------
 */

#ifndef _usb_hubd_h_
#define _usb_hubd_h_

#include <asm/spinlock.h>   /* spinlock_t */

#include <uusbd/usb-stddesc.h>  /* Standard USB descriptors */
#include <uusbd/usb-hub.h>      /* Hub Manager stuff */

#define HUBD_CLASS  0x9     /* From http://www.usb.org */

#ifdef __KERNEL__

    /* Driver housekeeping (driver.c) */

extern ts_usb_driver *hubd_driver;
extern const ts_usb_hub_ops hubd_operations;

extern int hubd_init (void);
extern int hubd_release (void);
extern int hubd_shutdown (void);
extern int hubd_reference (ts_usb_hub*, int);

    /* Configuration (cfg.c) */

extern int hubd_config (ts_usb_cfel*, ts_usb_device*, 
                        ts_usb_cfg*, t_usb_cfg, 
                        ts_usb_iface*, t_usb_iface,
                        ts_usb_aset*, t_usb_aset, __u32);
extern int hubd_unconfig (ts_usb_device*, __u32);

    /* house keeping (hkeep.c) */

extern int hubd_free (ts_usb_hub*);

    /* driver methods (requests.c) */

extern int hubd_descriptor_get (ts_usb_hub*, __u8, void*, size_t);
extern int hubd_descriptor_set (ts_usb_hub*, __u8, const void*, size_t);
extern int hubd_descriptor_fill (ts_usb_hub*);
extern __s32 hubd_status_get (ts_usb_hub*);
extern __s8 hubd_bus_status_get (ts_usb_hub*, __u8);
extern int hubd_feature (ts_usb_hub*, e_usb_hub_featsel, e_usb_req);
extern int hubd_schedule_status_change (ts_usb_hub*);

    /* driver ports' methods (ports.c) */

extern int hubd_port_feature (ts_usb_hub*, __u8,
                              e_usb_port_featsel, e_usb_req);
extern __s32 hubd_port_status_get (ts_usb_hub*, __u8);
extern int hubd_port_reset (ts_usb_hub*, __u8);

#endif /* __KERNEL__ */

#endif /* _usb_hubd_h_ */


/* Log:
 *
 * $Log: usb-hubd.h,v $
 * Revision 1.1  1998/11/14 00:38:27  inaky
 * Isolated all the header files
 *
 * Revision 1.2  1998/11/09 21:37:04  inaky
 * Fixed positions of declarations of functions to match the file they're
 * declared at.
 *
 * Revision 1.1  1998/09/24 16:09:29  inaky
 * Moved from root, uneeded here.
 *
 * Revision 1.4  1998/09/14 02:32:59  inaky
 * Fixed prototypes for hubd_descriptor_*()
 *
 * Revision 1.3  1998/07/12 22:47:57  inaky
 * Added new proto.
 *
 * Revision 1.2  1998/06/21 20:12:48  inaky
 * Fixed the declaration of hubd_reference() and added hubd_free() (for
 * the operations structure).
 *
 * Revision 1.1  1998/06/20 21:54:51  inaky
 * Massive changes to split the old monolithic hubd driver into a hub
 * manager (hub wrappers and daemon) and a dynamic driver, which can be
 * replaced and its different parts reused.
 *
 * Revision 1.28  1998/05/24 12:01:18  inaky
 * Added inclusion of spinlock.h for hubd data queue locking.
 *
 * Revision 1.27  1998/05/02 00:47:52  inaky
 * Moved in the definitions, some declarations and prototypes from the
 * khubdd.
 *
 * Revision 1.26  1998/04/13 21:56:14  inaky
 * Added some standard definitions, fixed 'current' mask definition,
 * changed some stuff for stetical purposes, added comments on fields
 * which are automagically updated by hub*_status_get(). New fields to
 * ts_hubd [cfel position, status and trbuffer]; fixed some declarations,
 * adapted the hubd_config() to the new scheme; fixed flags for unconfig
 * and config methods; added destructor; added callback and callback
 * handling methods; fixed more declarations and finished rest of hub
 * functions.
 *
 * Revision 1.25  1998/03/28 19:26:10  inaky
 * Fixed declaration of hubd_search()
 *
 * Revision 1.24  1998/01/11 15:44:16  inaky
 * Finished functions from the descriptor, inteface and endpoint modules,
 * fixed docs, pretty printed and removed craps.
 *
 * Revision 1.23  1998/01/08 12:19:09  inaky
 * Added copyright strings to all important files
 *
 * Revision 1.22  1998/01/08 11:49:11  inaky
 * Finishing the configuration system. Only the final cfel-stuff is
 * missing to implement; however, nothing is tested.
 *
 * Revision 1.21  1998/01/02 00:12:16  inaky
 * Finished the new driver scheme. Documented at usbd-drv.c
 *
 * Revision 1.20  1997/12/31 05:37:00  inaky
 * Almost finished all the lists and life cycle for hosts, drivers and
 * devices. Some craps to be fixed yet.
 *
 * Revision 1.19  1997/12/30 22:30:17  inaky
 * Added the foundations of the HCD and fixed some bugs.
 *
 * Revision 1.18  1997/12/30 20:15:43  inaky
 * Moved everything to the new lists scheme. Untested yet.
 *
 * Revision 1.17  1997/12/29 00:25:36  inaky
 * Fixed unimplemented functions to return -ENOSYS and moving the khubdd
 * to use signals as commands.
 * Also added function traces on most files that had them disabled (donkey!)
 *
 * Revision 1.16  1997/12/27 20:06:27  inaky
 * Finished fixinf the __KERNEL__ #ifdef wrapping and moved the ts_dtor.h
 * to usb-dtor.h
 *
 * Revision 1.15  1997/12/27 16:44:24  inaky
 * Moving to GDLF and fixinf docs. Removing more cruft.
 *
 * Revision 1.14  1997/12/22 02:44:29  inaky
 * Trying to move all the documentation to the source files from the
 * header files, as it was duplicated.
 *
 * Revision 1.13  1997/12/13 20:56:31  inaky
 * Moving all the documentation to the .c files except the ones for data
 * structures and inline functions. Helps making the header files cleaner.
 *
 * Revision 1.12  1997/12/08 02:40:59  inaky
 * Fixed some stuff of the #includes, modified the makefile targets to be
 * more flexible, tried to finish the device configuration section.
 *
 * Revision 1.11  1997/12/06 00:19:40  inaky
 * Almost finished the usbd/host, usbd/cfg and a lot of meat from
 * usbd/device. Also added the driver definition. Got to polish it yet
 * ...
 *
 * Revision 1.10  1997/11/29 20:21:48  inaky
 * Documented and almost fixes the read/write/abort API, documented a
 * damn lot and implemented some stuff
 *
 * Revision 1.9  1997/11/29 00:46:30  inaky
 * Documented almost all the hub basic API, added spin locks to
 * usb_host_register() and some more stuff. Going for more.
 *
 * Revision 1.8  1997/10/31 05:44:30  inaky
 * *** empty log message ***
 *
 * Revision 1.7  1997/10/17 00:16:00  inaky
 * *** empty log message ***
 *
 * Revision 1.6  1997/10/02 21:29:01  inaky
 * Remade all the directory structure to make it cleaner when the USB
 * class drivers come.
 *
 * Revision 1.5  1997/10/01 21:28:11  inaky
 * Re-making a lot of stuff in the hadnling of host, hub and device
 * initialization and handling, following a more correct approach of
 * registering/unregistering. Also moved the usb_host array to be an
 * array of pointers, so dynamic reallocation of it is easier, and the
 * host structure needs not to be moved in memory. The same applies to
 * the device arrays for each host.
 *
 * Revision 1.4  1997/09/30 13:04:42  inaky
 * Moved the name of the package to uusbd: Universal USB Driver
 *
 * Revision 1.3  1997/04/20 21:25:56  inaky
 * Finished the spec for the Hub Class (now gotta implement it)
 * :). Started coding the Frame Lists code, hope UHCI support will be
 * finished in some sessions.
 * This code is breaking, I'm doing something bad with kmallocs. kFree()ing
 * something not kmalloc()ed.
 *
 * Revision 1.2  1997/04/06 21:14:31  inaky
 * Still with the definition of the access interface. More or less it is
 * taking an stable form.
 *
 * Revision 1.1  1997/03/31 20:24:46  inaky
 * Pfff ... a lot. Mostly added descriptors stuff (HUB class still is
 * missing a lot) and I think I've got a definitive idea of how to
 * implement the storing of configurations and interfaces.
 *
 * Revision 1.1  1997/02/22 22:25:44  inaky
 * *** empty log message ***
 */

