
/* USB Driver Stack v$Revision: 1.1 $ 
 *
 * (C) 1997 by Iaky Prez Gonzlez <inaky@peloncho.fis.ucm.es>
 *
 * Distributed under the terms of the GNU Public License Version 2
 * See files COPYRIGHT and COPYING at the root distribution directory
 *
 * $Id: usb-misc.h,v 1.1 1998/11/14 00:38:27 inaky Exp $
 *
 * Changelog at the bottom 
 *
 *
 *     Misc stuff
 *     ----------
 *
 * Documentation at misc.c, even for inlined functions.
 *
 *  This file should provide stubs for the many operating systems
 * on which UUSB should compile. Currently it is only Linux, but this
 * is on preparation for others to come.
 *
 *  IANF, this file should just #include misc-PLATFORM.h.
 */

#ifndef _usb_misc_h_
#define _usb_misc_h_

#include <asm/io.h>         /* IO stuff */
#include <asm/atomic.h>     /* atomic_t and atomic_*() */
#include <linux/delay.h>    /* mdelay() */

void usb_dump_bytes (const __u8*, size_t);

static inline 
void usb_waitms (unsigned long int ms) 
{
    return mdelay (ms);
}

unsigned usb_seek_iorange (unsigned, unsigned);
unsigned usb_seek_irq (unsigned);
__u32 usb_signature (const char*, ssize_t);

#define outb_mask(_oldval, _newval, _mask, _port)           \
    outb ((_newval & _mask) | (_oldval & ~_mask), _port)

#define outw_mask(_oldval, _newval, _mask, _port)           \
    outw ((_newval & _mask) | (_oldval & ~_mask), _port)

#define outl_mask(_oldval, _newval, _mask, _port)           \
    outl ((_newval & _mask) | (_oldval & ~_mask), _port)

#define le_readl(a) (le32_to_cpu (readl (a)))
#define le_writel(a,p) (writel (cpu_to_le32 (a), p))

static inline 
void le_writel_mask (__u32 _oldval, __u32  _newval, 
                  __u32 _mask, unsigned long _addr)
{
    writel (cpu_to_le32 ((_newval & _mask) | (_oldval & ~_mask)),
            _addr);
}


        /*
         *  These ones are neccessary, as they allow us to write bits
         * in ports, by reading and writing masked.
         */

static inline __u8 usb_bits8 (unsigned int portno, __u8 nval, __u8 mask)
{
    __u8 val;

    val = inw (portno);                               /* read old value */
    if (mask != 0)                                    /* got to write? */
        outw ((nval & mask) | (val & ~mask), portno); /* write it ... */
    return val;
}


static inline __u16 usb_bits16 (unsigned int portno, __u16 nval, __u16 mask)
{
    __u16 val;

    val = inw (portno);                               /* read old value */
    if (mask != 0)                                    /* got to write? */
        outw ((nval & mask) | (val & ~mask), portno); /* write it ... */
    return val;
}


static inline __u32 usb_bits32 (unsigned int portno, __u32 nval, __u32 mask)
{
    __u32 val;

    val = inl (portno);                               /* read old value */
    if (mask != 0)                                    /* got to write? */
        outl ((nval & mask) | (val & ~mask), portno); /* write it ... */
    return val;
}


    /* Convert between addresses stored in physical/little endian and
     * vice-versa. 
     *
     *  This does th bus/virtual translation and also little
     * endian/cpu endian translation.
     *
     * Parameters:  
     *
     *  unsigned long   32 bits address from the QH or TD. Note
     *                  they're stored there using the 28 upper
     *                  bits. You MUST shift it left 4 (<<4) firstly
     *                  (or AND it).
     *
     *  void*           Kernel land pointer to convert. It must be
     *                  aligned to 16 bytes or you'll crack it when
     *                  storing it as 28 bits.
     *
     * Returns:
     *
     *  void*           Pointer converted from TD/QH address. 
     *
     *  unsigned long   Pointer converted to TD/QH format from the
     *                  const void* pointer.
     */

static inline
void* ptr2kvoid (unsigned long ptr)
{
    return bus_to_virt (le32_to_cpu (ptr));
}

static inline
unsigned long kvoid2ptr (void* kvoid)
{
    return cpu_to_le32 (virt_to_bus (kvoid));
}

    /* Calculate log in base 2 from 0 to 255 */

extern __u8 log2table[256];

static inline
__u8 log2 (__u8 arg)
{
    return log2table[arg];
}

#endif /* _usb_misc_h_ */

/* Log:
 *
 * $Log: usb-misc.h,v $
 * Revision 1.1  1998/11/14 00:38:27  inaky
 * Isolated all the header files
 *
 * Revision 1.3  1998/10/31 02:08:58  inaky
 * Moved the logarithm base 2 function to use a table for 255 elements.
 *
 * Revision 1.2  1998/09/14 02:33:54  inaky
 * Added little endian readl/writel from memory with auto order
 * conversion. Moved the ptr2kvoid, kvoid2ptr inlines to usb-misc.h.
 *
 * Revision 1.1  1998/08/24 02:20:56  inaky
 * Changed misc.h's name
 *
 * Revision 1.16  1998/07/28 23:18:46  inaky
 * Added macros for outputting with mask.
 *
 * Revision 1.15  1998/06/20 21:54:50  inaky
 * Massive changes to split the old monolithic hubd driver into a hub
 * manager (hub wrappers and daemon) and a dynamic driver, which can be
 * replaced and its different parts reused.
 *
 * Revision 1.14  1998/02/12 12:35:51  inaky
 * Added usb_signature() declaration.
 *
 * Revision 1.13  1998/02/10 17:17:15  inaky
 * Added comment on preparation for making this sub-module the
 * stub-module. Changed GDLF ident to USB-MISC and added the undefinition
 * of all the GDLF stuff.
 *
 * Revision 1.12  1998/01/08 12:19:07  inaky
 * Added copyright strings to all important files
 *
 * Revision 1.11  1997/12/29 00:25:36  inaky
 * Fixed unimplemented functions to return -ENOSYS and moving the khubdd
 * to use signals as commands.
 * Also added function traces on most files that had them disabled (donkey!)
 *
 * Revision 1.10  1997/12/27 03:24:22  inaky
 * Moved the monitored memory management code to it's own module
 *
 * Revision 1.9  1997/12/27 02:24:03  inaky
 * Moving everything to use the GDLF for logging and debug checkings.
 * Removed the global array of USB hosts and moved it to a linked
 * list. Added an address field to each USB host controller.
 * Removed lots of cruft that were accumulated and useless. Still some
 * missing, but may be useful someday.
 *
 * Revision 1.8  1997/12/22 02:44:29  inaky
 * Trying to move all the documentation to the source files from the
 * header files, as it was duplicated.
 *
 * Revision 1.7  1997/12/08 02:40:58  inaky
 * Fixed some stuff of the #includes, modified the makefile targets to be
 * more flexible, tried to finish the device configuration section.
 *
 * Revision 1.6  1997/09/01 19:14:42  inaky
 * Backup check in
 *
 * Revision 1.5  1997/06/22 19:54:31  inaky
 * Seems that the moving of all the hw dependants stuff to the HCI
 * modules is done. Got to test it yet, but at least it compiles cleanly.
 *
 * Revision 1.4  1997/06/19 20:38:17  inaky
 * Moving al hardware dependant stuff to the HCD modules
 *
 * Revision 1.3  1997/04/06 21:14:32  inaky
 * Still with the definition of the access interface. More or less it is
 * taking an stable form.
 *
 * Revision 1.2  1997/03/14 20:47:30  inaky
 * Fixing auto-detection code. Still fails when the PCI USB device
 * isn't enabled on BIOS ??? Why???
 *
 * Revision 1.1  1997/02/26 22:29:09  inaky
 * Miscellaneous code
 *
 * Revision 1.1  1997/02/24 22:30:20  inaky
 * Adding some missing files
 *
 * Revision 1.1  1997/02/22 22:25:44  inaky
 * *** empty log message ***
 *
 *
 */

