
/* USB Driver Stack v$Revision: 1.1 $ 
 *
 * (C) 1997 by Iaky Prez Gonzlez <inaky@peloncho.fis.ucm.es>
 *
 * Distributed under the terms of the GNU Public License Version 2
 * See files COPYRIGHT and COPYING at the root distribution directory
 *
 * $Id: usb-mm.h,v 1.1 1998/11/14 00:38:27 inaky Exp $
 *
 * Changelog at the bottom 
 *
 * Header file for using the monitored memory management.
 *
 */

#ifndef _usb_mm_h_
#define _usb_mm_h_

#ifdef __KERNEL__

    /* 
     * USB MM configuration
     */

#define USB_KMALLOC /* define this to get monitored memory management */


#include <linux/mm.h>

    /*
     * WARNING! aligments may only be 2, 4, 8, 16, 32, ... pow (2,n)
     */

extern int usb_mm_init (void);
extern int usb_mm_shutdown (void);
extern void* __usb_kmalloc_align (size_t, int, int);
extern void* __usb_realloc (void*, size_t, size_t, int);

#ifdef USB_KMALLOC          /* checking malloc routines */

extern unsigned kmalloccnt; /* number of times kmalloc is called */
extern unsigned kpagecnt;   /* number of times get_free_pages is called */

extern void* __usb_kmalloc (size_t, int, const char*, int);
extern void* __usb_kmalloc_align_d (size_t, int, int, const char*, int);
extern void* __usb_realloc_d (void*, size_t, size_t, int, const char*, int);
extern void __usb_kfree (void*, const char*, int);
extern unsigned long __usb_get_free_pages (int, unsigned long, int, 
                                           const char*, int);
extern void __usb_free_pages (unsigned long, unsigned long, 
                              const char*, int);

#define usb_kmalloc(size, prio) \
    __usb_kmalloc ((size), (prio), __FILE__, __LINE__)

#define usb_kmalloc_align(size, alignment, prio) \
    __usb_kmalloc_align_d ((size), (alignment), (prio), __FILE__, __LINE__)

#define usb_realloc(ptr, osize, nsize, prio) \
    __usb_realloc_d ((ptr), (osize), (nsize), (prio), __FILE__, __LINE__)

#define usb_kfree(ptr) \
    __usb_kfree ((ptr), __FILE__, __LINE__)

#define usb_get_free_pages(prio, order, dma) \
    __usb_get_free_pages ((prio), (order), (dma), __FILE__, __LINE__)

#define usb_free_pages(addr, order) \
    __usb_free_pages ((addr), (order), __FILE__, __LINE__)

#else   /* USB_KMALLOC */

#define usb_kmalloc(prio,size)  \
    kmalloc ((prio),(size))

#define usb_kmalloc_align(size, alignment, prio) \
    __usb_kmalloc_align ((size), (alignment), (prio))

#define usb_realloc(ptr, osize, nsize, prio) \
    __usb_realloc ((ptr), (osize), (nsize), (prio))

#define usb_kfree(ptr)          \
    kfree ((ptr))

#define usb_get_free_pages(prio,order,dma) \
    __get_free_pages ((prio), (order), (dma))

#define usb_free_pages(addr,order) \
    __usb_free_pages ((addr), (order))

#endif /* USB_KMALLOC */
#endif /* __KERNEL__ */
#endif /* _usb_mm_h_ */


/* Log:
 *
 * $Log: usb-mm.h,v $
 * Revision 1.1  1998/11/14 00:38:27  inaky
 * Isolated all the header files
 *
 * Revision 1.6  1998/08/19 23:47:39  inaky
 * Added alignment allocation functions, quite hacky.
 *
 * Revision 1.5  1998/07/01 21:36:29  inaky
 * Added a realloc function
 *
 * Revision 1.4  1998/01/08 12:19:09  inaky
 * Added copyright strings to all important files
 *
 * Revision 1.3  1997/12/29 00:25:37  inaky
 * Fixed unimplemented functions to return -ENOSYS and moving the khubdd
 * to use signals as commands.
 * Also added function traces on most files that had them disabled (donkey!)
 *
 * Revision 1.2  1997/12/27 16:44:24  inaky
 * Moving to GDLF and fixinf docs. Removing more cruft.
 *
 * Revision 1.1  1997/12/27 03:24:23  inaky
 * Moved the monitored memory management code to it's own module
 *
 */

