
/* USB Mass Storage Devices Driver v$Revision: 1.1 $ 
 *
 * (C) 1998 by Iaky Prez Gonzlez <inaky@peloncho.fis.ucm.es>
 *
 * Distributed under the terms of the GNU Public License Version 2
 * See files COPYRIGHT and COPYING at the root distribution directory
 *
 * $Id: usb-msdd.h,v 1.1 1998/11/14 00:38:27 inaky Exp $
 *
 * Changelog at the bottom 
 *
 *
 *     Mass Storage Driver
 *     -------------------
 */

#ifndef _usb_msdd_h_
#define _usb_msdd_h_

#include <uusbd/usb-stddesc.h>    /* Standard USB descriptors */


    /* 
     * For each driver, you define some data for it, as the names, et
     * al and then depending on which type of driver you'll use private
     * class, subclass, protocol or vendor specific data.
     *
     *  Thus, the structure of this file is (should be :) -Note that
     * for SI we denote an specific item, wether it is class, subclass,
     * protocol or vendor related-:
     *
     *  Names
     *  SI specific declarations
     *      Data structures types (DST)
     *      DST housekeeping functions
     *      SI specific variables
     *      SI specific housekeeping functions
     *      SI Device [un]configuration functions
     *  Driver structure
     *  Initialization/Shutdown routines
     *
     *  Normally you'll have different structures for class, subclass,
     * protocol, vendor and/or device stuff, and thus, different
     * lists, variables, etc ... though probably interrelated. Thus,
     * you may repeat the above structure.
     *
     *  Note that for using the lists code provided by usb-slist.h,
     * you MUST set as first vairable of your structures a ts_snode. 
     * Take a look at the hub driver for a sample implementation.
     */



    /* 
     * Class' stuff
     *
     *
     * This example is only done for class stuff. It is expandable to
     * subclass, protocol, vendor ... any kind of stuff.
     */


    /* Class structure */

typedef struct s_msdd ts_msdd;
struct s_msdd    /* structure */
{
    ts_snode        snode;
    ts_usb_device   *device;
    
        /* FIXME: your data here */
};

    /* Class structure housekeeping (class-generic.c) */

#ifdef __KERNEL__
#define ts_msdd_clean(class)             \
do                                              \
{                                               \
    memset ((class), 0, sizeof (*(class)));     \
    ts_msdd_nozero (class);              \
} while (0)

#define ts_msdd_nozero(class)            \
do                                              \
{                                               \
    ts_snode_nozero (&(class)->snode);          \
} while (0)

static inline
ts_msdd* ts_msdd_init (void)
{
    ts_msdd* class;
    
    gdlf_fntrace ("ts_msdd_init()");
    class = usb_kmalloc (sizeof (ts_msdd), GFP_KERNEL);
    if (class == NULL)
    {
        gdlf_err ("Cannot allocate space for a "
                  "ts_msdd structure\n");
        return NULL;
    }
    ts_msdd_clean (class);
    return class;              
}

extern int msdd_destroy (ts_msdd*);
static inline
void ts_msdd_release (ts_msdd* class)
{
    gdlf_fntrace ("ts_msdd_release (0x%p)", class);
    gdlf_aassert (class != NULL, return);
    gdlf_assert (class->snode.dtor.fn == (tf_dtor)msdd_destroy);
#warning FIXME: release class stuff
    return;
}

#endif /* __KERNEL__ */

    /* Class specific variables && housekeeping (class-hkeep.c) */

#ifdef __KERNEL__
extern ts_slist msdd_list;
extern int msdd_register (ts_msdd*);
extern int msdd_unregister (ts_msdd*);
extern int msdd_destroy (ts_msdd*);

    /* Class interface to configuration (class-cfg.c) */

extern int msdd_config (ts_usb_cfel*, ts_usb_device*, 
                               ts_usb_cfg*, t_usb_cfg, 
                               ts_usb_iface*, t_usb_iface, 
                               ts_usb_aset*, t_usb_aset, __u32);
extern int msdd_unconfig (ts_usb_device*, unsigned);

    /* Driver housekeeping (driver.c) */

extern ts_usb_driver *msdd_driver;
extern int msdd_init (void);
extern int msdd_release (void);
extern int msdd_shutdown (void);
extern int msdd_reference (ts_usb_device*, int);
#endif /* __KERNEL__ */

#endif /* _usb_msdd_h_ */


/* Log:
 *
 * $Log: usb-msdd.h,v $
 * Revision 1.1  1998/11/14 00:38:27  inaky
 * Isolated all the header files
 *
 * Revision 1.2  1998/11/09 21:39:22  inaky
 * Fixed names [removed _class]
 *
 * Revision 1.1  1998/10/14 21:48:50  inaky
 * Beginnings of a driver ...
 *
 */

