
/* USB Driver Stack v$Revision: 1.1 $ 
 *
 * (C) 1997 by Iaky Prez Gonzlez <inaky@peloncho.fis.ucm.es>
 *
 * Distributed under the terms of the GNU Public License Version 2
 * See files COPYRIGHT and COPYING at the root distribution directory
 *
 * $Id: usb-nls.h,v 1.1 1998/11/14 00:38:27 inaky Exp $
 *
 * Changelog at the bottom 
 *
 *
 *     NLS support for USB
 *     -------------------
 *
 */


#ifndef _usb_nls_h_
#define _usb_nls_h_

#include <linux/nls.h>      /* struct nls_table */

    /* 
     * String descriptor language codes
     * --------------------------------
     *
     *  This info is extracted from somewhere from M$, don't ask me
     * where, coz I don't know. Is funny, didn't we have std ISO codes
     * for languages? Didn't UNICODE define'em? If these codes follow
     * ISO or UNICODE, why ain't them referred as being so in the USB
     * docs instead of pointing to M$'s docs? Why the Coyote isn't
     * able to drop a hand on the Roadrunner?
     *
     *  Well, the fact is these codes are 16 bit, and have the lower
     * 10 bit for specifying the mother language, and the rest, the
     * upper 6 bits for a dialect of that mother language.
     *
     *  The USB_LNG_* are mother languages, and USB_SLNG_* are
     * dialects. Use the USB_LNG_BUILD() macro to construct'em and
     * USB_LNG_MOTHER() and USB_LNG_DIALECT() to extract the
     * components.
     */

#define USB_LNG_BUILD(m,s) ( (( ((__u16)(s)) << 10 ) & 0xe0) | ( ((__u16)(m)) & 0x1f ) )
#define USB_LNG_MOTHER(l) ((__u16)(l)) & 0x1f
#define USB_LNG_DIALECT(l) ((((__u16)(l)) >> 10) & 0x1f)

    /* Mother language codes */

#define USB_LNG_NONE       0x00                           

#define USB_LNG_AFRIKAANS  0x36
#define USB_LNG_ALBANIAN   0x1c
#define USB_LNG_ARABIC     0x01
#define USB_LNG_BASQUE     0x2d
#define USB_LNG_BELARUSIAN 0x23
#define USB_LNG_BULGARIAN  0x02
#define USB_LNG_CATALAN    0x03
#define USB_LNG_CHINESE    0x04
#define USB_LNG_CROATIAN   0x1a
#define USB_LNG_CZECH      0x05
#define USB_LNG_DANISH     0x06
#define USB_LNG_DUTCH      0x13
#define USB_LNG_ENGLISH    0x09
#define USB_LNG_ESTONIAN   0x25
#define USB_LNG_FAEROESE   0x38
#define USB_LNG_FARSI      0x29
#define USB_LNG_FINNISH    0x0b
#define USB_LNG_FRENCH     0x0c
#define USB_LNG_GERMAN     0x07
#define USB_LNG_GREEK      0x08
#define USB_LNG_HEBREW     0x0d
#define USB_LNG_HINDI      0x39
#define USB_LNG_HUNGARIAN  0x0e
#define USB_LNG_ICELANDIC  0x0f
#define USB_LNG_INDONESIAN 0x21
#define USB_LNG_ITALIAN    0x10
#define USB_LNG_JAPANESE   0x11
#define USB_LNG_KOREAN     0x12
#define USB_LNG_LATVIAN    0x26
#define USB_LNG_LITHUANIAN 0x27
#define USB_LNG_MACEDONIAN 0x2f
#define USB_LNG_MALAY      0x3e
#define USB_LNG_NORWEGIAN  0x14
#define USB_LNG_POLISH     0x15
#define USB_LNG_PORTUGUESE 0x16
#define USB_LNG_ROMANIAN   0x18
#define USB_LNG_RUSSIAN    0x19
#define USB_LNG_SERBIAN    0x1a
#define USB_LNG_SLOVAK     0x1b
#define USB_LNG_SLOVENIAN  0x24
#define USB_LNG_SPANISH    0x0a
#define USB_LNG_SWAHILI    0x41
#define USB_LNG_SWEDISH    0x1d
#define USB_LNG_THAI       0x1e
#define USB_LNG_TURKISH    0x1f
#define USB_LNG_UKRAINIAN  0x22
#define USB_LNG_VIETNAMESE 0x2a

    /* Dialect codes */

#define USB_SLNG_NONE                       0x00  /* no dialect */
#define USB_SLNG_DEFAULT                    0x01  /* user-wide default */
#define USB_SLNG_SYS_DEFAULT                0x02  /* system-wide default */
                                                  
#define USB_SLNG_ARABIC_SAUDI_ARABIA        0x01  /* Arabic/Saudi Arabia */
#define USB_SLNG_ARABIC_IRAQ                0x02  /* Arabic/Iraq */
#define USB_SLNG_ARABIC_EGYPT               0x03  /* Arabic/Egypt */
#define USB_SLNG_ARABIC_LIBYA               0x04  /* Arabic/Libya */
#define USB_SLNG_ARABIC_ALGERIA             0x05  /* Arabic/Algeria */
#define USB_SLNG_ARABIC_MOROCCO             0x06  /* Arabic/Morocco */
#define USB_SLNG_ARABIC_TUNISIA             0x07  /* Arabic/Tunisia */
#define USB_SLNG_ARABIC_OMAN                0x08  /* Arabic/Oman */
#define USB_SLNG_ARABIC_YEMEN               0x09  /* Arabic/Yemen */
#define USB_SLNG_ARABIC_SYRIA               0x0a  /* Arabic/Syria */
#define USB_SLNG_ARABIC_JORDAN              0x0b  /* Arabic/Jordan */
#define USB_SLNG_ARABIC_LEBANON             0x0c  /* Arabic/Lebanon */
#define USB_SLNG_ARABIC_KUWAIT              0x0d  /* Arabic/Kuwait */
#define USB_SLNG_ARABIC_UAE                 0x0e  /* Arabic/U.A.E */
#define USB_SLNG_ARABIC_BAHRAIN             0x0f  /* Arabic/Bahrain */
#define USB_SLNG_ARABIC_QATAR               0x10  /* Arabic/Qatar */
#define USB_SLNG_CHINESE_TRADITIONAL        0x01  /* Chinese/Taiwan */
#define USB_SLNG_CHINESE_SIMPLIFIED         0x02  /* Chinese/PR China */
#define USB_SLNG_CHINESE_HONGKONG           0x03  /* Chinese/Hong Kong */
#define USB_SLNG_CHINESE_SINGAPORE          0x04  /* Chinese/Singapore */
#define USB_SLNG_CHINESE_MACAU              0x05  /* Chinese/Macau */
#define USB_SLNG_DUTCH                      0x01  /* Dutch */
#define USB_SLNG_DUTCH_BELGIAN              0x02  /* Dutch/Belgian */
#define USB_SLNG_ENGLISH_US                 0x01  /* English/USA */
#define USB_SLNG_ENGLISH_UK                 0x02  /* English/UK */
#define USB_SLNG_ENGLISH_AUS                0x03  /* English/Australian */
#define USB_SLNG_ENGLISH_CAN                0x04  /* English/Canadian */
#define USB_SLNG_ENGLISH_NZ                 0x05  /* English/New Zealand */
#define USB_SLNG_ENGLISH_EIRE               0x06  /* English/Irish */
#define USB_SLNG_ENGLISH_SOUTH_AFRICA       0x07  /* English/South Africa */
#define USB_SLNG_ENGLISH_JAMAICA            0x08  /* English/Jamaica */
#define USB_SLNG_ENGLISH_CARIBBEAN          0x09  /* English/Caribbean */
#define USB_SLNG_ENGLISH_BELIZE             0x0a  /* English/Belize */
#define USB_SLNG_ENGLISH_TRINIDAD           0x0b  /* English/Trinidad */
#define USB_SLNG_ENGLISH_ZIMBABWE           0x0c  /* English/Zimbabwe */
#define USB_SLNG_ENGLISH_PHILIPPINES        0x0d  /* English/Philippines */
#define USB_SLNG_FRENCH                     0x01  /* French */
#define USB_SLNG_FRENCH_BELGIAN             0x02  /* French/Belgian */
#define USB_SLNG_FRENCH_CANADIAN            0x03  /* French/Canadian */
#define USB_SLNG_FRENCH_SWISS               0x04  /* French/Swiss */
#define USB_SLNG_FRENCH_LUXEMBOURG          0x05  /* French/Luxembourg */
#define USB_SLNG_FRENCH_MONACO              0x06  /* French/Monaco */
#define USB_SLNG_GERMAN                     0x01  /* German */
#define USB_SLNG_GERMAN_SWISS               0x02  /* German/Swiss */
#define USB_SLNG_GERMAN_AUSTRIAN            0x03  /* German/Austrian */
#define USB_SLNG_GERMAN_LUXEMBOURG          0x04  /* German/Luxembourg */
#define USB_SLNG_GERMAN_LIECHTENSTEIN       0x05  /* German/Liechtenstein */
#define USB_SLNG_ITALIAN                    0x01  /* Italian */
#define USB_SLNG_ITALIAN_SWISS              0x02  /* Italian/Swiss */
#define USB_SLNG_KOREAN                     0x01  /* Korean/Extended Wansung */
#define USB_SLNG_KOREAN_JOHAB               0x02  /* Korean/Johab */
#define USB_SLNG_LITHUANIAN                 0x01  /* Lithuanian */
#define USB_SLNG_LITHUANIAN_CLASSIC         0x02  /* Lithuanian/Classic */
#define USB_SLNG_MALAY_MALAYSIA             0x01  /* Malay/Malaysia */
#define USB_SLNG_MALAY_BRUNEI_DARUSSALAM    0x02  /* Malay/Brunei Darussalam */
#define USB_SLNG_NORWEGIAN_BOKMAL           0x01  /* Norwegian/Bokmal */
#define USB_SLNG_NORWEGIAN_NYNORSK          0x02  /* Norwegian/Nynorsk */
#define USB_SLNG_PORTUGUESE                 0x02  /* Portuguese */
#define USB_SLNG_PORTUGUESE_BRAZILIAN       0x01  /* Portuguese/Brazilian */
#define USB_SLNG_SERBIAN_LATIN              0x02  /* Serbian/Latin */
#define USB_SLNG_SERBIAN_CYRILLIC           0x03  /* Serbian/Cyrillic */
#define USB_SLNG_SPANISH                    0x01  /* Spanish/Castilian */
#define USB_SLNG_SPANISH_MEXICAN            0x02  /* Spanish/Mexican */
#define USB_SLNG_SPANISH_MODERN             0x03  /* Spanish/Modern */
#define USB_SLNG_SPANISH_GUATEMALA          0x04  /* Spanish/Guatemala */
#define USB_SLNG_SPANISH_COSTA_RICA         0x05  /* Spanish/Costa Rica */
#define USB_SLNG_SPANISH_PANAMA             0x06  /* Spanish/Panama */
#define USB_SLNG_SPANISH_DOMINICAN_REPUBLIC 0x07  /* Spanish/Dominican Rep. */
#define USB_SLNG_SPANISH_VENEZUELA          0x08  /* Spanish/Venezuela */
#define USB_SLNG_SPANISH_COLOMBIA           0x09  /* Spanish/Colombia */
#define USB_SLNG_SPANISH_PERU               0x0a  /* Spanish/Peru */
#define USB_SLNG_SPANISH_ARGENTINA          0x0b  /* Spanish/Argentina */
#define USB_SLNG_SPANISH_ECUADOR            0x0c  /* Spanish/Ecuador */
#define USB_SLNG_SPANISH_CHILE              0x0d  /* Spanish/Chile */
#define USB_SLNG_SPANISH_URUGUAY            0x0e  /* Spanish/Uruguay */
#define USB_SLNG_SPANISH_PARAGUAY           0x0f  /* Spanish/Paraguay */
#define USB_SLNG_SPANISH_BOLIVIA            0x10  /* Spanish/Bolivia */
#define USB_SLNG_SPANISH_EL_SALVADOR        0x11  /* Spanish/El Salvador */
#define USB_SLNG_SPANISH_HONDURAS           0x12  /* Spanish/Honduras */
#define USB_SLNG_SPANISH_NICARAGUA          0x13  /* Spanish/Nicaragua */
#define USB_SLNG_SPANISH_PUERTO_RICO        0x14  /* Spanish/Puerto Rico */
#define USB_SLNG_SWEDISH                    0x01  /* Swedish */
#define USB_SLNG_SWEDISH_FINLAND            0x02  /* Swedish/Finland */

    /*
     * USB Language defaults
     * ---------------------
     *
     *  usb_lang defines the language to use to get string codes from
     * USB devices. By default it is set to US English.
     *
     *  FIXME: make sysctl/proc settable
     */

extern __u16 usb_lang;

    /*
     * Default language table to convert UNICODE to. If NULL, then we
     * keep on UNICODE. 
     */

extern struct nls_table* usb_nls;

#endif /* _usb_nls_h_ */

/* Log:
 *
 * $Log: usb-nls.h,v $
 * Revision 1.1  1998/11/14 00:38:27  inaky
 * Isolated all the header files
 *
 * Revision 1.1  1998/05/01 19:12:20  inaky
 * Added the language file and some help acks ...
 *
 *
 */
