
/* USB Driver Stack v$Revision: 1.1 $ 
 *
 * (C) 1997 by Iaky Prez Gonzlez <inaky@peloncho.fis.ucm.es>
 *
 * Distributed under the terms of the GNU Public License Version 2
 * See files COPYRIGHT and COPYING at the root distribution directory
 *
 * $Id: usb-skeld.h,v 1.1 1998/11/14 00:38:27 inaky Exp $
 *
 * Changelog at the bottom 
 *
 *
 *     Skeleton driver
 *     ---------------
 *
 *  Sample driver for testing and reference. Note the joke :)
 *
 */

#ifndef _usb_skeld_h_
#define _usb_skeld_h_

#include <uusbd/usb-stddesc.h>    /* Standard USB descriptors */


    /* 
     * For each driver, you define some data for it, as the names, et
     * al and then depending on which type of driver you'll use private
     * class, subclass, protocol or vendor specific data.
     *
     *  Thus, the structure of this file is (should be :) -Note that
     * for SI we denote an specific item, wether it is class, subclass,
     * protocol or vendor related-:
     *
     *  Names
     *  SI specific declarations
     *      Data structures types (DST)
     *      DST housekeeping functions
     *      SI specific variables
     *      SI specific housekeeping functions
     *      SI Device [un]configuration functions
     *  Driver structure
     *  Initialization/Shutdown routines
     *
     *  Normally you'll have different structures for class, subclass,
     * protocol, vendor and/or device stuff, and thus, different
     * lists, variables, etc ... though probably interrelated. Thus,
     * you may repeat the above structure.
     *
     *  Note that for using the lists code provided by usb-slist.h,
     * you MUST set as first vairable of your structures a ts_snode. 
     * Take a look at the hub driver for a sample implementation.
     */



    /* 
     * Class' stuff
     *
     *
     * This example is only done for class stuff. It is expandable to
     * subclass, protocol, vendor ... any kind of stuff.
     */


    /* Class structure */

typedef struct s_skeld_class ts_skeld_class;
struct s_skeld_class    /* structure */
{
    ts_snode        snode;
    ts_usb_device   *device;
    
        /* FIXME: your data here */
};

    /* Class structure housekeeping (class-generic.c) */

#ifdef __KERNEL__
#define ts_skeld_class_clean(class)             \
do                                              \
{                                               \
    memset ((class), 0, sizeof (*(class)));     \
    ts_skeld_class_nozero (class);              \
} while (0)

#define ts_skeld_class_nozero(class)            \
do                                              \
{                                               \
    ts_snode_nozero (&(class)->snode);          \
} while (0)

inline
ts_skeld_class* ts_skeld_class_init (void)
{
    ts_skeld_class* class;
    
    gdlf_fntrace ("ts_skeld_class_init()");
    class = usb_kmalloc (sizeof (ts_skeld_class), GFP_KERNEL);
    if (class == NULL)
    {
        gdlf_err ("Cannot allocate space for a "
                  "ts_skeld_class structure\n");
        return NULL;
    }
    ts_skeld_class_clean (class);
    return class;              
}

inline
void ts_skeld_class_release (ts_skeld_class* class)
{
    gdlf_fntrace ("ts_skeld_class_release (0x%p)", class);
    gdlf_aassert (class != NULL, return);
    gdlf_assert (class->snode.dtor.fn == (tf_dtor)skeld_class_destroy);
#warning FIXME: release class stuff
    return;
}

#endif /* __KERNEL__ */

    /* Class specific variables && housekeeping (class-hkeep.c) */

#ifdef __KERNEL__
extern ts_slist skeld_class_list;
extern int skeld_class_register (ts_skeld_class*);
extern int skeld_class_unregister (ts_skeld_class*);
extern int skeld_class_destroy (ts_skeld_class*);

    /* Class interface to configuration (class-cfg.c) */

extern int skeld_class_config (ts_usb_cfel*, ts_usb_device*, 
                               ts_usb_cfg*, t_usb_cfg, 
                               ts_usb_iface*, t_usb_iface, 
                               ts_usb_aset*, t_usb_aset, __u32);
extern int skeld_class_unconfig (ts_usb_device*, unsigned);

    /* Driver housekeeping (driver.c) */

extern ts_usb_driver *skeld_driver;
extern int skeld_init (void);
extern int skeld_release (void);
extern int skeld_shutdown (void);
extern int skeld_class_reference (ts_usb_device*, int);
#endif /* __KERNEL__ */

#endif /* _usb_skeld_h_ */


/* Log:
 *
 * $Log: usb-skeld.h,v $
 * Revision 1.1  1998/11/14 00:38:27  inaky
 * Isolated all the header files
 *
 * Revision 1.11  1998/10/14 21:49:57  inaky
 * Removed all the errno variables, were unused.
 *
 * Revision 1.10  1998/08/29 00:07:47  inaky
 * Actualized.
 *
 * Revision 1.9  1998/08/25 04:07:36  inaky
 * Fixed notes and added destructor declaration.
 *
 * Revision 1.8  1998/07/23 14:39:43  inaky
 * Made all errnos volatile.
 *
 * Revision 1.7  1998/06/20 21:55:35  inaky
 * Some changes, this is getting quite outdated.
 *
 * Revision 1.6  1998/01/08 12:19:20  inaky
 * Added copyright strings to all important files
 *
 * Revision 1.5  1998/01/08 11:50:07  inaky
 * Adapted to the new configuration scheme
 *
 * Revision 1.4  1998/01/02 00:12:20  inaky
 * Finished the new driver scheme. Documented at usbd-drv.c
 *
 * Revision 1.3  1997/12/31 05:37:46  inaky
 * Fixed some craps.
 *
 * Revision 1.2  1997/12/30 22:30:36  inaky
 * Fixed some bugs.
 *
 * Revision 1.1  1997/12/30 20:15:14  inaky
 * Moved to the new lists scheme. Untested yet.
 *
 * Revision 1.5  1997/12/29 00:26:33  inaky
 * Added function traces
 *
 * Revision 1.4  1997/12/27 18:10:39  inaky
 * *** empty log message ***
 *
 * Revision 1.3  1997/12/27 17:25:45  inaky
 * Finished moving to GDLF. Deprecated usb_printk and __LOCATION__
 *
 * Revision 1.2  1997/12/08 02:42:35  inaky
 * Created a basic skeleton for a driver. Still some enhancements missing.
 *
 * Revision 1.1  1997/12/06 01:56:37  inaky
 * Began to create the Skeleton Driver. This one will be the one to use
 * as a sample and guideline, as well as experimentation.
 *
 */

