
/* USB Driver Stack v$Revision: 1.2 $ 
 *
 * (C) 1997 by Iaky Prez Gonzlez <inaky@peloncho.fis.ucm.es>
 *
 * Distributed under the terms of the GNU Public License Version 2
 * See files COPYRIGHT and COPYING at the root distribution directory
 *
 * $Id: usbd.h,v 1.2 1998/11/16 00:01:30 inaky Exp $
 *
 * Changelog at the bottom 
 *
 *
 *     USB Driver API
 *     --------------
 *
 *  Declarations of the functions which make up the USB Driver API.
 * All these functions are publicly usable [I guess :)]; however, not
 * all need to be used, as they call themselves to do dirty
 * work. Use for the most high-level functions. FIXME: doc this.
 
 * NOTE: The authoritative source of documentation for each function
 *       is the source file, not the header file. 
 *
 */

#ifndef _usbd_h_
#define _usbd_h_

#ifdef __KERNEL__

#include <uusbd/usb.h>
#include <uusbd/usb-stddesc.h>
#include <uusbd/usb-hub.h>
#include <uusbd/usb-misc.h>


    /* Misc utility functions */

static inline
t_usb_host usb_host_addr (ts_usb_device* device)
{
    return device->hub->host->addr;
}

static inline
t_usb_addr usb_dev_addr (ts_usb_device* device)
{
    return device->addr;
}

static inline
t_usb_addr usb_endp_addr (ts_usb_endp* endp)
{
    return endp->addr >> 1;
}

static inline
void usb_configuration_ref (ts_usb_cfg* cfg)
{
    atomic_inc (&cfg->usecnt);
}

static inline
void usb_configuration_unref (ts_usb_cfg* cfg)
{
    atomic_dec (&cfg->usecnt);
}

static inline
void usb_device_ref (ts_usb_device* device)
{
    ts_3snode_ref (&device->msnode);
}

static inline
void usb_device_unref (ts_usb_device* device, int* dst)
{
    ts_3snode_unref (&device->msnode, dst);
}

static inline
void usb_device_unref_nd (ts_usb_device* device)
{
    ts_3snode_unref_nd (&device->msnode);
}

static inline
void usb_host_ref (ts_usb_host* host)
{
    ts_snode_ref (&host->snode);
}

static inline
void usb_host_unref (ts_usb_host* host, int* dst)
{
    ts_snode_unref (&host->snode, dst);
}

static inline
void usb_host_unref_nd (ts_usb_host* host)
{
    ts_snode_unref_nd (&host->snode);
}


    /* kernel-interface.c */

extern int usb_init (void);
extern int usb_release (void);
extern int usb_shutdown (void);


    /* usbd.c */

extern int usb_open (const ts_usb_addr*, ts_usb_open*);
extern int usb_close (ts_usb_open*);
extern int usb_dst_check (const ts_usb_open*);

extern int usb_dst_endp_unset (ts_usb_open*);
extern int usb_dst_endp_set (ts_usb_open*, t_usb_endp);

extern int usb_dst_dev_unset (ts_usb_open*);
extern int usb_dst_dev_set (ts_usb_open*, t_usb_addr);

extern int usb_dst_host_unset (ts_usb_open*);
extern int usb_dst_host_set (ts_usb_open*, t_usb_host);

extern int usb_dst_unset (ts_usb_open*);
extern int usb_dst_set (ts_usb_open*, const ts_usb_addr*);

extern int usb_sof_set (ts_usb_device*, unsigned);


    /*  Return the SOF for the given host.
     *
     *
     * Parameters:  
     *
     *  ts_usb_host*    Pointer to the host struct for which you want
     *                  to know the frame length.
     *
     * Returns:
     *
     *  unsigned        Host's frame length
     *
     */

static inline 
unsigned usb_sof_get (ts_usb_host* host)
{
    return host->sof;
}

extern int usb_master_become (ts_usb_device*);
extern int usb_master_drop (ts_usb_device*);


    /* usbd-tr.c */

extern int usb_tr_setup (ts_usb_tr*);
extern int usb_tr_setdn (ts_usb_tr*);
                 
extern int usb_tr_insert (ts_usb_tr*);
extern int usb_tr_remove (ts_usb_tr*);
                 
extern int usb_tr_run (ts_usb_tr*);
extern int usb_tr_abort (ts_usb_tr*);
extern int usb_tr_reset (ts_usb_tr*);

extern int usb_tr_destroy (ts_usb_tr*);
extern void usb_tr_timer (ts_usb_tr*);


    /* usbd-device.c */

extern ts_usb_device* usb_device_ptr_hn (t_usb_host, t_usb_addr);
extern ts_usb_device* usb_device_ptr_hp (ts_usb_host*, t_usb_addr);
extern int usb_device_register (ts_usb_device*);
extern int usb_device_unregister (ts_usb_device*);
extern int usb_device_attach (ts_usb_device*, ts_usb_hub*, __u8, __u32);
extern int usb_device_detach (ts_usb_device*, __u32);
extern int usb_device_destroy (ts_usb_device*);


    /* 
     * usbd-cfg.c 
     *
     * usb_device_config()
     * usb_device_unconfig()
     * usb_configuration_set()
     * usb_configuration_get()
     * usb_configuration_best()
     *
     *
     * Flags passed to the configuration routines
     * ------------------------------------------
     *
     *  These indicate events which happened and forced the
     * configuration system to tell the drivers to
     * configure/unconfigure the device.
     */

#define USB_CFG_ATTACH      0x00000001 /* Device attachment if == 1 */
#define USB_CFG_DETACH      0x00000002 /* Device detachment if == 1 */

    /* 
     * Aggregate device type. If (flags | USB_CFG_AGTYPE) is 0, then
     * the device is an aggregate device with independant
     * interfaces [so there's no device cfel]. If 1, it has dependant
     * interfaces [and there's device cfel]. 
     */

#define USB_CFG_AGTYPE      0x00000004 /* Aggregate device type */
#define USB_CFG_RESET       0x00000008 /* Device is being resetted */
#define USB_CFG_RELEASE     0x00000010 /* Driver is being released */
#define USB_CFG_CHANGE      0x00000020 /* USB configuration change */

#define USB_CFG_HUB_PWR_DN  0x00000040 /* Port powered off */
#define USB_CFG_HUB_OVR_CUR 0x00000080 /* Port/hub-wide overcurrent */
#define USB_CFG_HUB_DESTROY 0x00000100 /* Hub is being destroyed */

#define USB_CFG_HOST_RESET  0x00000200 /* Host is going to be reset */
#define USB_CFG_HOST_RM     0x00000400 /* Host is being removed */
#define USB_CFG_DESTROY     0x00000800 /* Device structure destruction */


extern int usb_device_config (ts_usb_device*, __u32);
extern int usb_device_unconfig (ts_usb_device*, __u32);
extern ts_usb_cfg* usb_configuration_best (ts_usb_device*);
extern int usb_configuration_set (ts_usb_device*, t_usb_cfg, int);
extern int usb_cfg_pendps_resize (ts_usb_cfg* , t_usb_endp);


    /* Return the active configuration of a device.
     *
     * Parameters:  
     *
     *  ts_usb_device*  Pointer to the device structure.
     *
     * Returns:
     *
     *  ts_usb_cfg*     Pointer to the active configuration on the
     *                  device or NULL if the device is not
     *                  configured.
     *
     * Global:
     *
     *  int usb_errno   On error, error code.  
     */

static inline
ts_usb_cfg* usb_configuration_get (ts_usb_device* device)
{
        /* FIXME: do consistency check, put it as usb_iface_get() */
    return device->active;
}


    /* 
     * usbd-ops.c 
     *
     * usb_device_reset()
     * usb_device_feature_clear()
     * usb_device_feature_set()
     * usb_device_status_get()
     * usb_device_address_set()
     * usb_synch_frame()
     */

extern int usb_request_dataless (ts_usb_device*, ts_usb_request*, 
                                 const char*);
extern ssize_t usb_request_data (ts_usb_device*, ts_usb_request*, 
                                 void*, const char*);
extern char* usb_string_get (ts_usb_device*, __u8, int);
extern int usb_device_reset (ts_usb_device*);

    /* INTERNAL FUNCTION, do _not_ use */

extern int usb_device_feature (ts_usb_device*, e_usb_featsel, e_usb_req);

    /* Clear a feature on a device
     *
     * Parameters:  
     *
     *  ts_usb_device*  Pointer to the device whose feature we want to
     *                  clear.
     *
     * Returns:
     *
     *  int             0 if ok, errno code < 0 on error.
     *
     * Global:
     *
     *  int usb_errno   On error, error code.  
     */

static inline
int usb_device_feature_clear (ts_usb_device* device, e_usb_featsel feature)
{
    return usb_device_feature (device, feature, e_usb_req_clear_feature);
}


    /* Set a device feature
     *
     * Compliance:      USB Spec 1.0 xxxx
     *
     * Parameters:  
     *
     *  ts_usb_device*  Pointer to the device whose feature we want to
     *                  clear.
     *
     * Returns:
     *
     *  int             0 if ok, errno code < 0 on error.
     *
     * Global:
     *
     *  int usb_errno   On error, error code.  
     */

static inline
int usb_device_feature_set (ts_usb_device* device, e_usb_featsel feature)
{
    return usb_device_feature (device, feature, e_usb_req_set_feature);
}

extern __s32 usb_device_status_get (ts_usb_device*);
extern int usb_device_address_set (ts_usb_device*, t_usb_addr);


    /* 
     * usbd-iface.c 
     *   
     * usb_iface_feature_set()
     * usb_iface_feature_clear()
     * usb_iface_status_get()
     * usb_iface_status_get()
     * usb_iface_set()
     * usb_iface_get()
     */

    /* internal, do NOT use, use usb_iface_feature_*() instead */
extern int usb_iface_feature (ts_usb_device*, t_usb_iface, 
                              e_usb_featsel, e_usb_req);


    /* Clear a feature on an interface
     *
     *  This call will clear a feature on an interface, thus,
     * eliminating the need to make a control request, etc ...
     *
     *  The only restriction is you can only set/clear features on the
     * ACTIVE alternate setting of an interface which is in the ACTIVE
     * USB configuration. This is an USB restriction, not an UUSBD's
     * one. 
     *
     * Parameters:  
     *
     *  ts_usb_device*  Pointer to the device structure where the
     *                  interface is located.
     *
     *  t_usb_iface     Interface number.
     *
     * Returns:
     *
     *  int             0 if ok, errno code < 0 on error.
     *
     * Global:
     *
     *  int usb_errno   On error, error code.  
     */

static inline
int usb_iface_feature_clear (ts_usb_device* device, t_usb_iface ifacen, 
                             e_usb_featsel feature)
{
    return usb_iface_feature (device, ifacen, feature, 
                              e_usb_req_clear_feature);
}


    /* Set a feature on an interface
     *
     *  Same considerations as usb_iface_feature_clear().
     *
     * Parameters:  
     *
     *  ts_usb_device*  Pointer to the device structure where the
     *                  interface is located.
     *
     *  t_usb_iface     Interface number.
     *
     * Returns:
     *
     *  int             0 if ok, errno code < 0 on error.
     *
     * Global:
     *
     *  int usb_errno   On error, error code.  
     */

static inline
int usb_iface_feature_set (ts_usb_device* device, t_usb_iface ifacen, 
                           e_usb_featsel feature)
{
    return usb_iface_feature (device, ifacen, feature, 
                              e_usb_req_set_feature);
}

/*
extern __s32 usb_iface_status_get (ts_usb_device*, t_usb_iface);
*/
extern int usb_iface_set (ts_usb_device*, t_usb_iface, t_usb_aset);
extern int usb_iface_get (ts_usb_device*, t_usb_iface);
extern int usb_iface_pendp_set (ts_usb_cfg* , t_usb_iface);
extern void usb_iface_pendp_unset (ts_usb_cfg* , t_usb_iface);


    /*
     * usbd-endp.c 
     *
     * usb_endp_feature_clear()
     * usb_endp_feature_set()
     * usb_endp_status_get()
     * usb_endp_stall_clear()
     */

    /* internal, do NOT use, use usb_endp_feature_*() instead */
extern int usb_endp_feature (ts_usb_device*, t_usb_endp, 
                              e_usb_featsel, e_usb_req);


    /* Clear a feature on an endp
     *
     *  This call will clear a feature on an endp, thus,
     * eliminating the need to make a control request, etc ...
     *
     *  The only restriction is you can only set/clear features on the
     * enpoints of the active ACTIVE USB configuration. This is an USB
     * restriction, not an UUSBD's one. 
     *
     * Parameters:  
     *
     *  ts_usb_device*  Pointer to the device structure where the
     *                  endpoint is located.
     *
     *  t_usb_endp      Endpoint number.
     *
     * Returns:
     *
     *  int             0 if ok, errno code < 0 on error.
     *
     * Global:
     *
     *  int usb_errno   On error, error code.  
     */

static inline
int usb_endp_feature_clear (ts_usb_open* open, e_usb_featsel feature)
{
    return usb_endp_feature (open->dev, open->endp->addr, feature, 
                             e_usb_req_clear_feature);
}


    /* Set a feature on an endpoint
     *
     *  Same considerations as usb_endp_feature_clear().
     *
     * Parameters:  
     *
     *  ts_usb_open*    Open descriptor to the endpoint whose
     *                  feature's we want to clear. 
     *
     *  t_usb_endp     Endpoint number.
     *
     * Returns:
     *
     *  int             0 if ok, errno code < 0 on error.
     *
     * Global:
     *
     *  int usb_errno   On error, error code.  
     */

static inline
int usb_endp_feature_set (ts_usb_open* open, e_usb_featsel feature)
{
    return usb_endp_feature (open->dev, open->endp->addr, feature, 
                             e_usb_req_set_feature);
}


extern __s32 usb_endp_status_get (ts_usb_open*);
extern __s32 usb_synch_frame (ts_usb_open*);


    /* Unstall an stalled pipe (endpoint)
     *
     *  Note that when an endpoint is stalled, all the operations
     * (but unstalling) on it return -EREMOTEIO.
     *
     * Parameters:  
     *
     *  ts_usb_open*    Open descriptor of the endpoint whose stall
     *                  condition we want to clear.
     *
     * Returns:
     *
     *  int             0 if ok, errno code < 0 on error.
     *
     * Global:
     *
     *  int usb_errno   On error, error code.  
     */

static inline
int usb_endp_stall_clear (ts_usb_open* open)
{
    return usb_endp_feature_clear (open, e_usb_fes_endp_stall);
}



    /* usbd-descr.c */

extern int usb_descriptor_fill (ts_usb_device*);
extern ssize_t usb_descriptor_get (ts_usb_device*, e_usb_desc, 
                                   __u8, __u16, void*, size_t);
extern ssize_t usb_descriptor_set (ts_usb_device*, e_usb_desc, 
                                   __u8, __u16, const void*, size_t);


    /*  Resume/suspend a USB device.
     *
     *  This function is used to force a USB device to suspend or to
     * resume operation.
     *
     *  Note that, when resuming, we must ask for resume, then wait
     * for it to complete and then check if it completed [to be
     * safe]. We have to wait, as USB Spec 1.0 states, 20 ms of resume
     * signaling and 3 ms of after EOP
     *
     * Parameters:  
     *
     *  ts_usb_device*  Device structure of the device we wish to
     *                  suspend/resume.
     *
     * Returns:
     *
     *  int             0 if ok, errno code < 0 on error.
     *
     * Global:
     *
     *  int usb_errno   On error, error code.  
     */

static inline 
int usb_device_resume (ts_usb_device* device)
{
    int errcode;
    int cnt = USB_PORT_RESET_RETRIES;
    
    errcode = usb_port_feature_clear (device->hub, device->port,
                                      e_usb_port_fes_suspend);
    current->state = TASK_INTERRUPTIBLE;
    schedule_timeout (HZ /*1/s*/ * 20 /*ms*/ * 1 /*s*/ / 100 /*ms*/);
    while (cnt--)
    {
        usb_waitms (5);
        errcode = usb_port_status_get (device->hub, device->port);
        if (errcode >= 0 && !device->hub->port[device->port].suspend)
            return 0;
    }
    return -EIO;
}

static inline 
int usb_device_suspend (ts_usb_device* device)
{
    return usb_port_feature_set (device->hub, device->port, 
                                 e_usb_port_fes_suspend);
}


    /* usbd-host.c */

extern int usb_host_reset (ts_usb_host*);
extern int usb_host_register (ts_usb_host*);
extern int usb_host_unregister (ts_usb_host*);
extern int usb_host_destroy (ts_usb_host*);
extern int usb_host_register_device (ts_usb_device*);
extern int usb_host_unregister_device (ts_usb_device*);
extern int usb_host_free_address (ts_usb_host*);
extern ts_usb_host* usb_host_ptr (t_usb_addr);
extern ts_usb_host* usb_host_hci (ts_snode*);


    /* usbd-drv.c */

static inline
int usb_drv_vicsp (ts_usb_driver* driver)
{
    return
        (driver->vendor == 0)? 0 : 1 << 4
        | (driver->product == 0? 0 : 1) << 3
        | (driver->class == 0? 0 : 1) << 2
        | (driver->subclass == 0? 0 : 1) << 1
        | (driver->protocol == 0? 0 : 1);
} 

extern int usb_drv_register (ts_usb_driver*);
extern int usb_drv_release (ts_usb_driver*);
extern void usb_drv_release_use (ts_usb_driver*, ts_usb_drv_use*, ts_usb_drv_use*);
extern int usb_drv_reference (ts_usb_driver*, ts_snode*, int);
extern ts_usb_driver* usb_drv_load (int, __u16, __u16, ts_usb_cfel*);
extern ts_usb_driver* usb_drv_reload (ts_usb_driver*);
extern int usb_drv_unregister (ts_usb_driver*);
extern int usb_drv_destroy (ts_usb_driver*);
extern int usb_drv_match (ts_usb_driver**, __u16, __u16, 
                          ts_usb_cfel*, ts_usb_driver*);
extern ts_usb_driver* usb_drv_find (__u16, __u16, ts_usb_cfel*);

#endif /* __KERNEL__ */

#endif /* _usbd_h_ */

/* Log:
 *
 * $Log: usbd.h,v $
 * Revision 1.2  1998/11/16 00:01:30  inaky
 * Added the function to look for a loaded driver which matches _exactly_
 * the vicsp codes.
 *
 * Revision 1.1  1998/11/14 00:38:27  inaky
 * Isolated all the header files
 *
 * Revision 1.40  1998/11/09 21:32:23  inaky
 * Moved all device ref/unref to usb_device_[un]ref(). Moved all host
 * ref/unref to usb_host_[un]ref(). Changed the name of the configuration
 * pointer array to pcfg.
 *
 * Revision 1.39  1998/10/31 02:14:41  inaky
 * Added a method to get real endpoint address from the one at
 * endp->addr.
 *
 * Revision 1.38  1998/10/22 00:04:20  inaky
 * Fixed missed prototypes ... it didn't show up or I missed it?
 *
 * Revision 1.37  1998/09/24 15:57:59  inaky
 * Moved 'hub_port' to 'port'.
 *
 * Revision 1.36  1998/09/14 02:35:36  inaky
 * Fixed declarations.
 *
 * Revision 1.35  1998/08/30 13:52:47  inaky
 * Added endpoint pointers actualization routines.
 *
 * Revision 1.34  1998/08/28 23:48:21  inaky
 * Fixed declaration of usb_drv_match(). Now it works ok.
 *
 * Revision 1.33  1998/08/27 22:18:03  inaky
 * Added timers to be controlled from USBD.
 *
 * Revision 1.32  1998/08/25 04:04:50  inaky
 * Added some inlines to make things easier and hide some facts, also a
 * vicsp mask calculator.
 *
 * Revision 1.31  1998/08/24 02:28:37  inaky
 * Added a parameter to usb_configuration_set() to evitate double
 * configuration (usb_device_config()) calls when attaching.
 *
 * Changed name of all enumerations to be prefixed with the enumeration
 * type name.
 *
 * Revision 1.30  1998/08/03 13:36:00  inaky
 * Added the declaration of usb_host_hci()
 *
 * Revision 1.29  1998/07/24 15:37:00  inaky
 * Removed the PAUSED state, it was quite useless ...
 *
 * Revision 1.28  1998/07/23 14:36:39  inaky
 * Added new methods.
 *
 * Revision 1.27  1998/07/16 14:14:10  inaky
 * Moved from the old read/write interface to the new, request-based
 * interface and fixed some interfaces.
 *
 * Revision 1.26  1998/07/14 22:05:23  inaky
 * Tried to fix some of the resume code ... not good. Have to rework it
 * almost all.
 *
 * Revision 1.25  1998/06/21 20:15:45  inaky
 * Added missing declarations.
 *
 * Revision 1.24  1998/06/20 21:54:55  inaky
 * Massive changes to split the old monolithic hubd driver into a hub
 * manager (hub wrappers and daemon) and a dynamic driver, which can be
 * replaced and its different parts reused.
 *
 * Revision 1.23  1998/05/01 19:11:46  inaky
 * Added the extern declaration for the default USB language for
 * selection.
 *
 * Revision 1.22  1998/04/26 03:37:53  inaky
 * Added the declarations for string getting.
 *
 * Revision 1.21  1998/04/13 22:17:09  inaky
 * Fixed prototypes for updated functions, docs; added new bits for the
 * USB_CFG_* macros.
 *
 * Revision 1.20  1998/02/12 16:35:11  inaky
 * Finished the device ops, and moved synch_frame getting to usbd-endp,
 * where it belongs. Fixed some stuff here and there, and starting to get
 * worried on locks ... I'm gonna have to debug *lots* of deadlocks.
 *
 * Revision 1.19  1998/02/12 14:40:48  inaky
 * Added the method for driver reload.
 *
 * Revision 1.18  1998/02/10 17:38:28  inaky
 * *** empty log message ***
 *
 * Revision 1.17  1998/01/11 15:44:21  inaky
 * Finished functions from the descriptor, inteface and endpoint modules,
 * fixed docs, pretty printed and removed craps.
 *
 * Revision 1.16  1998/01/08 12:19:13  inaky
 * Added copyright strings to all important files
 *
 * Revision 1.15  1998/01/08 11:49:13  inaky
 * Finishing the configuration system. Only the final cfel-stuff is
 * missing to implement; however, nothing is tested.
 *
 * Revision 1.14  1998/01/03 00:13:51  inaky
 * Updated the usb_open function
 *
 * Revision 1.13  1998/01/02 19:39:38  inaky
 * Added the definition for endpoint zero to the device structure and
 * removed it from the list of endpoints kept at each configuration. Also
 * updated it on places where needed and removed the concept of basic
 * configuration, as is no longer needed.
 *
 * Revision 1.12  1998/01/02 00:12:18  inaky
 * Finished the new driver scheme. Documented at usbd-drv.c
 *
 * Revision 1.11  1997/12/31 05:37:01  inaky
 * Almost finished all the lists and life cycle for hosts, drivers and
 * devices. Some craps to be fixed yet.
 *
 * Revision 1.10  1997/12/30 20:15:45  inaky
 * Moved everything to the new lists scheme. Untested yet.
 *
 * Revision 1.9  1997/12/27 20:06:28  inaky
 * Finished fixinf the __KERNEL__ #ifdef wrapping and moved the ts_dtor.h
 * to usb-dtor.h
 *
 * Revision 1.8  1997/12/27 19:54:22  inaky
 * Added all the prototypes and skeleton definitions (incomplete) of all
 * of the USBD functions to each source file.
 *
 * Revision 1.7  1997/12/27 02:24:05  inaky
 * Moving everything to use the GDLF for logging and debug checkings.
 * Removed the global array of USB hosts and moved it to a linked
 * list. Added an address field to each USB host controller.
 * Removed lots of cruft that were accumulated and useless. Still some
 * missing, but may be useful someday.
 *
 * Revision 1.6  1997/12/22 02:44:33  inaky
 * Trying to move all the documentation to the source files from the
 * header files, as it was duplicated.
 *
 * Revision 1.5  1997/12/06 01:55:49  inaky
 * Now finishing the configuration system
 *
 * Revision 1.4  1997/12/06 00:19:42  inaky
 * Almost finished the usbd/host, usbd/cfg and a lot of meat from
 * usbd/device. Also added the driver definition. Got to polish it yet
 * ...
 *
 * Revision 1.3  1997/11/29 20:21:49  inaky
 * Documented and almost fixes the read/write/abort API, documented a
 * damn lot and implemented some stuff
 *
 * Revision 1.2  1997/11/29 00:46:31  inaky
 * Documented almost all the hub basic API, added spin locks to
 * usb_host_register() and some more stuff. Going for more.
 *
 * Revision 1.1  1997/10/02 21:29:04  inaky
 * Remade all the directory structure to make it cleaner when the USB
 * class drivers come.
 *
 * Revision 1.1  1997/02/22 22:25:44  inaky
 * *** empty log message ***
 *
 *
 */
