
/* USB Driver Stack v$Revision: 1.21 $ 
 *
 * (C) 1997 by Iaky Prez Gonzlez <inaky@peloncho.fis.ucm.es>
 *
 * Distributed under the terms of the GNU Public License Version 2
 * See files COPYRIGHT and COPYING at the root distribution directory
 *
 * $Id: misc.c,v 1.21 1998/11/14 17:14:59 inaky Exp $
 *
 * Changelog at the bottom 
 *
 *
 *     Miscellaneous functions
 *     -----------------------
 *
 * These functions do miscellaneous stuff ...
 */

#include <linux/ioport.h>   /* request_region() */
#include <linux/sched.h>    /* free_irq() */
#include <asm/io.h>

#include <uusbd/usb.h>
#include <uusbd/usb-misc.h>

#define GDLF_IDENT          "USB-MISC"
#define GDLF_LOCAL          /* Define if want local settings */
#define GDLF_DEF_LOGLEVEL   9
#define GDLF_ASSERTIONS     /* want assertions (#undef if not) */
#undef GDLF_FNTRACE        /* want function traces */
#define GDLF_LOCATIONS      /* want locations in log messages */
#include <uusbd/gdlf.h>


    /* 256 long log2 table */

__u8 log2table[256] =
{
    255, /* 0 */ 
    0, /* 1 */ 
    1, 1, /* 3 */ 
    2, 2, 2, 2, /* 7 */ 
    3, 3, 3, 3, 3, 3, 3, 3, /* 15 */ 
    4, 4, 4, 4, 4, 4, 4, 4, 4, 4, 4, 4, 4, 4, 4, 4, /* 31 */ 

    5, 5, 5, 5, 5, 5, 5, 5, 5, 5, 5, 5, 5, 5, 5, 5, 5, 5, 5, /* 50 */ 
    5, 5, 5, 5, 5, 5, 5, 5, 5, 5, 5, 5, 5, /* 63 */ 

    6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, /* 82 */ 
    6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, /* 101 */ 
    6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, /* 120 */ 
    6, 6, 6, 6, 6, 6, 6, /* 127 */ 

    7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, /* 146 */ 
    7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, /* 165 */ 
    7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, /* 184 */ 
    7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, /* 203 */ 
    7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, /* 222 */ 
    7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, /* 241 */ 
    7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7 /* 255 */ 
};


    /* Do a memory dump showing the hex values of bytes.
     *
     * Parameters:  
     *
     *  const char*     Pointer to zone to dump in kernel-land.
     *
     *  size_t          Size of the zone to dump.
     */

void usb_dump_bytes (const __u8* data, size_t size)
{
    const __u8* top;
    
    gdlf_fntrace ("usb_dump_bytes (0x%p, %u)", data, size);
    
    gdlf_aassert (data != NULL, return);
    gdlf_aassert (size != 0, return);

        /* Round size to next 8's multiple */

#warning FIXME: make print only size bytes ...
    if (size & 0x7)
        size = (size & ~0x7) + 0x8;
    top = data + size;
    
    while (data < top)
    {
        gdlf_debug ("0x%p: %02x %02x %02x %02x  %02x %02x %02x %02x\n",
                    data,
                    (unsigned)data[0], (unsigned)data[1],
                    (unsigned)data[2], (unsigned)data[3],
                    (unsigned)data[4], (unsigned)data[5],
                    (unsigned)data[6], (unsigned)data[7]);
        data += 8;
    }
    return;
}


    /* Seek a range of IO ports free to use
     *
     * Parameters:  
     *  unsigned    IO address to start at
     *  unsigned    Range of IO addresses to search
     *
     * Returns:
     *  unsigned    Address of free range. 0 if not found.
     */

#define IO_TOP  0x10000  /* FIXME: this is weird! Nowhere defined? */

unsigned usb_seek_iorange (unsigned beg, unsigned range)
{
    unsigned top = IO_TOP - range;

    gdlf_fntrace ("usb_seek_iorange (%u, %u)", beg, range);

    beg--;
    while (beg++ < top &&
           !check_region (beg, range));
    return beg < top? beg : 0;
}


    /* Seek a free IRQ (in non-shared mode)
     *
     * Parameters:  
     *  unsigned    IRQ number to begin with
     *
     * Returns:
     *  unsigned    0 if none found, else IRQ number
     */

void usb_intr_dummy (int irq, void* dev_id, struct pt_regs* regs)
{
    return;
}

#define IRQ_TOP 16    /* FIXME: Nasty! Nowhere defined? */

unsigned usb_seek_irq (unsigned beg)
{
    gdlf_fntrace ("usb_seek_irq (%u)", beg);

    beg--;
    while (beg++ < IRQ_TOP
           && request_irq (beg, usb_intr_dummy, 0,
                           "IRQ seek", &beg));

    if (beg == IRQ_TOP)         /* if not found ... */
        return 0;               /* ... just return 0 */
    free_irq (beg, &beg);       /* else free ... */
    return beg;                 /* ... and return it */
}


    /* Read/write from/to one of the IO ports of a USB host controller.
     *
     * Parameters:  
     *
     *  unsigned int    Index of the host controller in the host
     *                  controller array. 
     *
     *  unsigned int    Offset from the base IO port of the host
     *                  controller where it has to be written.
     *
     *  __u{8,16,32}    Value to write (applicable if mask != 0).
     *
     *  __u{8,16,32}    Mask for the value to write. No writes will be
     *                  done if zero.
     *
     * Returns:     
     *
     *  __u{8,16,32}    Value readen from the register before writing
     *                  the new one (if wrote). 
     */

__u8  usb_bits8  (unsigned int,  __u8,  __u8);
__u16 usb_bits16 (unsigned int, __u16, __u16);
__u32 usb_bits32 (unsigned int, __u32, __u32);


    /* Generate a unique and reproducible 32 bit signature from a data
     * buffer.
     *
     *  This function will take a data buffer and generate from it a
     * signature using a simple sum of byte values and lower counter
     * byte. algorithm. FIXME: perhaps it's needed something more
     * `serious'. However, not this is not intented as to be something
     * unbreakable, but just to distinguish.
     *
     *  Note we use a counter instead of an iteration pointer, as if
     * we used the pointer we could be in risk of having it differently
     * aligned on each call, what would cause to generate different
     * signatures for same data blocks.
     *
     * Parameters:  
     *
     *  const void*     Pointer to the data buffer.
     *  size_t          Length of the data buffer.
     *
     * Returns:
     *
     *  __u32           Signature. If pointer is NULL or size is zero,
     *                  0 will be returned. If all the data is zero,
     *                  the return value may not be zero.  
     */

__u32 usb_signature (const char* data, ssize_t len)
{
    __u32 signature = 0;
    ssize_t cnt;
    
    gdlf_fntrace ("usb_signature (0x%p, %ud)", data, len);

    if (data == NULL || len == 0)
        return 0;
    
    for (cnt = 0; cnt < len; cnt++)
    {
            /* 
             * Add to signature the actual byte of data, shifted to a
             * byte position (0, 1, 2, 3) which is computed from the
             * actual byte's position in the data buffer by getting
             * it's remainder of 4 (to get on which position we want
             * to put it) and multiplying it by 8 (shitf left 3) to
             * get the bit displacement.
             */

        signature += ((__u32)data[cnt]) << ((cnt % 4) << 3);

            /*
             * Now add, in the same fashion, to each byte, but in a
             * different order, the counter position.
             */

        signature += ((__u32)cnt&&0xFF) << ((3 - (cnt %4)) << 3);
    }
    return signature;
}


/* Log:
 *
 * $Log: misc.c,v $
 * Revision 1.21  1998/11/14 17:14:59  inaky
 * Moved the include files to uusbd/include/uusbd, to ease integration
 * into the Linux kernel.
 *
 * Revision 1.20  1998/10/31 02:08:27  inaky
 * Added a table for logarithm of 2 for a max of 255 numbers.
 *
 * Revision 1.19  1998/08/27 22:14:06  inaky
 * Removed traces.
 *
 * Revision 1.18  1998/08/24 02:16:34  inaky
 * Added a memory dumping function.
 *
 * Revision 1.17  1998/06/20 21:54:50  inaky
 * Massive changes to split the old monolithic hubd driver into a hub
 * manager (hub wrappers and daemon) and a dynamic driver, which can be
 * replaced and its different parts reused.
 *
 * Revision 1.16  1998/04/26 03:24:24  inaky
 * Added a fixme wishlist
 *
 * Revision 1.15  1998/03/28 19:25:20  inaky
 * Removed inclusion of usb-methods.h
 *
 * Revision 1.14  1998/02/12 12:36:09  inaky
 * Added usb_signature()
 *
 * Revision 1.13  1998/02/10 17:16:11  inaky
 * Added GDLF stuff
 *
 * Revision 1.12  1998/01/08 12:19:07  inaky
 * Added copyright strings to all important files
 *
 * Revision 1.11  1997/12/27 02:24:03  inaky
 * Moving everything to use the GDLF for logging and debug checkings.
 * Removed the global array of USB hosts and moved it to a linked
 * list. Added an address field to each USB host controller.
 * Removed lots of cruft that were accumulated and useless. Still some
 * missing, but may be useful someday.
 *
 * Revision 1.10  1997/12/22 02:44:29  inaky
 * Trying to move all the documentation to the source files from the
 * header files, as it was duplicated.
 *
 * Revision 1.9  1997/12/08 02:40:58  inaky
 * Fixed some stuff of the #includes, modified the makefile targets to be
 * more flexible, tried to finish the device configuration section.
 *
 * Revision 1.8  1997/10/02 22:29:37  inaky
 * Finally managed to understand the workings of the MODVERSIONS (ok, not
 * exactly the working, but how to make it work).
 *
 * Now I can go to sleep happy :) -- I know, I'm really sick --
 *
 * Revision 1.7  1997/09/01 19:14:42  inaky
 * Backup check in
 *
 * Revision 1.6  1997/07/03 20:00:02  inaky
 * Moved all the hw dependencies to the HCD module, modified the
 * controller detections stuff to use just PCI means (okay, other buses
 * to come, but when they come). General fixes, but no really new deep
 * enhancements.
 *
 * Revision 1.5  1997/06/19 20:38:17  inaky
 * Moving al hardware dependant stuff to the HCD modules
 *
 * Revision 1.4  1997/04/06 21:14:32  inaky
 * Still with the definition of the access interface. More or less it is
 * taking an stable form.
 *
 * Revision 1.3  1997/03/14 20:47:30  inaky
 * Fixing auto-detection code. Still fails when the PCI USB device
 * isn't enabled on BIOS ??? Why???
 *
 * Revision 1.2  1997/03/09 23:10:31  inaky
 * *** empty log message ***
 *
 * Revision 1.1  1997/02/26 22:29:09  inaky
 * Miscellaneous code
 *
 * Revision 1.2  1997/02/24 22:29:50  inaky
 * Hmmm ... I'm getting trouble with the IO ports. I guess I'm writing to
 * the bad places. Moved the PCI initialization code to a function (to
 * make it easier to read; on the other hand, it begins to seem Spaghetti
 * Basic :).
 *
 * Revision 1.1  1997/02/22 22:25:44  inaky
 * *** empty log message ***
 *
 */
