
/* USB Mass Storage Devices Driver v$Revision: 1.3 $ 
 *
 * (C) 1998 by Iaky Prez Gonzlez <inaky@peloncho.fis.ucm.es>
 *
 * Distributed under the terms of the GNU Public License Version 2
 * See files COPYRIGHT and COPYING at the root distribution directory
 *
 * $Id: class-hkeep.c,v 1.3 1998/11/14 17:22:48 inaky Exp $
 *
 * Changelog at the bottom 
 *
 *
 *     Mass Storage Driver - Class structures housekeeping
 *     ---------------------------------------------------
 */

#include <linux/module.h>   /* MOD_*_USE_COUNT */
#include <asm/spinlock.h>   /* locks */

#include <uusbd/usbd.h>

#define GDLF_IDENT          "USB-MSDD"
#define GDLF_LOCAL          /* Define if want local settings */
#define GDLF_DEF_LOGLEVEL   9
#define GDLF_ASSERTIONS     /* want assertions (#undef if not) */
#define GDLF_FNTRACE        /* want function traces */
#define GDLF_LOCATIONS      /* want locations in log messages */
#include <uusbd/gdlf.h>

#include <uusbd/usb-msdd.h>


ts_slist msdd_list;         /* list of registered class structures */


    /* Unregister an msdd element
     *
     * This is the destructor function.
     *
     * Parameters:  
     *
     *  ts_msdd*        Class element to unregister
     *
     * Returns:
     *
     *  int             0 if ok, errno code < 0 on error.
     */

int msdd_unregister (ts_msdd* class)
{
    gdlf_fntrace ("msdd_unregister (0x%p)", class);
    gdlf_aassert (class != NULL, return -EINVAL);
    gdlf_assert (class->snode.dtor.fn == (tf_dtor)msdd_destroy);

        /* FIXME: do your class' unregistration stuff */
#warning FIXME: class stuff not implemented!

        /* Let's get it removed from the list */

    ts_slist_del (&msdd_list, &class->snode);
    ts_msdd_release (class);        /* clean the struct */
#warning FIXME: remove this, it makes modules stay for ever
    MOD_DEC_USE_COUNT;              /* we ain't using the module */
    return 0;                       /* bye */
}


    /* FIXME: Detailed desc
     *
     * Compliance:      USB Spec 1.0 xxxx
     *
     * Parameters:  
     *
     *  type            descr
     *
     * Global:
     *
     *  type name       descr
     *
     * Returns:
     *
     *  type            descr
     *
     * Global:
     *
     *  type name       descr
     */

int msdd_destroy (ts_msdd* class)
{
    t_usb_cfg cfg_cnt;
    ts_usb_cfg *cfg;
    ts_usb_iface *iface, *iface_top;
    ts_usb_aset *aset, *aset_top;
    
    gdlf_fntrace ("msdd_destroy (0x%p)", class);
    gdlf_aassert (class != NULL, return -EINVAL);
    gdlf_assert (class->snode.dtor.fn == (tf_dtor)msdd_destroy);
    gdlf_aassert (class->device, return -EINVAL);
    gdlf_aassert (class->device->pcfg, return -EINVAL);
    
    ts_snode_ref (&class->snode);

#warning FIXME: do your class destruction stuff

        /* 
         *  Ok, walk the cfgs, ifaces and aset, searching for an
         * element configured for this class. If found, unref class and
         * keep on. 
         */

    for (cfg_cnt = 0; cfg_cnt < class->device->pcfgs; cfg_cnt++)
    {
        cfg = class->device->pcfg[cfg_cnt];
        if (cfg == NULL)        /* unloaded? skip */
            continue;
        
            /* Walk cfg's interfaces */

        for (iface = cfg->iface, iface_top = iface + cfg->ifaces; 
             iface < iface_top; iface++)
        {
                /* Walk the iface's asets */

            for (aset = iface->aset, aset_top = aset + iface->asets; 
                 aset < aset_top; aset++)

                      /* cfged by us? */

#warning WARNIG: as a sample, we ID the struct with the configured field

                if (aset->cfel.configured == (unsigned long)&class->snode)
                {
                    ts_snode_unref (&class->snode, NULL);
                    aset->cfel.configured = 0;      /* uncfged */
                }
        }         
    }

    msdd_unregister (class);    /* unregister */
    ts_snode_unref_nd (&class->snode);
    ts_msdd_release (class);    /* clean the struct */
    return 0;                   /* bye */
}


    /* Register an msdd element
     *
     * Parameters:  
     *
     *  ts_msdd* Class element to register
     *
     * Returns:
     *
     *  int             0 if ok, errno code < 0 on error.
     */

int msdd_register (ts_msdd* class)
{
    int errcode;
    
    gdlf_fntrace ("msdd_register (0x%p)", class);
    gdlf_aassert (class != NULL, return -EINVAL);

        /* FIXME: do your class' stuff */
#warning FIXME: class stuff not implemented!

    ts_dtor_fill (&class->snode.dtor, (tf_dtor)msdd_destroy);

        /* get into the list */

    errcode = ts_slist_prep (&msdd_list, &class->snode);
    if (errcode)
        return errcode;
#warning FIXME: remove this, it makes modules stay for ever
    MOD_INC_USE_COUNT;          /* request we're using the module */
    return 0;                   /* bye */
}

/* Log:
 *
 * $Log: class-hkeep.c,v $
 * Revision 1.3  1998/11/14 17:22:48  inaky
 * Moved the include files to uusbd/include/uusbd, to ease integration
 * into the Linux kernel.
 *
 * Revision 1.2  1998/11/09 21:38:28  inaky
 * Fixed inclussion of usb-msdd.h to let it use GDLF. Changed the name of
 * the configuration pointer array to pcfg.
 *
 * Revision 1.1  1998/10/14 21:48:50  inaky
 * Beginnings of a driver ...
 *
 */

