
/* USB Mass Storage Devices Driver v$Revision: 1.3 $ 
 *
 * (C) 1998 by Iaky Prez Gonzlez <inaky@peloncho.fis.ucm.es>
 *
 * Distributed under the terms of the GNU Public License Version 2
 * See files COPYRIGHT and COPYING at the root distribution directory
 *
 * $Id: driver.c,v 1.3 1998/11/14 17:22:49 inaky Exp $
 *
 * Changelog at the bottom 
 *
 *
 *     Mass Storage Driver - Driver housekeeping
 *     -----------------------------------------
 */

#include <linux/init.h>     /* __init */
#define __NO_VERSION__
#include <linux/module.h>   /* MOD_*_USE_COUNT */
#include <asm/spinlock.h>   /* locks */

#include <uusbd/usbd.h>

#define GDLF_IDENT          "USB-MSDD"
#define GDLF_LOCAL          /* Define if want local settings */
#define GDLF_DEF_LOGLEVEL   9
#define GDLF_ASSERTIONS     /* want assertions (#undef if not) */
#define GDLF_FNTRACE        /* want function traces */
#define GDLF_LOCATIONS      /* want locations in log messages */
#include <uusbd/gdlf.h>

#include <uusbd/usb-msdd.h>


atomic_t usecnt;

#define NAME   /* short!! */                    \
    "Mass Storage Devices Driver v$Revision: 1.3 $ "

#define COPYRIGHT                               \
    "(c) 1998 Inaky Perez Gonzalez "            \
    "<inaky@peloncho.fis.ucm.es>"

#define VENDOR 0
#define VENDORNAME NULL

#define PRODUCT 0
#define DEVICENAME NULL

#define CLASS 0x8
#define CLASSNAME                               \
    "Mass Storage Devices"

#define SUBCLASS 0
#define SUBCLASSNAME NULL

#define PROTOCOL 0
#define PROTOCOLNAME NULL

ts_usb_driver* msdd_driver = NULL;


    /* 
     * Linux module support
     */

#ifdef MODULE

int init_module (void)
{
    return msdd_init();
}

void cleanup_module (void)
{
    msdd_release();
    msdd_shutdown();
    return;
}

#endif


    /* Initialize the msdd driver
     *
     * Returns:
     *
     *  int             0 if ok, errno code < 0 on error.
     */

int __init msdd_init (void)
{
    int errcode;
    
    gdlf_fntrace ("msdd_init()");
    gdlf_info (NAME "\n");
#ifdef MODULE
    gdlf_info (COPYRIGHT "\n");
#endif

    msdd_driver = ts_usb_driver_init();
    if (msdd_driver == NULL)
    {
        usb_errno = -USBE_NO_MEMORY;
        return -ENOMEM;
    }

        /* Set driver's attributes */

    msdd_driver->vendor = VENDOR;
    msdd_driver->product = PRODUCT;
    msdd_driver->class = CLASS;
    msdd_driver->subclass = SUBCLASS;
    msdd_driver->protocol = PROTOCOL;

        /* Set info */

    msdd_driver->name = NAME;                  /* driver name */
    msdd_driver->copyright = COPYRIGHT;        /* driver copyright */
    msdd_driver->vendorname = VENDORNAME;      /* device's vendor name */
    msdd_driver->productname = DEVICENAME;     /* device's name */
    msdd_driver->classname = CLASSNAME;        /* class' name */
    msdd_driver->subclassname = SUBCLASSNAME;  /* subclass' name */
    msdd_driver->protocolname = PROTOCOLNAME;  /* protocol name */

    msdd_driver->config = msdd_config;          /* config method */
    msdd_driver->unconfig = msdd_unconfig;      /* unconfig method */
    msdd_driver->reference = msdd_reference;    /* unconfig method */
    ts_snode_ref (&msdd_driver->snode);         /* we hold a reference to it */
    
    ts_slist_clean (&msdd_list);                /* initialize the list */
    errcode = usb_drv_register (msdd_driver);   /* register driver */
    if (errcode)
    {
        gdlf_err ("Cannot register driver "
                  "(errno/usb = %d/%d)\n",
                  errcode, usb_errno);
        if (msdd_driver->snode.next == NULL)
        {
            ts_usb_driver_clean (msdd_driver);
            usb_kfree (msdd_driver);
        }
        return errcode;
    }
        
        /* FIXME: do you class' stuf */
#warning FIXME: do class stuff!

    return 0;
}


    /* Release the msdd driver
     *
     *  Do what is needed to ensure the use count of the driver is
     * zero and can be unloaded from the kernel.
     *
     * Returns:
     *
     *  int             0 if ok, errno code < 0 on error.
     */

int msdd_release (void)
{
    gdlf_fntrace ("msdd_release()");

#warning FIXME: msdd_release() not implemented!

    return -ENOSYS;
}


    /* Shutdown the skel driver
     *
     * Returns:
     *
     *  int             0 if ok, errno code < 0 on error.
     */

int msdd_shutdown (void)
{
    gdlf_fntrace ("msdd_shutdown()");

        /* FIXME: do you class' stuf */

    ts_snode_unref (&msdd_driver->snode, NULL);
    return usb_drv_unregister (msdd_driver);
}


    /* FIXME: Detailed desc
     *
     *  The 'reference' method is used when we take any of the driver's
     * code or data structures to use it. If so, we call it with a
     * positive argument (one reference, regardless the value of the
     * _positive_ number). When we are done, we release the reference
     * calling it with a negative number. It always returns the number
     * of references or < 0 on error. Calling it with an argument of
     * zero will just return the number of references. 
     * Compliance:      USB Spec 1.0 xxxx
     *
     * Parameters:  
     *
     *  type            descr
     *
     * Global:
     *
     *  type name       descr
     *
     * Returns:
     *
     *  type            descr
     *
     * Global:
     *
     *  type name       descr
     */

int msdd_reference (ts_usb_device* device, int n)
{
    gdlf_fntrace ("msdd_reference (0x%p, %d)", device, n);
    gdlf_aassert (device != NULL, return -EINVAL);

#warning FIXME: add more stuff, keep the device ptr ...

    if (n > 0)
    {
        atomic_inc (&usecnt);
        MOD_INC_USE_COUNT;
    }
    else if (n < 0)
    {
        atomic_dec (&usecnt);
        MOD_DEC_USE_COUNT;
    }
    return atomic_read (&usecnt);
}


/* Log:
 *
 * $Log: driver.c,v $
 * Revision 1.3  1998/11/14 17:22:49  inaky
 * Moved the include files to uusbd/include/uusbd, to ease integration
 * into the Linux kernel.
 *
 * Revision 1.2  1998/11/09 21:38:52  inaky
 * Fixed inclusion of usb-msdd.h to let it use GDLF. Fixed names.
 *
 * Revision 1.1  1998/10/14 21:48:50  inaky
 * Beginnings of a driver ...
 *
 */

