
/* USB Driver Stack v$Revision: 1.3 $ 
 *
 * (C) 1997 by Iaky Prez Gonzlez <inaky@peloncho.fis.ucm.es>
 *
 * Distributed under the terms of the GNU Public License Version 2
 * See files COPYRIGHT and COPYING at the root distribution directory
 *
 * $Id: irq-handler.c,v 1.3 1998/11/16 00:03:17 inaky Exp $
 *
 * Changelog at the bottom 
 *
 *
 *     OHCI stuff - IRQ handling routines
 *     ----------------------------------
 */

#include <uusbd/usbd.h>

#define GDLF_IDENT          "USB-OHCI"
#define GDLF_LOCAL          /* Define if want local settings */
#define GDLF_DEF_LOGLEVEL   9
#define GDLF_ASSERTIONS     /* want assertions (#undef if not) */
#define GDLF_FNTRACE        /* want function traces */
#define GDLF_LOCATIONS      /* want locations in log messages */
#include <uusbd/gdlf.h>

#include "ohci.h"           /* this has to be include after GDLF */


    /* FIXME: Detailed desc
     *
     *
     *
     * Parameters:  
     *
     *  ts_ohci_host*   Pointer to OHCI host.
     *
     */

static inline
void cook_done_head (ts_ohci_host* hci)
{
    gdlf_fntrace ("cook_done_head (0x%p)", hci);
    gdlf_aassert (hci != NULL, return);
#warning FIXME: cook_done_head not implemented
    return;
}


    /* FIXME: Detailed desc
     *
     *  Note we ignore some change bits (IGNORE_BITS), as we don't
     * need them. In case they are the only ones set, we don't poll
     * the hub.
     *
     * Parameters:  
     *
     *  ts_ohci_host*   Pointer to OHCI host.
     *
     */

#define IGNORE_BITS (OHCI_RHPSTAT_PRSC)

static inline
void cook_root_hub (ts_ohci_host* hci)
{
    __u8 cnt;
#ifdef OHCI_RHSC_BUG
    __u8 ports;
#endif
    int do_poll = 0;
    
    gdlf_fntrace ("cook_root_hub (0x%p)", hci);
    gdlf_aassert (hci != NULL, return);
    gdlf_aassert (hci->host != NULL, return);
#ifdef OHCI_RHSC_BUG
    gdlf_aassert (hci->rhpstatus != NULL, return);
#endif

#ifdef OHCI_RHSC_BUG
        /*
         *  Ok, the engineer who did this should be shot :) right in
         * the forehead :].
         *
         *  The thing is the RHSC status bit of the OHCI controller
         * IRQ status register should only be set _if_ [and fucking
         * only if] any of the bits in the HcRhStatus or
         * HcRhPortStatus registers change [OHCI 1.0, 7.1.4] but, _at
         * least_ on my CMD670r4 [debug mode disabled], it keeps that
         * bit set after ack'ing it if any status change bits are
         * set. Smells like wired all the change bits trough an OR
         * gate to the IRQ RHSC.
         *
         *  Sooooo ... the workaround: save'em all, put the change
         * bits to zero, ack the RHSC, and use the saved ones. Thus,
         * before accessing them, refresh and use. Ack to the real
         * ones. 
         *
         *  We gotta do it fast, to minimize the chance to have them
         * changing their state.
         */

    ports = hci->host->root.ports;

        /* save rh status and ports status */

    hci->rhstatus = ~0xffff & le_readl (hci->regs.rhub_status);
    le_writel_mask (hci->rhstatus, OHCI_RHSTAT_OCIC, OHCI_RHSTAT_OCIC,
                    hci->regs.rhub_status);
    do_poll = hci->rhstatus;
    if (do_poll)    
        hci->tr_buffer |= 0x1;
    
    for (cnt = 0; cnt < ports; cnt++)
    {
        hci->rhpstatus[cnt] = ~0xffff & 
            le_readl (hci->regs.rhub_pstatus_base
                      + cnt * sizeof (hci->regs.rhub_pstatus_base));
        le_writel_mask (hci->rhpstatus[cnt], 
                        OHCI_RHPSTAT_CSC | OHCI_RHPSTAT_PESC
                        | OHCI_RHPSTAT_PSSC | OHCI_RHPSTAT_OCIC 
                        | OHCI_RHPSTAT_PRSC, 
                        OHCI_RHPSTAT_CSC | OHCI_RHPSTAT_PESC
                        | OHCI_RHPSTAT_PSSC | OHCI_RHPSTAT_OCIC 
                        | OHCI_RHPSTAT_PRSC,
                        hci->regs.rhub_pstatus_base
                        + cnt * sizeof (hci->regs.rhub_pstatus_base));
        if (hci->rhpstatus[cnt] & ~IGNORE_BITS)
        {
            do_poll = 1;
            hci->tr_buffer |= 0x2 << cnt;
        }
    }
 
#else /* OHCI_RHSC_BUG */
    do_poll = ~0xffff & le_readl (hci->regs.rhub_status);
    
    for (cnt = 0; cnt < ports; cnt++)
        do_poll |= ~0xffff & ~IGNORE_BITS & 
            le_readl (hci->regs.rhub_pstatus_base
                      + cnt * sizeof (hci->regs.rhub_pstatus_base));
#endif /* OHCI_RHSC_BUG */

        /* Now ack the RHSC bit */

    le_writel (OHCI_IRQSTAT_RHSC, hci->regs.irq_status);

    if (do_poll)
        usb_hub_status_poll (&hci->host->root);
    return;
}


    /* FIXME: Detailed desc
     *
     * Parameters:  
     *
     *  ts_ohci_host*   Pointer to OHCI host.
     *
     */

static inline
void cook_controller (ts_ohci_host* hci)
{
    __u32 status;
    
    gdlf_fntrace ("cook_controller (0x%p)", hci);
    gdlf_aassert (hci != NULL, return);

    status = le_readl (hci->regs.irq_status);

        /* cook states */

    if (status & OHCI_IRQSTAT_SO)
    {
        gdlf_err ("Host 0x%p (%d): Scheduling overrun\n",
                  hci->host, hci->host->addr);
        hci->errors++;
    }

        /* OHCI_IRQSTAT_WDH is handled by ohci_irqhandler() */

    if (status & OHCI_IRQSTAT_RD)
    {
        gdlf_err ("Host 0x%p (%d): Resume received\n",
                  hci->host, hci->host->addr);
#warning FIXME: send an APM wake up event?
    }

    if (status & OHCI_IRQSTAT_UE)
    {
        gdlf_err ("Host 0x%p (%d): Unrecoverable error\n",
                  hci->host, hci->host->addr);
#warning FIXME: take action, retire offending TDs
    }

    if (status & OHCI_IRQSTAT_RHSC)
        cook_root_hub (hci);
    
    le_writel (~0, hci->regs.irq_status);   /* Acknowledge */
    return;
}


    /* FIXME: Detailed desc
     *
     * Parameters:  
     *
     *  ts_ohci_host*   Pointer to OHCI host.
     */

void ohci_irqhandler (int irq, ts_usb_host* host, struct pt_regs* regs)
{
#warning DEBUG: remove me
    static int counter;
    ts_ohci_host *hci;
    
    gdlf_fntrace ("ohci_irqhandler (0x%x, 0x%p, 0x%p)", 
                  irq, host, regs);
    gdlf_aassert (host != NULL, return);
    gdlf_aassert (host->hci != NULL, return);
    
    hci = OHCI (host->hci);
    gdlf_assert (irq == hci->irq);  /* paranoia? */    
    gdlf_aassert (hci->hcca != NULL, return);
    gdlf_aassert (hci->host == host, return);

    le_writel (OHCI_IRQDIS_MIE, hci->regs.irq_disable); /* disable IRQs */

#warning DEBUG: remove me
    if (counter++ > 4)
        return;

    write_lock (&hci->snode.lock);
#ifdef OHCI_DEBUG
    atomic_inc (&hci->sem);
    if (atomic_read (&hci->sem) > 1)
    {
        gdlf_debug ("Missed IRQ, still busy!!\n");
        atomic_dec (&hci->sem);
        write_unlock (&hci->snode.lock);
        le_writel (OHCI_IRQEN_MIE, hci->regs.irq_enable);
        return;
    }
#endif

#warning DEBUG: remove me
    gdlf_debug ("IRQ, status 0x%x\n", le_readl (hci->regs.irq_status));
    
        /* check source and cook it */

    if (hci->hcca->done_head)
    {
        if (hci->hcca->done_head & 0x1)
            cook_controller (hci);
        cook_done_head (hci);
    }
    else
        cook_controller (hci);

    gdlf_debug ("IRQ, status after cooking 0x%x\n", le_readl (hci->regs.irq_status));

#ifdef OHCI_DEBUG
    atomic_dec (&hci->sem);
#endif
    write_unlock (&hci->snode.lock);
    le_writel (OHCI_IRQEN_MIE, hci->regs.irq_enable);   /* re-enable */
    return;
}


/*
 * $Log: irq-handler.c,v $
 * Revision 1.3  1998/11/16 00:03:17  inaky
 * Moved the include files to uusbd/include/uusbd, to ease integration
 * into the Linux kernel.
 *
 * Revision 1.2  1998/10/22 00:12:21  inaky
 * Added a silly comment.
 *
 * Revision 1.1  1998/09/14 20:31:12  inaky
 * Added all the code for OHCI, a lot missing to complete yet.
 *
 */
