
/* USB Driver Stack v$Revision: 1.2 $ 
 *
 * (C) 1997 by Iaky Prez Gonzlez <inaky@peloncho.fis.ucm.es>
 *
 * Distributed under the terms of the GNU Public License Version 2
 * See files COPYRIGHT and COPYING at the root distribution directory
 *
 * $Id: operations.c,v 1.2 1998/11/16 00:04:34 inaky Exp $
 *
 * Changelog at the bottom 
 *
 *
 *     OHCI stuff - Driver and host initialization
 *     -------------------------------------------
 */


#include <uusbd/usb.h>
#include <uusbd/usb-hub.h>

#define GDLF_IDENT          "USB-OHCI"
#define GDLF_LOCAL          /* Define if want local settings */
#define GDLF_DEF_LOGLEVEL   9
#define GDLF_ASSERTIONS     /* want assertions (#undef if not) */
#define GDLF_FNTRACE        /* want function traces */
#define GDLF_LOCATIONS      /* want locations in log messages */
#include <uusbd/gdlf.h>

#include "ohci.h"


    /* FIXME: Detailed desc
     *
     * Compliance:      USB Spec 1.0 xxxx
     *
     * Parameters:  
     *
     *  type            descr
     *
     * Global:
     *
     *  type name       descr
     *
     * Returns:
     *
     *  type            descr
     *
     * Global:
     *
     *  type name       descr
     */

void ts_ohci_regs_printk (ts_ohci_regs* regs)
{
#ifdef OHCI_DEBUG
    int cnt;
    __u8 ports;
    
    gdlf_aassert (regs != NULL, return);

    ports = le_readl (regs->rhub_descr_a) & OHCI_RHDA_NDP;
    gdlf_debug ("ts_ohci_regs 0x%p (values)\n", regs);
    gdlf_debug ("  revision:          0x%x\n", le_readl (regs->revision));
    gdlf_debug ("  control:           0x%x\n", le_readl (regs->control));
    gdlf_debug ("  cmd_status:        0x%x\n", le_readl (regs->cmd_status));
    gdlf_debug ("  irq_status:        0x%x\n", le_readl (regs->irq_status));
    gdlf_debug ("  irq_enable:        0x%x\n", le_readl (regs->irq_enable));
    gdlf_debug ("  irq_disable:       0x%x\n", le_readl (regs->irq_disable));
                                                                
    gdlf_debug ("  hcca:              0x%x\n", le_readl (regs->hcca));
    gdlf_debug ("  periodic_cur:      0x%x\n", le_readl (regs->periodic_cur));
    gdlf_debug ("  ctl_head:          0x%x\n", le_readl (regs->ctl_head));
    gdlf_debug ("  ctl_cur:           0x%x\n", le_readl (regs->ctl_cur));
    gdlf_debug ("  bulk_head:         0x%x\n", le_readl (regs->bulk_head));
    gdlf_debug ("  bulk_cur:          0x%x\n", le_readl (regs->bulk_cur));
    gdlf_debug ("  done_head:         0x%x\n", le_readl (regs->done_head));
                                                                
    gdlf_debug ("  fm_interval:       0x%x\n", le_readl (regs->fm_interval));
    gdlf_debug ("  fm_remaining:      0x%x\n", le_readl (regs->fm_remaining));
    gdlf_debug ("  fm_number:         0x%x\n", le_readl (regs->fm_number));
    gdlf_debug ("  periodic_start:    0x%x\n", le_readl (regs->periodic_start));
    gdlf_debug ("  ls_threshold:      0x%x\n", le_readl (regs->ls_threshold));
                                                                
    gdlf_debug ("  rhub_descr_a:      0x%x\n", le_readl (regs->rhub_descr_a));
    gdlf_debug ("  rhub_descr_b:      0x%x\n", le_readl (regs->rhub_descr_b));
    gdlf_debug ("  rhub_status:       0x%x\n", le_readl (regs->rhub_status));
    for (cnt = 0; cnt < ports; cnt++)
        gdlf_debug ("  rhub_pstatus[%02d]:  0x%x\n", cnt, 
                    le_readl (regs->rhub_pstatus_base 
                              + cnt
                              * sizeof (regs->rhub_pstatus_base)));
#endif
}


void ts_ohci_hcca_printk (ts_ohci_hcca* hcca)
{
#ifdef OHCI_DEBUG
    int cnt;

    gdlf_aassert (hcca != NULL, return);

    gdlf_debug ("ts_ohci_hcca 0x%p\n", hcca);
    for (cnt = 0; cnt < OHCI_INTR_SLOTS; cnt++)
        gdlf_debug ("  interrupt[%02d]: 0x%x (0x%p)\n", cnt, 
                    hcca->interrupt[cnt], ptr2kvoid (hcca->interrupt[cnt]));
    gdlf_debug ("  frame:       0x%04u\n", (unsigned) hcca->frame);
    gdlf_debug ("  done_head:   0x%x (0x%p)\n", hcca->done_head, 
                hcca->done_head? ptr2kvoid (hcca->done_head) : NULL);
#endif
}


void ts_ohci_ed_printk (ts_ohci_ed* ed)
{
#ifdef OHCI_DEBUG

    gdlf_aassert (ed != NULL, return);

    gdlf_debug ("ts_ohci_ed 0x%p\n", ed);
#endif
}


void ts_ohci_host_printk (ts_ohci_host* hci)
{
#ifdef OHCI_DEBUG
    gdlf_aassert (hci != NULL, return);

    gdlf_debug ("ts_ohci_host 0x%p\n", hci);
    ts_snode_printk (&hci->snode);
    gdlf_debug ("  membase:   0x%08lx\n", hci->membase);
    gdlf_debug ("  irq:       %d\n", hci->irq);
    gdlf_debug ("  host:      0x%p\n", hci->host);

    gdlf_debug ("  pci:       0x%p\n", hci->pci);

    gdlf_debug ("  sem:       %d\n", atomic_read (&hci->sem));
    gdlf_debug ("  errors:    %u\n", hci->errors);

    ts_ohci_regs_printk (&hci->regs);
    ts_ohci_hcca_printk (hci->hcca);
    ts_ohci_ed_printk (hci->iso_head);
    gdlf_debug ("  tr_buffer: 0x%x\n", hci->tr_buffer);

#endif /* OHCI_DEBUG */
}


    /* FIXME: Detailed desc
     *
     * Compliance:      USB Spec 1.0 xxxx
     *
     * Parameters:  
     *
     *  type            descr
     *
     * Global:
     *
     *  type name       descr
     *
     * Returns:
     *
     *  type            descr
     *
     * Global:
     *
     *  type name       descr
     */

int ohci_setsof (ts_usb_host* host, unsigned fl)
{
    gdlf_fntrace ("ohci_setsof (0x%p, %u)", host, fl);
#warning FIXME: ohci_setsof() not implemented
    return -ENOSYS;
}


    /* FIXME: Detailed desc
     *
     * Compliance:      USB Spec 1.0 xxxx
     *
     * Parameters:  
     *
     *  type            descr
     *
     * Global:
     *
     *  type name       descr
     *
     * Returns:
     *
     *  type            descr
     *
     * Global:
     *
     *  type name       descr
     */

int ohci_reset (ts_usb_host* host)
{
    gdlf_fntrace ("ohci_reset (0x%p)", host);

        /*
         *  Assert cmd_status.OHCI_CMDSTAT_HCR
         *
         * Move to RESET, wait 20ms (USB reset time) and done, return
         * to operational? Register values are ignored and preserved.
         */

#warning FIXME: ohci_reset() not implemented
    return -ENOSYS;
}


/*
 * $Log: operations.c,v $
 * Revision 1.2  1998/11/16 00:04:34  inaky
 * Moved the include files to uusbd/include/uusbd, to ease integration
 * into the Linux kernel.
 *
 * Revision 1.1  1998/09/14 20:31:13  inaky
 * Added all the code for OHCI, a lot missing to complete yet.
 *
 */
