
/* USB Driver Stack v$Revision: 1.3 $ 
 *
 * (C) 1997 by Iaky Prez Gonzlez <inaky@peloncho.fis.ucm.es>
 *
 * Distributed under the terms of the GNU Public License Version 2
 * See files COPYRIGHT and COPYING at the root distribution directory
 *
 * $Id: vhub.c,v 1.3 1998/11/16 00:05:25 inaky Exp $
 *
 * Changelog at the bottom 
 *
 *
 *     OHCI stuff - Virtual root hub
 *     -----------------------------
 *
 *  Note _all_ status _change_ bits have to be read from
 * hci->rh*status*. See irq-handler.c:cook_root_hub() for the reason.  
 */


#include <uusbd/usb.h>
#include <uusbd/usb-hub.h>

#define GDLF_IDENT          "USB-OHCI"
#define GDLF_LOCAL          /* Define if want local settings */
#define GDLF_DEF_LOGLEVEL   9
#define GDLF_ASSERTIONS     /* want assertions (#undef if not) */
#define GDLF_FNTRACE        /* want function traces */
#define GDLF_LOCATIONS      /* want locations in log messages */
#include <uusbd/gdlf.h>

#include "ohci.h"


    /* FIXME: Detailed desc
     *
     * Compliance:      USB Spec 1.0 xxxx
     *
     * Parameters:  
     *
     *  type            descr
     *
     * Global:
     *
     *  type name       descr
     *
     * Returns:
     *
     *  type            descr
     *
     * Global:
     *
     *  type name       descr
     */

ssize_t ohci_hub_descriptor_get (ts_usb_hub* hub, __u8 port, 
                                 void* data, size_t size)
{
    gdlf_fntrace ("ohci_hub_descriptor_get (0x%p, %u, 0x%p, %u)",
                  hub, port, data, size);
#warning FIXME: ohci_hub_descriptor_get() not implemented
    return -ENOSYS;
}


    /* Returns hub status
     *
     *  Read the rhub_status port (HcRhStatus) and set up the hub
     * structure's fields, then convert it to hub [chapter 11] format
     * and return it.
     *
     * NOTE: this funcion will update the hub structure's fields
     *       `local_power' and `overcurrent'.
     *
     *  See hubd_status_get() and usb_hub_status_get()
     *
     * Compliance:      USB Spec 1.0 11.12.2.5
     *
     * Parameters:  
     *
     *  ts_usb_hub*     Pointer to the hub structure whose status we
     *                  want to check out.
     *
     * Returns:
     *
     *  __s32           If > 0, status code (and with HUBD_STAT_* to
     *                  get the bits) [NOTE: things may get hairy if
     *                  sometime someone decides to use bit 31 for
     *                  status ...]. On error, <0 errno code.
     */

__s32 ohci_hub_status_get (ts_usb_hub* hub)
{
    __u32 status;
    
    gdlf_fntrace ("ohci_hub_status_get (0x%p)", hub);
    gdlf_aassert (hub != NULL, return -EINVAL);
    gdlf_aassert (hub->host != NULL, return -EINVAL);
    gdlf_aassert (hub->host->hci != NULL, return -EINVAL);
                       
    read_lock (& OHCI (hub->host->hci)->snode.lock);
#ifdef OHCI_RHSC_BUG
    status = OHCI (hub->host->hci)->rhstatus | 
              (0xffff & le_readl (OHCI (hub->host->hci)->regs.rhub_status));
#else
    status = le_readl (OHCI (hub->host->hci)->regs.rhub_status);
#endif
    read_unlock (& OHCI (hub->host->hci)->snode.lock);

        /*
         *  OHCI guys are _so_ nice all fields match as in the
         * get_status request, but bit 15. We clear it. Just in case,
         * we also clear bit 31, though it is suppossed to be readen
         * 0, but it won't cost anything.
         */

    return status & ~OHCI_RHSTAT_DRWE & ~OHCI_RHSTAT_CRWE;
}


    /* Fill a root hub's data from OHCI info
     *
     * Compliance:      USB Spec 1.0 xxxx
     *
     * Parameters:  
     *
     *  type            descr
     *
     * Global:
     *
     *  type name       descr
     *
     * Returns:
     *
     *  type            descr
     *
     * Global:
     *
     *  type name       descr
     */

int ohci_hub_descriptor_fill (ts_usb_hub* hub)
{
    int cnt;
    ts_ohci_host *hci;
    __u32 desca, descb;
    
    gdlf_fntrace ("ohci_hub_descriptor_fill (0x%p)", hub);
    gdlf_aassert (hub != NULL, return -EINVAL);
    gdlf_aassert (hub->host != NULL, return -EINVAL);
    gdlf_aassert (hub->host->hci != NULL, return -EINVAL);

    hci = OHCI (hub->host->hci);
    
        /* device is NULL, we ain't associated with none  */

        /* initialize ports */

    desca = le_readl (hci->regs.rhub_descr_a);
    descb = le_readl (hci->regs.rhub_descr_b);
    hub->ports = desca & OHCI_RHDA_NDP;
    if (hub->ports == 0)
    {
        gdlf_err ("Aieee! 0 ports on root hub 0x%p (%d -1)\n",
                  hub, hub->host->addr);
        return -EINVAL;
    }
    hub->port = ts_usb_port_a_init (hub->ports);
    if (hub->port == NULL)
    {
        ts_usb_hub_release (hub);
        return -ENOMEM;
    }        
#ifdef OHCI_RHSC_BUG
    hci->rhpstatus = usb_kmalloc (hub->ports * sizeof (__u32), GFP_KERNEL);
    if (hci->rhpstatus == NULL)
    {
        gdlf_err ("Cannot allocate memory to save hub state!\n");
        ts_usb_hub_release (hub);
        return -ENOMEM;
    }        
#endif

    for (cnt = 0; cnt < hub->ports; cnt++)
    {
        hub->port[cnt].removable = (descb & 1 << (cnt+1))? 1 : 0;
        hub->port[cnt].pwrctl = (descb & 1 << (cnt+16+1))? 1 : 0;
        le_writel (OHCI_RHPSTAT_PPS, hci->regs.rhub_pstatus_base + 
                   cnt*sizeof (hci->regs.rhub_pstatus_base));
    }

        /* initialize misc hub stuff */

    hub->pwrontime = (desca & OHCI_RHDA_POTPGT) >> 24;
        /* hub->maxcurrent = 0; [Already set] */
        /* hub->compound = 0; [Already set] */
    hub->overc_prot = (desca & OHCI_RHDA_OCPM) >> 11;
    hub->power = (desca & OHCI_RHDA_NPS)? 2 : (desca & OHCI_RHDA_PSM) >> 8;
        /* hub->local_power = 0; [Already set] */
    ohci_hub_status_get (hub);  /* actualize status */
        /* set connect status change to raise an IRQ as resume detected */
#warning FIXME: set this or not?
#if 0
    le_writel (OHCI_RHSTAT_DRWE, hci->regs.rhub_status);
#endif
    return 0;
}


    /* FIXME: Detailed desc
     *
     * Compliance:      USB Spec 1.0 xxxx
     *
     * Parameters:  
     *
     *  type            descr
     *
     * Global:
     *
     *  type name       descr
     *
     * Returns:
     *
     *  type            descr
     *
     * Global:
     *
     *  type name       descr
     */

int ohci_hub_feature (ts_usb_hub* hub, e_usb_hub_featsel feat, 
                      e_usb_req request)
{
    gdlf_fntrace ("ohci_hub_feature (0x%p, %u, %u)",
                  hub, feat, request);
    gdlf_aassert (hub != NULL, return -EINVAL);
    gdlf_aassert (hub->host != NULL, return -EINVAL);
    gdlf_aassert (hub->host->hci != NULL, return -EINVAL);
    
        /* 
         * Only hub features implemented, for the other ones, use the
         * wrappers 
         */  

    switch (feat)
    {
      case e_usb_req_set_feature:
        gdlf_err ("Cannot set hub features (asked hub "
                  "0x%p (%d -1), feature %d, request %d)\n",
                  hub, hub->host->addr, feat, request);
        return -EINVAL;
        
      case e_usb_req_clear_feature:
        switch (request)
        {
          case e_usb_hub_fes_c_local_pwr:
                /* root hub doesn't support this ... */
            break;
            
          case e_usb_hub_fes_c_over_current:
#ifdef OHCI_RHSC_BUG
            OHCI (hub->host->hci)->rhstatus &= ~OHCI_RHSTAT_OCIC;
#else
            le_writel (OHCI_RHSTAT_OCIC, 
                       OHCI (hub->host->hci)->regs.rhub_status);
#endif
            break;

          default:
            gdlf_err ("Bad feature %d to clear on hub 0x%p (%d -1)\n",
                      feat, hub, hub->host->addr);
            return -EINVAL;
        }
        break;
        
      default:
        gdlf_err ("Bad feature request %d to hub 0x%p (%d -1)\n",
                  request, hub, hub->host->addr);
        return -EINVAL;
    }
    return 0;
}


    /* FIXME: Detailed desc
     *
     * Compliance:      USB Spec 1.0 xxxx
     *
     * Parameters:  
     *
     *  type            descr
     *
     * Global:
     *
     *  type name       descr
     *
     * Returns:
     *
     *  type            descr
     *
     * Global:
     *
     *  type name       descr
     */

int ohci_port_feature (ts_usb_hub* hub, __u8 port,
                       e_usb_hub_featsel feat, 
                       e_usb_req request)
{
    int errcode = 0;
#ifdef UHCI_RHSC_BUG
    unsigned long *preg;
#endif
    unsigned long reg;
    ts_ohci_host *hci;
    
    gdlf_fntrace ("ohci_port_feature (0x%p, %u, %u, %u)",
                  hub, port, feat, request);
    gdlf_aassert (hub != NULL, return -EINVAL);
    gdlf_aassert (hub->host != NULL, return -EINVAL);
    gdlf_aassert (hub->host->hci != NULL, return -EINVAL);

    hci = OHCI (hub->host->hci);
#ifdef UHCI_RHSC_BUG
    preg = &hci->rhpstatus[port];
#else
    reg = hci->regs.rhub_pstatus_base
        + port * sizeof (hci->regs.rhub_pstatus_base);
#endif

    if (request != e_usb_req_set_feature
        && request != e_usb_req_clear_feature)
    {
        gdlf_err ("Bad feature request %d to port %u of "
                  "hub 0x%p (%d -1)\n",
                  request, port, hub, hub->host->addr);
        return -EINVAL;
    }

    write_lock (hci->snode.lock);
    switch (feat)
    {
        
            /*
             * NOTE: I can't find any valid use for this as a feature
             *       being set or cleared in the spec. 
             */

      case e_usb_port_fes_connection:
        errcode = -ENOSYS;
        break;
        
            /* Enable or disable a port */

      case e_usb_port_fes_enable:
        switch (request)
        {
          case e_usb_req_clear_feature:
            hub->port[port].enable = 0;
            le_writel (OHCI_RHPSTAT_CCS, reg);
            break;
            
          case e_usb_req_set_feature:
            hub->port[port].enable = 1;
            le_writel (OHCI_RHPSTAT_PES, reg);
            break;
            
          default:
            errcode = -EINVAL;
        }
        break;

            /* Resume or suspend a port */
        
      case e_usb_port_fes_suspend:
        switch (request)
        {
          case e_usb_req_clear_feature:
            hub->port[port].suspend = 0;
            le_writel (OHCI_RHPSTAT_POCI, reg);
            break;
            
          case e_usb_req_set_feature:
            hub->port[port].suspend = 1;
            le_writel (OHCI_RHPSTAT_PSS, reg);
            break;
            
          default:
            errcode = -EINVAL;
        }
        break;
        
            /* Set/clear overcurrent change - OHCI unsupported */

      case e_usb_port_fes_over_current:
        errcode = -ENOSYS;  /* perhaps 0? */
        break;
        
            /* There's an specific function for this */
        
      case e_usb_port_fes_reset:
        errcode = -EPERM;
        break;


            /* Switch power */

      case e_usb_port_fes_power:
        switch (request)
        {
          case e_usb_req_clear_feature:
            hub->port[port].power = 0;
            le_writel (OHCI_RHPSTAT_LSDA, reg);
            break;
            
          case e_usb_req_set_feature:
            hub->port[port].power = 1;
            le_writel (OHCI_RHPSTAT_PPS, reg);
            break;
            
          default:
            errcode = -EINVAL;
        }
        break;
        
            /* NOTE: What's this feature for? */

      case e_usb_port_fes_low_speed:
        errcode = -ENOSYS;
        break;

            /* Ack a connection change */

      case e_usb_port_fes_c_connection:
        switch (request)
        {
          case e_usb_req_clear_feature:
#ifdef UHCI_RHSC_BUG
            (*preg) &= ~OHCI_RHPSTAT_CSC;
#else
            le_writel (OHCI_RHPSTAT_CSC, reg);
#endif
            break;
            
          default:
            gdlf_debug ("Cannot set e_usb_port_fes_c_connection "
                        "on port %u of hub 0x%p (%d -1)\n",
                        port, hub, hub->host->addr);
            errcode = -EINVAL;
        }
        break;
        
            /* Ack a enable/disable change */

      case e_usb_port_fes_c_enable:
        switch (request)
        {
          case e_usb_req_clear_feature:
#ifdef UHCI_RHSC_BUG
            (*preg) &= ~OHCI_RHPSTAT_PESC;
#else
            le_writel (OHCI_RHPSTAT_PESC, reg);
#endif
            break;
            
          default:
            gdlf_debug ("Cannot set e_usb_port_fes_c_enable "
                        "on port %u of hub 0x%p (%d -1)\n",
                        port, hub, hub->host->addr);
            errcode = -EINVAL;
        }
        break;
        
            /* Ack a resume/suspend change */

      case e_usb_port_fes_c_suspend:
        switch (request)
        {
          case e_usb_req_clear_feature:
#ifdef UHCI_RHSC_BUG
            (*preg) &= ~OHCI_RHPSTAT_PSSC;
#else
            le_writel (OHCI_RHPSTAT_PSSC, reg);
#endif
            break;
            
          default:
            gdlf_debug ("Cannot set e_usb_port_fes_c_suspend "
                        "on port %u of hub 0x%p (%d -1)\n",
                        port, hub, hub->host->addr);
            errcode = -EINVAL;
        }
        break;

            /* Ack over-current */

      case e_usb_port_fes_c_over_current:
        switch (request)
        {
          case e_usb_req_clear_feature:
#ifdef UHCI_RHSC_BUG
            (*preg) &= ~OHCI_RHPSTAT_OCIC;
#else
            le_writel (OHCI_RHPSTAT_OCIC, reg);
#endif
            break;
            
          default:
            gdlf_debug ("Cannot set e_usb_port_fes_c_over_current "
                        "on port %u of hub 0x%p (%d -1)\n",
                        port, hub, hub->host->addr);
            errcode = -EINVAL;
        }
        break;

            /* Ack reset */

      case e_usb_port_fes_c_reset:
        switch (request)
        {
          case e_usb_req_clear_feature:
#ifdef UHCI_RHSC_BUG
            (*preg) &= ~OHCI_RHPSTAT_PRSC;
#else
            le_writel (OHCI_RHPSTAT_PRSC, reg);
#endif
            break;
            
          default:
            gdlf_debug ("Cannot clear e_usb_port_fes_c_reset "
                        "on port %u of hub 0x%p (%d -1)\n",
                        port, hub, hub->host->addr);
            errcode = -EINVAL;
        }
        break;

            /* Unknown? */

      default:
        gdlf_err ("Bad feature %d request to port %u of "
                  "hub 0x%p (%d -1)\n",
                  feat, port, hub, hub->host->addr);
        errcode = -EINVAL;
    }
    write_unlock (hci->snode.lock);
    return errcode;
}


    /* Get an OHCI root hub's port status
     *
     * Parameters:  
     *
     *  ts_usb_hub*     Pointer to hub structure
     *
     *  __u8            Port number
     *
     * Returns:
     *
     *  __s32            < 0 on error, else port status in the format
     *                  expected by a USB get_port_state request.
     */

__s32 ohci_port_status_get (ts_usb_hub* hub, __u8 port)
{
    __u32 status;
    
    gdlf_fntrace ("ohci_port_status_get (0x%p, %u)", hub, port);
    gdlf_aassert (hub != NULL, return -EINVAL);

    if (port > hub->ports)
        return -EINVAL;
                       
    read_lock (& OHCI (hub->host->hci)->snode.lock);
    status = OHCI (hub->host->hci)->rhpstatus[port] |
        (le_readl (OHCI (hub->host->hci)->regs.rhub_pstatus_base
                       + port*sizeof (OHCI (hub->host->hci)->regs 
                                      .rhub_pstatus_base))
         & 0xffff);
    read_unlock (& OHCI (hub->host->hci)->snode.lock);

        /*
         *  OHCI guys are _so_ nice all fields match as in the
         * get_port_status request. 
         */

    return status;
}


    /* FIXME: Detailed desc
     *
     * Compliance:      USB Spec 1.0 xxxx
     *
     * Parameters:  
     *
     *  type            descr
     *
     * Global:
     *
     *  type name       descr
     *
     * Returns:
     *
     *  type            descr
     *
     * Global:
     *
     *  type name       descr
     */

#define OHCI_RESET_WAIT_RETRIES 10

int ohci_port_reset (ts_usb_hub* hub, __u8 port)
{
    ts_ohci_host *hci;
    int cnt;
    ts_usb_port *pport;
    __u32 status;
    
    gdlf_fntrace ("ohci_port_reset (0x%p, %u)", hub, port);
    gdlf_aassert (hub != NULL, return -EINVAL);
    gdlf_aassert (hub->host != NULL, return -EINVAL);
    hci = OHCI (hub->host->hci);
    gdlf_aassert (hci != NULL, return -EINVAL);

    if (port > hub->ports)
        return -EINVAL;
                       
    write_lock (&hci->snode.lock);
    pport = &hub->port[port];
    status = 0xffff & le_readl (hci->regs.rhub_pstatus_base);
    le_writel_mask
        (status, OHCI_RHPSTAT_PRS | OHCI_RHPSTAT_PRSC,
         OHCI_RHPSTAT_PRS | OHCI_RHPSTAT_PRSC,
         hci->regs.rhub_pstatus_base 
         + port * sizeof (hci->regs.rhub_pstatus_base));

    cnt = 0;
    while (cnt < OHCI_RESET_WAIT_RETRIES)
    {
        usb_waitms (USB_PORT_RESET_TIME);
        if (hci->rhpstatus[port] & OHCI_RHPSTAT_PRSC)
            break;
        cnt++;
    }

    if (cnt >= OHCI_RESET_WAIT_RETRIES) /* not resetted? */
    {
        gdlf_err ("Error resetting port %u of hub 0x%p (%d -1): "
                  "reset process was interrupted or not completed\n",
                  port, hub, hub->host->addr);
        usb_errno = -USBE_PORT_RESET_INT;
        write_unlock (&hci->snode.lock);
        return -ENOMEDIUM;
    }
    
        /* Now enable the port */

    usb_waitms (USB_PORT_AFTER_RESET_TIME * hub->port[port].try);
    le_writel (OHCI_RHPSTAT_PES, 
               hci->regs.rhub_pstatus_base
               + port*sizeof (hci->regs.rhub_pstatus_base));
    usb_waitms (USB_PORT_AFTER_RESET_TIME * hub->port[port].try);

    write_unlock (&hci->snode.lock);
    return 0;
}


    /* Free a reference to the OHCI hub driver
     *
     *  We do nothing here. When we register the root hubs, we will
     * not increase the OHCI reference count (by calling
     * ohci_reference()), because that way the module is kept at a
     * zero-reference count state _until_ there are devices and they
     * are really used.
     *
     * Parameters:  
     *
     *  ts_usb_hub*     Hub for which we want to release this
     *                  reference. 
     *
     * Returns:
     *
     *  int             0 if ok, errno code < 0 on error.
     */

int ohci_hub_free (ts_usb_hub* hub)
{
    gdlf_fntrace ("ohci_hub_free (0x%p)", hub);
    return 0;
}


    /* hub operations */

ts_usb_hub_ops ohci_hub_operations =
{
    NULL,
    ohci_hub_descriptor_get,
    NULL,
    ohci_hub_descriptor_fill,
    ohci_hub_status_get,
    NULL,
    ohci_hub_feature,
    ohci_port_feature,
    ohci_port_status_get,
    ohci_port_reset,
    ohci_hub_free
};


/*
 * $Log: vhub.c,v $
 * Revision 1.3  1998/11/16 00:05:25  inaky
 * Moved the include files to uusbd/include/uusbd, to ease integration
 * into the Linux kernel.
 *
 * Revision 1.2  1998/09/24 16:10:43  inaky
 * Moved 'hub_port' to 'port'.
 *
 * Revision 1.1  1998/09/14 20:31:14  inaky
 * Added all the code for OHCI, a lot missing to complete yet.
 *
 */
