
/* USB Driver Stack v$Revision: 1.32 $ 
 *
 * (C) 1997 by Iaky Prez Gonzlez <inaky@peloncho.fis.ucm.es>
 *
 * Distributed under the terms of the GNU Public License Version 2
 * See files COPYRIGHT and COPYING at the root distribution directory
 *
 * $Id: pci.c,v 1.32 1998/11/14 17:15:01 inaky Exp $
 *
 * Changelog at the bottom 
 *
 *
 */

#include <linux/config.h>

#ifdef CONFIG_PCI

#include <linux/pci.h>
#include <linux/ioport.h>   /* request_region() */
#include <linux/sched.h>    /* request_irq() */
#include <asm/system.h>

#include <uusbd/usb.h>
#include <uusbd/usb-pci.h>

#define GDLF_IDENT          "USB"
#define GDLF_LOCAL          /* Define if want local settings */
#define GDLF_DEF_LOGLEVEL   9
#define GDLF_ASSERTIONS     /* want assertions (#undef if not) */
#undef  GDLF_FNTRACE        /* don't want function traces */
#define GDLF_LOCATIONS      /* want locations in log messages */
#include <uusbd/gdlf.h>


    /* Set/get bits from a 8 bits PCI register. Will 
     * write back a new value if mask != 0.
     *
     * Parameters:
     *
     *  struct pci_dev* Pointer to PCI device structure.
     *
     *  __u8            Address of the 8 bits register
     *
     *  __u8            New value to write
     *
     *  __u8            Mask of the new value
     *
     * Returns:
     *
     *  __u8            Readen value
     *
     * Global:
     *
     *  int usb_errno   On error, error code.  
     */

__u8 usb_pci_bits8 (struct pci_dev* dev, t_pci_addr addr, __u8 nval, __u8 mask)
{
    __u8 val;
    int retval;
    
    gdlf_fntrace ("usb_pci_bits8 (0x%p, 0x%x, 0x%x, 0x%x)", 
                  dev, addr, nval, mask);

    retval = pci_read_config_byte (dev, addr, &val);
    if (retval != PCIBIOS_SUCCESSFUL)
    {
        gdlf_err ("Couldn't read 8 bits from PCI bus %u "
                  "dev %u fn %u, address 0x%x: %d\n",  
                  dev->bus->number, PCI_SLOT (dev->devfn), 
                  PCI_FUNC (dev->devfn), addr, retval);
        usb_errno = -USBE_PCI_READ_ERROR;
        return 0;
    }

    if (mask != 0)      /* Got to set something? */
    {
        retval = pci_write_config_byte (dev, addr, 
                                        (nval &mask) | (val & ~mask));
        if (retval != PCIBIOS_SUCCESSFUL)
        {
            gdlf_err ("Couldn't write 8 bits to PCI PCI bus %u "
                      "dev %u fn %u, address 0x%x: %d\n",  
                      dev->bus->number, PCI_SLOT (dev->devfn),
                      PCI_FUNC (dev->devfn), addr, retval);
            usb_errno = -USBE_PCI_WRITE_ERROR;
            return 0;
        }
    }    
    return val;
}


    /* Set/get bits from a 16 bits PCI register. Will 
     * write back a new value if mask != 0.
     *
     * Parameters:
     *
     *  struct pci_dev* Pointer to PCI device structure.
     *
     *  __u8            Address of the 16 bits register
     *
     *  __u16           New value to write
     *
     *  __u16           Mask of the new value
     *
     * Returns:
     *
     *  __u16           Readen value
     *
     * Global:
     *
     *  int usb_errno   On error, error code.  
     */

__u16 usb_pci_bits16 (struct pci_dev* dev, 
                      t_pci_addr addr, __u16 nval, __u16 mask)
{
    __u16 val;
    int retval;
    
    gdlf_fntrace ("usb_pci_bits16 (0x%p, 0x%x, 0x%x, 0x%x)", 
                  dev, addr, nval, mask);

    retval = pci_read_config_word (dev, addr, &val);
    if (retval != PCIBIOS_SUCCESSFUL)
    {
        gdlf_err ("Couldn't read 16 bits from PCI bus %u "
                  "dev %d fn %u, address 0x%x: %d\n",  
                  dev->bus->number, PCI_SLOT (dev->devfn),
                  PCI_FUNC (dev->devfn), addr, retval);
        usb_errno = -USBE_PCI_READ_ERROR;
        return 0;
    }

    if (mask != 0)      /* Got to set something? */
    {
        retval = pci_write_config_word (dev, addr, 
                                        (nval &mask) | (val & ~mask));
        if (retval != PCIBIOS_SUCCESSFUL)
        {
            gdlf_err ("Couldn't write 16 bits to PCI bus %u "
                      "dev %u fn %u, address 0x%x: %d\n",  
                      dev->bus->number, PCI_SLOT (dev->devfn),
                      PCI_FUNC (dev->devfn), addr, retval);
            usb_errno = -USBE_PCI_WRITE_ERROR;
            return 0;
        }
    }    
    return val;
}


    /* Set/get bits from a 32 bits PCI register. Will 
     * write back a new value if mask != 0.
     *
     * Parameters:
     *
     *  struct pci_dev* Pointer to PCI device structure.
     *
     *  __u8            Address of the 32 bits register
     *
     *  __u32           New value to write
     *
     *  __u32           Mask of the new value
     *
     * Returns:
     *
     *  __u32           Readen value
     *
     * Global:
     *
     *  int usb_errno   On error, error code.  
     */

__u32 usb_pci_bits32 (struct pci_dev* dev, 
                      t_pci_addr addr, __u32 nval, __u32 mask)
{
    __u32 val;
    int retval;
    
    gdlf_fntrace ("usb_pci_bits32 (0x%p, 0x%x, 0x%x, 0x%x)", 
                  dev, addr, nval, mask);

    retval = pci_read_config_dword (dev, addr, &val);
    if (retval != PCIBIOS_SUCCESSFUL)
    {
        gdlf_err ("Couldn't read 32 bits from PCI bus %u "
                  "dev %u fn %u, address 0x%x: %d\n",  
                  dev->bus->number, PCI_SLOT (dev->devfn), 
                  PCI_FUNC (dev->devfn), addr, retval);
        usb_errno = -USBE_PCI_READ_ERROR;
        return 0;
    }

    if (mask != 0)                  /* Got to set something? */
    {
        retval = pci_write_config_dword (dev, addr, 
                                         (nval &mask) | (val & ~mask));
        if (retval != PCIBIOS_SUCCESSFUL)
        {
            gdlf_err ("Couldn't write 32 bits to PCI bus %u "
                      "dev %u fn %u, address 0x%x: %d\n",  
                      dev->bus->number, PCI_SLOT (dev->devfn),
                      PCI_FUNC (dev->devfn), addr, retval);
            usb_errno = -USBE_PCI_WRITE_ERROR;
            return 0;
        }
    }    
    return val;
}
#endif /* CONFIG_PCI */

/* Log:
 *
 * $Log: pci.c,v $
 * Revision 1.32  1998/11/14 17:15:01  inaky
 * Moved the include files to uusbd/include/uusbd, to ease integration
 * into the Linux kernel.
 *
 * Revision 1.31  1998/09/24 15:51:23  inaky
 * Fixed the PCI error reporting for correct device and function numbers.
 *
 * Revision 1.30  1998/09/14 02:32:01  inaky
 * Fixed some pci error messages, removed the host compliance function.
 *
 * Revision 1.29  1998/07/21 21:34:22  inaky
 * Fixed an omission which prevented the PCI write to be correctly
 * verified in usb_pci_bits*(). Thansks to Olivier Montanuy
 * <olivier.montanuy@cnet.francetelecom.fr> for the advise and fix!
 *
 * Revision 1.28  1998/07/16 14:05:32  inaky
 * Corrected the names for enumeration elements.
 *
 * Revision 1.27  1998/05/01 12:37:19  inaky
 * Moved to use pci_dev structures
 *
 * Revision 1.26  1998/04/26 03:24:56  inaky
 * Semi fixed the pci code, as the pcibios_strerror() is gone.
 *
 * Revision 1.25  1998/04/13 21:51:02  inaky
 * Moved to use the new Linux PCI code. Some fixes are still required.
 *
 * Revision 1.24  1998/01/08 12:19:07  inaky
 * Added copyright strings to all important files
 *
 * Revision 1.23  1997/12/31 16:37:11  inaky
 * Finished the HCI interface
 *
 * Revision 1.22  1997/12/27 13:56:59  inaky
 * Moved to GDLF, fixed docs and removed cruft.
 *
 * Revision 1.21  1997/12/27 02:07:01  inaky
 * Moving everything to use the GDLF for logging and debug checkings.
 * Removed the global array of USB hosts and moved it to a linked
 * list. Added an address field to each USB host controller.
 * Removed lots of cruft that were accumulated and useless. Still some
 * missing, but may be useful someday.
 *
 * Revision 1.20  1997/12/27 02:05:59  inaky
 * Moving everything to use the GDLF for logging and debug checkings.
 * Removed the global array of USB hosts and moved it to a linked
 * list. Added an address field to each USB host controller.
 * Removed lots of cruft that were accumulated and useless. Still some
 * missing, but may be useful someday.
 *
 * Revision 1.19  1997/12/08 02:40:58  inaky
 * Fixed some stuff of the #includes, modified the makefile targets to be
 * more flexible, tried to finish the device configuration section.
 *
 * Revision 1.18  1997/12/06 00:19:39  inaky
 * Almost finished the usbd/host, usbd/cfg and a lot of meat from
 * usbd/device. Also added the driver definition. Got to polish it yet
 * ...
 *
 * Revision 1.17  1997/11/29 20:21:47  inaky
 * Documented and almost fixes the read/write/abort API, documented a
 * damn lot and implemented some stuff
 *
 * Revision 1.16  1997/10/02 22:29:37  inaky
 * Finally managed to understand the workings of the MODVERSIONS (ok, not
 * exactly the working, but how to make it work).
 *
 * Now I can go to sleep happy :) -- I know, I'm really sick --
 *
 * Revision 1.15  1997/10/02 21:29:00  inaky
 * Remade all the directory structure to make it cleaner when the USB
 * class drivers come.
 *
 * Revision 1.14  1997/09/08 10:54:48  inaky
 * Modified the generic functions to gain some performance, also some other checks
 *
 * Revision 1.13  1997/07/03 20:00:03  inaky
 * Moved all the hw dependencies to the HCD module, modified the
 * controller detections stuff to use just PCI means (okay, other buses
 * to come, but when they come). General fixes, but no really new deep
 * enhancements.
 *
 * Revision 1.12  1997/06/22 19:54:31  inaky
 * Seems that the moving of all the hw dependants stuff to the HCI
 * modules is done. Got to test it yet, but at least it compiles cleanly.
 *
 * Revision 1.11  1997/05/03 21:22:49  inaky
 * Adding a lot of code, with the only objetive of adding the first
 * transfer type: control :) Still there's a lot of stuff to do, mostly
 * concerning data structures handling.
 *
 * Revision 1.10  1997/04/10 21:30:49  inaky
 * Fixed the file code, so it opens/closes ok; also reads/writes ok ... ok, just theory
 *
 * Revision 1.9  1997/04/06 21:14:32  inaky
 * Still with the definition of the access interface. More or less it is
 * taking an stable form.
 *
 * Revision 1.8  1997/03/23 20:50:55  inaky
 * Fixed the IRQ stuff. Jochen Karrer
 * <karrer@wpfd25.physik.uni-wuerzburg.de> reported me that setting the
 * irq in the PCI conf space was useless, as the IRQ routing is done by
 * the ISA bridge. As all that stuff should be managed by the BIOS and
 * IANF, by the kernel PnP code, I'll raise an error if IRQ is not
 * configured. IO mapping seems to be ok :)
 *
 * Revision 1.7  1997/03/14 20:47:30  inaky
 * Fixing auto-detection code. Still fails when the PCI USB device
 * isn't enabled on BIOS ??? Why???
 *
 * Revision 1.6  1997/03/14 19:12:45  inaky
 * Tryed to enhance the initialization code to suport autoinitialization
 * ... let's see if it works :)
 *
 * Revision 1.5  1997/03/09 23:10:32  inaky
 * *** empty log message ***
 *
 * Revision 1.4  1997/03/02 22:11:02  inaky
 * *** empty log message ***
 *
 * Revision 1.3  1997/02/26 22:28:36  inaky
 * Some little enchancements ... not too much
 *
 * Revision 1.2  1997/02/24 22:29:50  inaky
 * Hmmm ... I'm getting trouble with the IO ports. I guess I'm writing to
 * the bad places. Moved the PCI initialization code to a function (to
 * make it easier to read; on the other hand, it begins to seem Spaghetti
 * Basic :).
 *
 * Revision 1.1  1997/02/22 22:25:44  inaky
 * *** empty log message ***
 *
 *
 */

