
/* USB Driver Stack v$Revision: 1.4 $ 
 *
 * (C) 1997 by Iaky Prez Gonzlez <inaky@peloncho.fis.ucm.es>
 *
 * Distributed under the terms of the GNU Public License Version 2
 * See files COPYRIGHT and COPYING at the root distribution directory
 *
 * $Id: procfs.c,v 1.4 1998/11/14 17:15:03 inaky Exp $
 *
 * Changelog at the bottom 
 *
 *
 *     Proc FS USB stuff
 *     -----------------
 *
 *  ** Unfinished, untested, unimplemented and unused yet (un* :) **
 * 
 *  This is the code to create the /proc/usb hierarchy. Take a look at
 * the document 'doc/Proc-FS.sgml' for the specifications.
 *
 * Guided by the work on parport_procfs.c of:
 *
 *   David Campbell <campbell@tirian.che.curtin.edu.au>
 *   Tim Waugh <tmw20@cam.ac.uk>
 *   Grant Guenther <grant@torque.net>
 *   Philip Blundell <Philip.Blundell@pobox.com> 
 */

#include <linux/proc_fs.h>

#include <uusbd/usb.h>

#define GDLF_IDENT          "USB-PROC"
#define GDLF_LOCAL          /* Define if want local settings */
#define GDLF_DEF_LOGLEVEL   9
#define GDLF_ASSERTIONS     /* want assertions (#undef if not) */
#define GDLF_FNTRACE        /* want function traces */
#define GDLF_LOCATIONS      /* want locations in log messages */
#include <uusbd/gdlf.h>


#define PROC_USB 40         /* FIXME: should be at proc_fs.h */

struct proc_dir_entry *proc_usb = NULL;
struct proc_dir_entry *proc_stats = NULL;
struct proc_dir_entry *proc_devices = NULL;
struct proc_dir_entry *proc_hosts = NULL;    

    /* Create w/o registering a default proc entry.
     *
     * Parameters:  
     *
     *  struct proc_dir_entry*
     *
     *                  The enw directory entry, dynamically allocated
     *                  or NULL on error (no memory or invalid
     *                  parameters).
     *
     *  const char*     Name for the new proc entry.
     *
     *  mode_t          Mode of the new proc entry (see S_I* at
     *                  linux/stat.h).
     *
     *  unsigned short  Inode number for this entry.
     *
     * Returns:
     *
     *  int             0 if ok, errno code < 0 on error.
     *
     * Global:
     *
     *  int usb_errno   On error, error code.  
     */

int usb_proc_entry_init (struct proc_dir_entry** pentry, const char* name, 
                         mode_t mode, aunsigned short inode)
{
    gdlf_fntrace ("usb_proc_create (\"%s\", %o, 0x%p, %u)", 
                  name, mode, up, inode);
    
    gdlf_apassert (pentry != NULL,
                   { usb_errno = -USBE_PROC_NAME_NULL; return -EINVAL; },
                   "Cannot create a proc entry in a NULL pointer!\n");

    gdlf_apassert (name != NULL,
                   { usb_errno = -USBE_PROC_NAME_NULL; return -EINVAL; },
                   "Cannot create a proc entry with NULL name!\n");

        /* FIXME: should check inode and mode */

	*pentry = usb_kmalloc (sizeof (struct proc_dir_entry), GFP_KERNEL);
	if (*pentry == NULL)
		return -ENOMEM;
	memset (ent, 0, sizeof (struct proc_dir_entry));
	
        /* Set data */

	*pentry->low_ino = inode;
	*pentry->name = name;
	*pentry->namelen = strlen (name);

        /* Set permissions */

	if (mode == S_IFDIR)            /* Directories get a+rx */
    {
		mode |= S_IRUGO | S_IXUGO;
		*pentry->nlink = 2;         /* Set two hard links for directories */
    }
	else if (mode == 0)             /* Regular files get a+r */
    {
		mode = S_IFREG | S_IRUGO;    
		*pentry->nlink = 1;
    }
	*pentry->mode = mode;
    return  proc_register (&proc_root, *pentry); /* register */
}


int usb_proc_entry_free (struct proc_dir_entry* entry)
{
    gdlf_fntrace ("usb_proc_entry_free (0x%p)", entry);

        /* free tree below */
        /* unregister */

#warning usb_proc_entry_free() not implemented!
    return -ENOSYS;
}


int usb_proc_init (void)
{
    int errcode;

    gdlf_fntrace ("usb_proc_init()");
    
        /* Create our main directory entry */

    errcode = usb_proc_entry_init (&proc_usb, "usb", S_IFDIR,
                                   PROC_USB, &proc_root);
    if (errcode < 0)
    {
        gdlf_err ("Cannot create proc entry \"usb\" "
                  "under /\"%s\": errno %d\n",
                  proc_root.name, errcode);
        usb_proc_entry_free (proc_usb);
        return errcode;
    }

        /* Statistics */
    
    errcode = usb_proc_entry_init (&proc_usb, "stats", 0, 
                                   PROC_USB, &proc_usb);
    if (errcode < 0)
        return errcode;

}


/* Log:
 *
 * $Log: procfs.c,v $
 * Revision 1.4  1998/11/14 17:15:03  inaky
 * Moved the include files to uusbd/include/uusbd, to ease integration
 * into the Linux kernel.
 *
 * Revision 1.3  1998/01/08 12:19:08  inaky
 * Added copyright strings to all important files
 *
 * Revision 1.2  1997/12/30 20:15:42  inaky
 * Moved everything to the new lists scheme. Untested yet.
 *
 * Revision 1.1  1997/12/30 00:03:45  inaky
 * Adding a generic system to handle singly linked lists and the HCI
 * registration control system
 *
 *
 */




