
/* USB Driver Stack v$Revision: 1.3 $ 
 *
 * (C) 1997 by Iaky Prez Gonzlez <inaky@peloncho.fis.ucm.es>
 *
 * Distributed under the terms of the GNU Public License Version 2
 * See files COPYRIGHT and COPYING at the root distribution directory
 *
 * $Id: cfg.c,v 1.3 1998/11/14 17:43:39 inaky Exp $
 *
 * Changelog at the bottom 
 *
 *
 *     Skeleton driver - Class inteface to configuration API
 *     -----------------------------------------------------
 */

#include <uusbd/usbd.h>
#include <uusbd/usb-cdcd.h>
#include "usb-avmq.h"

#define __NO_VERSION__      /* We just want MOD_*_USE_COUNT() */
#include <linux/module.h>   /* MOD_*_USE_COUNT() */

#define GDLF_IDENT          "USB-AVMQ"
#define GDLF_LOCAL          /* Define if want local settings */
#define GDLF_DEF_LOGLEVEL   9
#define GDLF_ASSERTIONS     /* want assertions (#undef if not) */
#define GDLF_FNTRACE        /* want function traces */
#define GDLF_LOCATIONS      /* want locations in log messages */
#include <uusbd/gdlf.h>


    /* Deal with AVM device quirks
     *
     *  This function is used to deal with AVM devices quirks, such as
     * uploading firmware, fixing little details in prototypes, etc ...
     *
     * Returns:
     *
     *  int             0 if ok, errno code < 0 on error.
     */

int avmq_config (ts_usb_cfel* cfel, ts_usb_device* device, 
                 ts_usb_cfg* cfg, t_usb_cfg cfg_idx, 
                 ts_usb_iface* iface, t_usb_iface iface_idx, 
                 ts_usb_aset* aset, t_usb_aset aset_idx, __u32 flags)
{
    gdlf_fntrace ("avmq_config (0x%p, 0x%p, 0x%p, "
                  "0x%p, 0x%p, %u)", cfel, device, cfg, 
                  iface, aset, flags);
    gdlf_aassert (cfel != NULL, return -EINVAL);
    gdlf_aassert (device != NULL, return -EINVAL);
    gdlf_aassert (device->vendor == AVMQ_USB_VENDOR, return -EBADFD);
    
    switch (device->product)
    {
      case AVMQ_B1_USB:

            /* Upload the firmware [if cfg'ing the device cfel] */

        if (cfel == &device->cfel)
        {
            gdlf_info ("Uploading firmware to AVM B1 USB, device (%d %d)\n",
                       usb_host_addr (device), usb_dev_addr (device));
#warning FIXME: upload the firmware, confirm we dont repeat
        }

            /*
             *  Some early B1 [prototype?] devices don't use the
             * CDC 1.0 code scheme as it wasn't 1.0 yet. As they
             * should behave more or less the same, we `patch' it,
             * instead of dealing with it at the driver level.
             */

        if (device->version <= 0x200 && device->cfel.class == 0xff)
        {
            gdlf_info ("Fixing vicsp codes on AVM B1 "
                       "USB, device (%d %d)\n",
                       usb_host_addr (device), usb_dev_addr (device));

            cfel = &device->cfel;
            cfel->class = 0;
            cfel->subclass = 0;
            cfel->protocol = 0;
            
            gdlf_aassert (device->pcfg != NULL, return -EINVAL);
            gdlf_aassert (device->pcfg[0] != NULL, return -EINVAL);
            gdlf_aassert (device->pcfg[0]->iface != NULL, return -EINVAL);
            gdlf_aassert (device->pcfg[0]->iface[0].aset != NULL, 
                          return -EINVAL);
            
            cfel = &device->pcfg[0]->iface[0].aset[0].cfel;            
            cfel->class = CDCD_IFACE_CLASS;
            cfel->subclass = CDCD_SUBCLASS_CAPI;
            cfel->protocol = CDCD_D_PROTOCOL_CAPI20;
        }        
        break;
    }
    return 0;
}


    /* Unconfigure a class configured configurable element
     *
     *  This function may be used to unconfigure a driver's
     * class-configured configurable elements.
     *
     * Parameters:  
     *
     *  ts_usb_device*  Pointer to the device structure to
     *                  unconfigure. 
     *
     *  unsigned        USB_CFG_* flags indicating why is this
     *                  function called.  
     *
     * Returns:
     *
     *  int             0 if ok, errno code < 0 on error.
     */

int avmq_unconfig (ts_usb_device* device, unsigned flags)
{
    gdlf_fntrace ("avmq_unconfig (0x%p, %u)", device, flags);

    gdlf_err ("Ein? I should not be called!\n");
    return 0;
}

/* Log:
 *
 * $Log: cfg.c,v $
 * Revision 1.3  1998/11/14 17:43:39  inaky
 * Moved the include files to uusbd/include/uusbd, to ease integration
 * into the Linux kernel.
 *
 * Revision 1.2  1998/11/09 21:40:59  inaky
 * Misc esthetic changes, added confirmation from Diego Friedel that AVM
 * B1 USB v2.0 will not need vicsp fixes. Changed the name of the
 * configuration pointer array to pcfg.
 *
 * Revision 1.1  1998/10/31 02:22:17  inaky
 * Makefile to generate the quirk modules for AVM devices.
 *
 * Revision 1.7  1998/10/14 21:49:53  inaky
 * Removed all the errno variables, were unused.
 *
 * Revision 1.6  1998/08/29 00:07:13  inaky
 * Actualized.
 *
 * Revision 1.5  1998/06/20 21:55:35  inaky
 * Some changes, this is getting quite outdated.
 *
 * Revision 1.4  1998/02/12 14:40:07  inaky
 * Drivers config method should inc the module use count when entering
 * and decrement it upon return.
 *
 * Revision 1.3  1998/01/08 12:19:20  inaky
 * Added copyright strings to all important files
 *
 * Revision 1.2  1998/01/08 11:50:06  inaky
 * Adapted to the new configuration scheme
 *
 * Revision 1.1  1997/12/30 20:15:13  inaky
 * Moved to the new lists scheme. Untested yet.
 *
 */

