
/* USB Driver Stack v$Revision: 1.3 $ 
 *
 * (C) 1997 by Iaky Prez Gonzlez <inaky@peloncho.fis.ucm.es>
 *
 * Distributed under the terms of the GNU Public License Version 2
 * See files COPYRIGHT and COPYING at the root distribution directory
 *
 * $Id: driver.c,v 1.3 1998/11/14 17:43:40 inaky Exp $
 *
 * Changelog at the bottom 
 *
 *
 *     AVM Quirks Driver - Driver housekeeping
 *     ---------------------------------------
 */

#include <linux/init.h>     /* __init */
#include <linux/module.h>   /* MOD_*_USE_COUNT + module stuff */
#include <asm/spinlock.h>   /* locks */

#include "usb-avmq.h"
#include <uusbd/usbd.h>

#define GDLF_IDENT          "USB-AVMQ"
#define GDLF_LOCAL          /* Define if want local settings */
#define GDLF_DEF_LOGLEVEL   9
#define GDLF_ASSERTIONS     /* want assertions (#undef if not) */
#define GDLF_FNTRACE        /* want function traces */
#define GDLF_LOCATIONS      /* want locations in log messages */
#include <uusbd/gdlf.h>


atomic_t usecnt;

#define NAME   /* short!! */                    \
    "AVM Generic Driver $Revision: 1.3 $ "

#define COPYRIGHT                               \
    "(c) 1998 Inaky Perez Gonzalez "            \
    "<inaky@peloncho.fis.ucm.es>"

#define VENDOR 0x57c
#define VENDORNAME                              \
    "AVM Berlin"

#define PRODUCT 0
#define DEVICENAME NULL

#define CLASS 0
#define CLASSNAME NULL

#define SUBCLASS 0
#define SUBCLASSNAME NULL

#define PROTOCOL 0
#define PROTOCOLNAME NULL

ts_usb_driver* avmq_driver = NULL;


    /* 
     * Linux module support
     */

#ifdef MODULE

int init_module (void)
{
    return avmq_init();
}

void cleanup_module (void)
{
    avmq_release();
    avmq_shutdown();
    return;
}

#endif


    /* Initialize the avmq driver
     *
     * Returns:
     *
     *  int             0 if ok, errno code < 0 on error.
     */

int __init avmq_init (void)
{
    int errcode;
    
    gdlf_fntrace ("avmq_init()");
    gdlf_info (NAME "\n");
#ifdef MODULE
    gdlf_info (COPYRIGHT "\n");
#endif

    avmq_driver = ts_usb_driver_init();
    if (avmq_driver == NULL)
    {
        usb_errno = -USBE_NO_MEMORY;
        return -ENOMEM;
    }

        /* Set driver's attributes */

    avmq_driver->vendor = VENDOR;
    avmq_driver->product = PRODUCT;
    avmq_driver->class = CLASS;
    avmq_driver->subclass = SUBCLASS;
    avmq_driver->protocol = PROTOCOL;

        /* Set info */

    avmq_driver->name = NAME;                  /* driver name */
    avmq_driver->copyright = COPYRIGHT;        /* driver copyright */
    avmq_driver->vendorname = VENDORNAME;      /* device's vendor name */
    avmq_driver->productname = DEVICENAME;     /* device's name */
    avmq_driver->classname = CLASSNAME;        /* class' name */
    avmq_driver->subclassname = SUBCLASSNAME;  /* subclass' name */
    avmq_driver->protocolname = PROTOCOLNAME;  /* protocol name */

    avmq_driver->config = avmq_config;          /* config method */
    avmq_driver->unconfig = avmq_unconfig;      /* unconfig method */
    avmq_driver->reference = avmq_reference;    /* unconfig method */
    ts_snode_ref (&avmq_driver->snode);         /* we hold a reference to it */
    
    errcode = usb_drv_register (avmq_driver);  /* register driver */
    if (errcode)
    {
        gdlf_err ("Cannot register driver "
                  "(errno/usb = %d/%d)\n",
                  errcode, usb_errno);
        if (avmq_driver->snode.next == NULL)
        {
            ts_usb_driver_clean (avmq_driver);
            usb_kfree (avmq_driver);
        }
        return errcode;
    }
        
    return 0;
}


    /* Release the avmq driver
     *
     *  Do what is needed to ensure the use count of the driver is
     * zero and can be unloaded from the kernel.
     *
     * Returns:
     *
     *  int             0 if ok, errno code < 0 on error.
     */

int avmq_release (void)
{
    gdlf_fntrace ("avmq_release()");
    return 0;
}


    /* Shutdown the skel driver
     *
     * Returns:
     *
     *  int             0 if ok, errno code < 0 on error.
     */

int avmq_shutdown (void)
{
    gdlf_fntrace ("avmq_shutdown()");

    ts_snode_unref (&avmq_driver->snode, NULL);
    return usb_drv_unregister (avmq_driver);
}


    /* FIXME: Detailed desc
     *
     *  The 'reference' method is used when we take any of the driver's
     * code or data structures to use it. If so, we call it with a
     * positive argument (one reference, regardless the value of the
     * _positive_ number). When we are done, we release the reference
     * calling it with a negative number. It always returns the number
     * of references or < 0 on error. Calling it with an argument of
     * zero will just return the number of references. 
     * Compliance:      USB Spec 1.0 xxxx
     *
     * Parameters:  
     *
     *  type            descr
     *
     * Global:
     *
     *  type name       descr
     *
     * Returns:
     *
     *  type            descr
     *
     * Global:
     *
     *  type name       descr
     */

int avmq_reference (ts_usb_device* device, int n)
{
    gdlf_fntrace ("avmq_class_reference (0x%p, %d)", device, n);
    gdlf_aassert (device != NULL, return -EINVAL);

    if (n > 0)
    {
        atomic_inc (&usecnt);
        MOD_INC_USE_COUNT;
    }
    else if (n < 0)
    {
        atomic_dec (&usecnt);
        MOD_DEC_USE_COUNT;
    }
    return atomic_read (&usecnt);
}


/* Log:
 *
 * $Log: driver.c,v $
 * Revision 1.3  1998/11/14 17:43:40  inaky
 * Moved the include files to uusbd/include/uusbd, to ease integration
 * into the Linux kernel.
 *
 * Revision 1.2  1998/11/09 21:41:20  inaky
 * Fixed the name, Quirks Driver sounds too lowsy.
 *
 * Revision 1.1  1998/10/31 02:22:17  inaky
 * Makefile to generate the quirk modules for AVM devices.
 *
 * Revision 1.17  1998/10/14 21:49:56  inaky
 * Removed all the errno variables, were unused.
 *
 * Revision 1.16  1998/08/29 00:07:39  inaky
 * Actualized.
 *
 * Revision 1.15  1998/07/23 14:39:34  inaky
 * Made all errnos volatile.
 *
 * Revision 1.14  1998/06/20 21:55:35  inaky
 * Some changes, this is getting quite outdated.
 *
 * Revision 1.13  1998/02/10 17:45:32  inaky
 * Fixed to use ts_[N]snode_[un]ref().
 *
 * Revision 1.12  1998/02/09 13:48:37  inaky
 * Moved all use counts to atomic, fixed the freeing of the driver
 * structure if the registration fails and added #defines for the
 * driver's vicsp codes.
 *
 * Revision 1.11  1998/01/11 15:44:41  inaky
 * Added docs?
 *
 * Revision 1.10  1998/01/08 12:19:20  inaky
 * Added copyright strings to all important files
 *
 * Revision 1.9  1998/01/08 11:50:06  inaky
 * Adapted to the new configuration scheme
 *
 * Revision 1.8  1998/01/02 00:12:20  inaky
 * Finished the new driver scheme. Documented at usbd-drv.c
 *
 * Revision 1.7  1997/12/30 22:30:35  inaky
 * Fixed some bugs.
 *
 * Revision 1.6  1997/12/30 20:15:13  inaky
 * Moved to the new lists scheme. Untested yet.
 *
 */

