
/* USB Driver Stack v$Revision: 1.9 $ 
 *
 * (C) 1997 by Iaky Prez Gonzlez <inaky@peloncho.fis.ucm.es>
 *
 * Distributed under the terms of the GNU Public License Version 2
 * See files COPYRIGHT and COPYING at the root distribution directory
 *
 * $Id: class-cfg.c,v 1.9 1998/11/14 17:44:13 inaky Exp $
 *
 * Changelog at the bottom 
 *
 *
 *     Skeleton driver - Class inteface to configuration API
 *     -----------------------------------------------------
 */

#define __NO_VERSION__      /* We just want MOD_*_USE_COUNT() */
#include <linux/module.h>   /* MOD_*_USE_COUNT() */

#include <uusbd/usbd.h>

#define GDLF_IDENT          "USB-SKELD"
#define GDLF_LOCAL          /* Define if want local settings */
#define GDLF_DEF_LOGLEVEL   9
#define GDLF_ASSERTIONS     /* want assertions (#undef if not) */
#define GDLF_FNTRACE        /* want function traces */
#define GDLF_LOCATIONS      /* want locations in log messages */
#include <uusbd/gdlf.h>

#include <uusbd/usb-skeld.h>


    /* Configure a device class interface
     *
     *  This function may be used to be called from skeld_config() to
     * configure the class part of a configurable element.
     *
     * Returns:
     *
     *  int             0 if ok, errno code < 0 on error.
     */

int skeld_class_config (ts_usb_cfel* cfel, ts_usb_device* device, 
                        ts_usb_cfg* cfg, t_usb_cfg cfg_idx, 
                        ts_usb_iface* iface, t_usb_iface iface_idx, 
                        ts_usb_aset* aset, t_usb_aset aset_idx, __u32 flags)
{
    gdlf_fntrace ("skeld_class_config (0x%p, 0x%p, 0x%p, "
                  "0x%p, 0x%p, %u)", cfel, device, cfg, 
                  iface, aset, flags);
    gdlf_aassert (cfel != NULL, return -EINVAL);
    gdlf_aassert (device != NULL, return -EINVAL);
#warning FIXME: skeld_class_config() not implemented!
    return -ENOSYS;
}


    /* Unconfigure a class configured configurable element
     *
     *  This function may be used to unconfigure a driver's
     * class-configured configurable elements.
     *
     * Parameters:  
     *
     *  ts_usb_device*  Pointer to the device structure to
     *                  unconfigure. 
     *
     *  unsigned        USB_CFG_* flags indicating why is this
     *                  function called.  
     *
     * Returns:
     *
     *  int             0 if ok, errno code < 0 on error.
     */

int skeld_class_unconfig (ts_usb_device* device, unsigned flags)
{
    t_usb_cfg cfg_cnt;
    ts_usb_cfg *cfg;
    ts_usb_iface *iface, *iface_top;
    ts_usb_aset *aset, *aset_top;
    
    gdlf_fntrace ("skeld_class_unconfig (0x%p, %u)", device, flags);
    gdlf_aassert (device != NULL, return -EINVAL);

    for (cfg_cnt = 0; cfg_cnt < device->cfgs; cfg_cnt++)
    {
        cfg = device->cfg[cfg_cnt];
        if (cfg == NULL)        /* unloaded? skip */
            continue;
        
            /* Walk cfg's interfaces */

        for (iface = cfg->iface, iface_top = iface + cfg->ifaces; 
             iface < iface_top; iface++)
        {
                /* Walk the iface's asets */

            for (aset = iface->aset, aset_top = aset + iface->asets; 
                 aset < aset_top; aset++)

                      /* cfged by us? */

                if (aset->cfel.configured
                    && aset->cfel.data
                    && aset->cfel.data->dtor.fn
                       == (tf_dtor)skeld_class_destroy)
                {

#warning FIXME: do class stuff here!!

                    ts_snode_unref (aset->cfel.data, NULL);
                    aset->cfel.data = NULL;
                    aset->cfel.configured = 0;      /* uncfged */
                }
        }         
    }
    return 0;
}

/* Log:
 *
 * $Log: class-cfg.c,v $
 * Revision 1.9  1998/11/14 17:44:13  inaky
 * Moved the include files to uusbd/include/uusbd, to ease integration
 * into the Linux kernel.
 *
 * Revision 1.8  1998/10/31 02:18:32  inaky
 * Removed all the d_* pointers from ts_usb_cfel and collapsed them into
 * a single pointer; only one was really used.
 *
 * Revision 1.7  1998/10/14 21:49:53  inaky
 * Removed all the errno variables, were unused.
 *
 * Revision 1.6  1998/08/29 00:07:13  inaky
 * Actualized.
 *
 * Revision 1.5  1998/06/20 21:55:35  inaky
 * Some changes, this is getting quite outdated.
 *
 * Revision 1.4  1998/02/12 14:40:07  inaky
 * Drivers config method should inc the module use count when entering
 * and decrement it upon return.
 *
 * Revision 1.3  1998/01/08 12:19:20  inaky
 * Added copyright strings to all important files
 *
 * Revision 1.2  1998/01/08 11:50:06  inaky
 * Adapted to the new configuration scheme
 *
 * Revision 1.1  1997/12/30 20:15:13  inaky
 * Moved to the new lists scheme. Untested yet.
 *
 */

