
/* USB Driver Stack v$Revision: 1.7 $ 
 *
 * (C) 1997 by Iaky Prez Gonzlez <inaky@peloncho.fis.ucm.es>
 *
 * Distributed under the terms of the GNU Public License Version 2
 * See files COPYRIGHT and COPYING at the root distribution directory
 *
 * $Id: class-hkeep.c,v 1.7 1998/11/14 17:44:15 inaky Exp $
 *
 * Changelog at the bottom 
 *
 *
 *     Skeleton driver - Class structures housekeeping
 *     -----------------------------------------------
 */

#include <linux/module.h>   /* MOD_*_USE_COUNT */
#include <asm/spinlock.h>   /* locks */

#include <uusbd/usbd.h>

#define GDLF_IDENT          "USB-SKELD"
#define GDLF_LOCAL          /* Define if want local settings */
#define GDLF_DEF_LOGLEVEL   9
#define GDLF_ASSERTIONS     /* want assertions (#undef if not) */
#define GDLF_FNTRACE        /* want function traces */
#define GDLF_LOCATIONS      /* want locations in log messages */
#include <uusbd/gdlf.h>

#include <uusbd/usb-skeld.h>


ts_slist skeld_class_list;  /* list of registered class structures */


    /* Unregister an skeld_class element
     *
     * This is the destructor function.
     *
     * Parameters:  
     *
     *  ts_skeld_class* Class element to unregister
     *
     * Returns:
     *
     *  int             0 if ok, errno code < 0 on error.
     */

int skeld_class_unregister (ts_skeld_class* class)
{
    gdlf_fntrace ("skeld_class_unregister (0x%p)", class);
    gdlf_aassert (class != NULL, return -EINVAL);
    gdlf_assert (class->snode.dtor.fn == (tf_dtor)skeld_class_destroy);

        /* FIXME: do your class' unregistration stuff */
#warning FIXME: class stuff not implemented!

        /* Let's get it removed from the list */

    ts_slist_del (&skeld_class_list, &class->snode);
    ts_skeld_class_release (class); /* clean the struct */
#warning FIXME: remove this, it makes modules stay for ever
    MOD_DEC_USE_COUNT;              /* we ain't using the module */
    return 0;                       /* bye */
}


    /* FIXME: Detailed desc
     *
     * Compliance:      USB Spec 1.0 xxxx
     *
     * Parameters:  
     *
     *  type            descr
     *
     * Global:
     *
     *  type name       descr
     *
     * Returns:
     *
     *  type            descr
     *
     * Global:
     *
     *  type name       descr
     */

int skeld_class_destroy (ts_skeld_class* class)
{
    t_usb_cfg cfg_cnt;
    ts_usb_cfg *cfg;
    ts_usb_iface *iface, *iface_top;
    ts_usb_aset *aset, *aset_top;
    
    gdlf_fntrace ("skeld_class_destroy (0x%p)", class);
    gdlf_aassert (class != NULL, return -EINVAL);
    gdlf_assert (class->snode.dtor.fn == (tf_dtor)skeld_class_destroy);
    gdlf_aassert (class->device, return -EINVAL);
    gdlf_aassert (class->device->cfg, return -EINVAL);
    
    ts_snode_ref (&class->snode);

#warning FIXME: do your class destruction stuff

        /* 
         *  Ok, walk the cfgs, ifaces and aset, searching for an
         * element configured for this class. If found, unref class and
         * keep on. 
         */

    for (cfg_cnt = 0; cfg_cnt < class->device->cfgs; cfg_cnt++)
    {
        cfg = class->device->cfg[cfg_cnt];
        if (cfg == NULL)        /* unloaded? skip */
            continue;
        
            /* Walk cfg's interfaces */

        for (iface = cfg->iface, iface_top = iface + cfg->ifaces; 
             iface < iface_top; iface++)
        {
                /* Walk the iface's asets */

            for (aset = iface->aset, aset_top = aset + iface->asets; 
                 aset < aset_top; aset++)

                      /* cfged by us? */

#warning WARNIG: as a sample, we ID the struct with the configured field

                if (aset->cfel.configured == (unsigned long)&class->snode)
                {
                    ts_snode_unref (&class->snode, NULL);
                    aset->cfel.configured = 0;      /* uncfged */
                }
        }         
    }

    skeld_class_unregister (class);     /* unregister */
    ts_snode_unref_nd (&class->snode);
    ts_skeld_class_release (class);     /* clean the struct */
    return 0;                           /* bye */
}


    /* Register an skeld_class element
     *
     * Parameters:  
     *
     *  ts_skeld_class* Class element to register
     *
     * Returns:
     *
     *  int             0 if ok, errno code < 0 on error.
     */

int skeld_class_register (ts_skeld_class* class)
{
    int errcode;
    
    gdlf_fntrace ("skeld_class_register (0x%p)", class);
    gdlf_aassert (class != NULL, return -EINVAL);

        /* FIXME: do your class' stuff */
#warning FIXME: class stuff not implemented!

    ts_dtor_fill (&class->snode.dtor, (tf_dtor)skeld_class_destroy);

        /* get into the list */

    errcode = ts_slist_prep (&skeld_class_list, &class->snode);
    if (errcode)
        return errcode;
#warning FIXME: remove this, it makes modules stay for ever
    MOD_INC_USE_COUNT;          /* request we're using the module */
    return 0;                   /* bye */
}

/* Log:
 *
 * $Log: class-hkeep.c,v $
 * Revision 1.7  1998/11/14 17:44:15  inaky
 * Moved the include files to uusbd/include/uusbd, to ease integration
 * into the Linux kernel.
 *
 * Revision 1.6  1998/10/14 21:49:55  inaky
 * Removed all the errno variables, were unused.
 *
 * Revision 1.5  1998/08/29 00:07:30  inaky
 * Actualized.
 *
 * Revision 1.4  1998/01/08 12:19:20  inaky
 * Added copyright strings to all important files
 *
 * Revision 1.3  1998/01/01 04:28:56  inaky
 * Tweaked makefiles to make'em work
 *
 * Revision 1.2  1997/12/31 17:06:41  inaky
 * Removed some unneeded locks and fixed some others.
 *
 * Revision 1.1  1997/12/30 20:15:13  inaky
 * Moved to the new lists scheme. Untested yet.
 *
 */

