
/* USB Driver Stack v$Revision: 1.18 $ 
 *
 * (C) 1997 by Iaky Prez Gonzlez <inaky@peloncho.fis.ucm.es>
 *
 * Distributed under the terms of the GNU Public License Version 2
 * See files COPYRIGHT and COPYING at the root distribution directory
 *
 * $Id: driver.c,v 1.18 1998/11/14 17:44:16 inaky Exp $
 *
 * Changelog at the bottom 
 *
 *
 *     Skeleton driver - Driver housekeeping
 *     -------------------------------------
 */

#include <asm/spinlock.h>   /* locks */
#include <linux/init.h>     /* __init */
#define __NO_VERSION__
#include <linux/module.h>   /* MOD_*_USE_COUNT */

#include <uusbd/usbd.h>

#define GDLF_IDENT          "USB-SKELD"
#define GDLF_LOCAL          /* Define if want local settings */
#define GDLF_DEF_LOGLEVEL   9
#define GDLF_ASSERTIONS     /* want assertions (#undef if not) */
#define GDLF_FNTRACE        /* want function traces */
#define GDLF_LOCATIONS      /* want locations in log messages */
#include <uusbd/gdlf.h>

#include <uusbd/usb-skeld.h>


atomic_t usecnt;

#define NAME   /* short!! */                    \
    "Skeleton uusbd driver v$Revision: 1.18 $ "

#define COPYRIGHT                               \
    "(c) 1998 Inaky Perez Gonzalez "            \
    "<inaky@peloncho.fis.ucm.es>"

#define VENDOR 0
#define VENDORNAME                              \
    "Tifos, Chuzos && Cocidos, hics!"

#define PRODUCT 0
#define DEVICENAME                              \
    "Vodka bottle"

#define CLASS 0
#define CLASSNAME                               \
    "Drinking devices"

#define SUBCLASS 0
#define SUBCLASSNAME                            \
    "Vodka drinking devices"

#define PROTOCOL 0
#define PROTOCOLNAME                            \
    "Ice, lemon juice and tube glass"

#define SNODE_NULL { {NULL}, NULL, 0}

ts_usb_driver* skeld_driver = NULL;


    /* 
     * Linux module support
     */

#ifdef MODULE

int init_module (void)
{
    return skeld_init();
}

void cleanup_module (void)
{
    skeld_release();
    skeld_shutdown();
    return;
}

#endif


    /* Initialize the skeld driver
     *
     * Returns:
     *
     *  int             0 if ok, errno code < 0 on error.
     */

int __init skeld_init (void)
{
    int errcode;
    
    gdlf_fntrace ("skeld_init()");
    gdlf_info (NAME "\n");
#ifdef MODULE
    gdlf_info (COPYRIGHT "\n");
#endif

    skeld_driver = ts_usb_driver_init();
    if (skeld_driver == NULL)
    {
        usb_errno = -USBE_NO_MEMORY;
        return -ENOMEM;
    }

        /* Set driver's attributes */

    skeld_driver->vendor = VENDOR;
    skeld_driver->product = PRODUCT;
    skeld_driver->class = CLASS;
    skeld_driver->subclass = SUBCLASS;
    skeld_driver->protocol = PROTOCOL;

        /* Set info */

    skeld_driver->name = NAME;                  /* driver name */
    skeld_driver->copyright = COPYRIGHT;        /* driver copyright */
    skeld_driver->vendorname = VENDORNAME;      /* device's vendor name */
    skeld_driver->productname = DEVICENAME;     /* device's name */
    skeld_driver->classname = CLASSNAME;        /* class' name */
    skeld_driver->subclassname = SUBCLASSNAME;  /* subclass' name */
    skeld_driver->protocolname = PROTOCOLNAME;  /* protocol name */

    skeld_driver->config = skeld_class_config;  /* config method */
    skeld_driver->unconfig = skeld_class_unconfig;  /* unconfig method */
    skeld_driver->reference = skeld_class_reference;  /* unconfig method */
    ts_snode_ref (&skeld_driver->snode);        /* we hold a reference to it */
    
    ts_slist_clean (&skeld_class_list);         /* initialize the list */
    errcode = usb_drv_register (skeld_driver);  /* register driver */
    if (errcode)
    {
        gdlf_err ("Cannot register driver "
                  "(errno/usb = %d/%d)\n",
                  errcode, usb_errno);
        if (skeld_driver->snode.next == NULL)
        {
            ts_usb_driver_clean (skeld_driver);
            usb_kfree (skeld_driver);
        }
        return errcode;
    }
        
        /* FIXME: do you class' stuf */
#warning FIXME: do class stuff!

    return 0;
}


    /* Release the skeld driver
     *
     *  Do what is needed to ensure the use count of the driver is
     * zero and can be unloaded from the kernel.
     *
     * Returns:
     *
     *  int             0 if ok, errno code < 0 on error.
     */

int skeld_release (void)
{
    gdlf_fntrace ("skeld_release()");

#warning FIXME: skeld_release() not implemented!

    return -ENOSYS;
}


    /* Shutdown the skel driver
     *
     * Returns:
     *
     *  int             0 if ok, errno code < 0 on error.
     */

int skeld_shutdown (void)
{
    gdlf_fntrace ("skeld_shutdown()");

        /* FIXME: do you class' stuf */

    ts_snode_unref (&skeld_driver->snode, NULL);
    return usb_drv_unregister (skeld_driver);
}


    /* FIXME: Detailed desc
     *
     *  The 'reference' method is used when we take any of the driver's
     * code or data structures to use it. If so, we call it with a
     * positive argument (one reference, regardless the value of the
     * _positive_ number). When we are done, we release the reference
     * calling it with a negative number. It always returns the number
     * of references or < 0 on error. Calling it with an argument of
     * zero will just return the number of references. 
     * Compliance:      USB Spec 1.0 xxxx
     *
     * Parameters:  
     *
     *  type            descr
     *
     * Global:
     *
     *  type name       descr
     *
     * Returns:
     *
     *  type            descr
     *
     * Global:
     *
     *  type name       descr
     */

int skeld_class_reference (ts_usb_device* device, int n)
{
    gdlf_fntrace ("skeld_class_reference (0x%p, %d)", device, n);
    gdlf_aassert (device != NULL, return -EINVAL);

#warning FIXME: add more stuff, keep the device ptr ...

    if (n > 0)
    {
        atomic_inc (&usecnt);
        MOD_INC_USE_COUNT;
    }
    else if (n < 0)
    {
        atomic_dec (&usecnt);
        MOD_DEC_USE_COUNT;
    }
    return atomic_read (&usecnt);
}


/* Log:
 *
 * $Log: driver.c,v $
 * Revision 1.18  1998/11/14 17:44:16  inaky
 * Moved the include files to uusbd/include/uusbd, to ease integration
 * into the Linux kernel.
 *
 * Revision 1.17  1998/10/14 21:49:56  inaky
 * Removed all the errno variables, were unused.
 *
 * Revision 1.16  1998/08/29 00:07:39  inaky
 * Actualized.
 *
 * Revision 1.15  1998/07/23 14:39:34  inaky
 * Made all errnos volatile.
 *
 * Revision 1.14  1998/06/20 21:55:35  inaky
 * Some changes, this is getting quite outdated.
 *
 * Revision 1.13  1998/02/10 17:45:32  inaky
 * Fixed to use ts_[N]snode_[un]ref().
 *
 * Revision 1.12  1998/02/09 13:48:37  inaky
 * Moved all use counts to atomic, fixed the freeing of the driver
 * structure if the registration fails and added #defines for the
 * driver's vicsp codes.
 *
 * Revision 1.11  1998/01/11 15:44:41  inaky
 * Added docs?
 *
 * Revision 1.10  1998/01/08 12:19:20  inaky
 * Added copyright strings to all important files
 *
 * Revision 1.9  1998/01/08 11:50:06  inaky
 * Adapted to the new configuration scheme
 *
 * Revision 1.8  1998/01/02 00:12:20  inaky
 * Finished the new driver scheme. Documented at usbd-drv.c
 *
 * Revision 1.7  1997/12/30 22:30:35  inaky
 * Fixed some bugs.
 *
 * Revision 1.6  1997/12/30 20:15:13  inaky
 * Moved to the new lists scheme. Untested yet.
 *
 */

