
/* USB Driver Stack v$Revision: 1.36 $ 
 *
 * (C) 1997 by Iaky Prez Gonzlez <inaky@peloncho.fis.ucm.es>
 *
 * Distributed under the terms of the GNU Public License Version 2
 * See files COPYRIGHT and COPYING at the root distribution directory
 *
 * $Id: generic.c,v 1.36 1998/11/14 17:45:42 inaky Exp $
 *
 * Changelog at the bottom 
 *
 *  UHCI data structures housekeeping
 *
 * FIXME: Some functions ain't needed, deprecate'em (#if 0/#endif)
 */

#include <linux/config.h>       /* Configuration */
#include <linux/malloc.h>       /* GFP_KERNEL */
#include <linux/ioport.h>       /* check_region(), release_region() */
#include <linux/sched.h>        /* free_irq() */

#include <uusbd/usbd.h>

#define GDLF_IDENT          "USB-UHCI"
#define GDLF_LOCAL          /* Define if want local settings */
#define GDLF_DEF_LOGLEVEL   9
#define GDLF_ASSERTIONS     /* want assertions (#undef if not) */
#undef GDLF_FNTRACE        /* want function traces */
#define GDLF_LOCATIONS      /* want locations in log messages */
#include <uusbd/gdlf.h>

#include "uhci.h"           /* this has to be include after GDLF */


    /* Allocate and initialize a ts_uhci_host structure.
     * 
     * Global:
     *
     *  int usb_errno   On error, error code.  
     */

ts_uhci_host* ts_uhci_host_init (void)
{
    ts_uhci_host *uhci;
    
    gdlf_fntrace ("ts_uhci_host_init()");

    uhci = usb_kmalloc (sizeof (ts_uhci_host), GFP_KERNEL);
    if (uhci == NULL)
    {
        gdlf_err ("No memory to initialize " 
                  "a UHCI host structure\n");
        usb_errno = -USBE_NO_MEMORY;
        return NULL;
    }
    ts_uhci_host_clean (uhci);
    return uhci;
}


    /* Release a ts_usb_uhci structure's content's
     * 
     * Parameters:
     *
     *  ts_uhci_host*   Pointer to the host controller structure of
     *                  the host.
     *
     * Global:
     *
     *  int usb_errno   On error, error code.  
     */

void ts_uhci_host_release (ts_uhci_host* uhci)
{
    gdlf_fntrace ("ts_uhci_host_release (0x%p)", uhci);
    gdlf_aassert (uhci != NULL, return); 
    
    ts_snode_release (&uhci->snode);

    gdlf_assert (atomic_read (&uhci->snode.usecnt) == 0);
    gdlf_assert (atomic_read (&uhci->sem) == 0);

    if (uhci->fl != NULL)
    {
        ts_uhci_flp_a_release (uhci->fl, UHCI_FL_SIZE);
        usb_free_pages ((unsigned long)uhci->fl, 0);
    }

        /* 
         * NOTE: IRQ 0 is always taken by timer, so we can suppose 0
         *       meaning unused.
         */

    if (uhci->irq)
    {
        free_irq (uhci->irq, uhci->host);
        uhci->irq = 0;
    }

    if (uhci->ioextent && uhci->iobase)
        release_region (uhci->iobase, uhci->ioextent);

    if (uhci->ls_ctrl)
    {
        gdlf_assert (uhci->ls_ctrl->last == &uhci->ls_ctrl->first);
        uhci->ls_ctrl->first.head.word = cpu_to_le32 (0x3);
        uhci->ls_ctrl->last = NULL;
        ts_uhci_tb_list_release (uhci->ls_ctrl);
        usb_kfree (uhci->ls_ctrl);
    }

    if (uhci->ctrl)
    {
        gdlf_assert (uhci->ctrl->last == &uhci->ctrl->first);
        uhci->ctrl->first.head.word = cpu_to_le32 (0x3);
        uhci->ctrl->last = NULL;
        ts_uhci_tb_list_release (uhci->ctrl);
        usb_kfree (uhci->ctrl);
    }

    if (uhci->bulk)
    {
        gdlf_assert (uhci->bulk->last == &uhci->bulk->first);
        uhci->bulk->first.head.word = cpu_to_le32 (0x3);
        uhci->bulk->last = NULL;
        ts_uhci_tb_list_release (uhci->bulk);
        usb_kfree (uhci->bulk);
    }

    ts_uhci_iso_hash_release (&uhci->iso);
    ts_uhci_intr_hash_release (&uhci->intr);
    return;
}


ts_uhci_flp* ts_uhci_flp_init (void)
{
    ts_uhci_flp* flp;

    gdlf_fntrace ("ts_uhci_flp_init()");

    flp = usb_kmalloc (sizeof (ts_uhci_flp), GFP_KERNEL);
    if (flp == NULL)
    {        
        gdlf_err ("No memory to initialize " 
                  "a Frame List Pointer structure\n");
        usb_errno = -USBE_NO_MEMORY;
        return NULL;
    }
    
    ts_uhci_flp_clean (flp);
    return flp;
}


void ts_uhci_flp_release (ts_uhci_flp* flp)
{
        /* If trace left, freein the 1024 FL list gets hideous! */
#warning DEBUG: remove me for getting trace of ts_uhci_flp_release
#if 0
    gdlf_fntrace ("ts_uhci_flp_release (0x%p)", flp);
#endif
    gdlf_aassert (flp != NULL, return);
    return;
}


    /* This one has to be like this. We may have a lot of memory
     * readily available to be used as a frame list pointer array */

ts_uhci_flp* ts_uhci_flp_a_init (ts_uhci_flp *flps, unsigned int size)
{
    ts_uhci_flp *itr;
    
    gdlf_fntrace ("ts_uhci_flp_a_init (0x%p, %u)", flps, size);
    gdlf_aassert (size != 0, return NULL);

    if (flps == NULL)
        flps = usb_kmalloc (sizeof (ts_uhci_flp) * size, GFP_KERNEL);
    if (flps == NULL)
    {        
        gdlf_err ("No memory to initialize " 
                  "an array of Frame List Pointer structures\n");
        usb_errno = -USBE_NO_MEMORY;
        return NULL;
    }

    for (itr = flps + size - 1; itr >= flps; itr--)
    {
                    /* Need braces, this is a macro! */
        ts_uhci_flp_clean (itr);
    }

    return flps;
}


void ts_uhci_flp_a_release (ts_uhci_flp* flps, unsigned int size)
{
    ts_uhci_flp *itr;

    gdlf_fntrace ("ts_uhci_flp_a_release (0x%p, %u)", flps, size);
    gdlf_aassert (flps != NULL, return);
    gdlf_aassert (size != 0, return);
        
    for (itr = flps + size - 1; itr >= flps; itr--)
        ts_uhci_flp_release (flps);
    return;
}


ts_uhci_td* ts_uhci_td_init (void)
{
    ts_uhci_td *td;

    gdlf_fntrace ("ts_uhci_td_init()");

    td = usb_kmalloc (sizeof (ts_uhci_td), GFP_KERNEL);
    if (td == NULL)
    {        
        gdlf_err ("No memory to initialize " 
                  "a Transfer Descriptor structure\n");
        usb_errno = -USBE_NO_MEMORY;
        return NULL;
    }
    
    ts_uhci_td_clean (td);
    return td;
}


void ts_uhci_td_release (ts_uhci_td* td)
{
#warning DEBUG: disabled ts_uhci_td_release() fntrace to remove clutter
#if 0
    gdlf_fntrace ("ts_uhci_td_release (0x%p)", td);
#endif
    gdlf_aassert (td != NULL,return);
    return;
}


ts_uhci_td* ts_uhci_td_a_init (unsigned int size)
{
    ts_uhci_td *tds, *itr;

    gdlf_fntrace ("ts_uhci_td_a_init (%u)", size);
    gdlf_aassert (size != 0, return NULL);

    tds = usb_kmalloc (sizeof (ts_uhci_td) * size, GFP_KERNEL);
    if (tds == NULL)
    {        
        gdlf_err ("No memory to initialize " 
                  "an array of Transfer Descriptor structures\n");
        usb_errno = -USBE_NO_MEMORY;
        return NULL;
    }
    
    for (itr = tds + size - 1; itr >= tds; itr--)
    {
            /* Need braces, this is a macro! */
        ts_uhci_td_clean (itr);
    }
    return tds;
}


void ts_uhci_td_a_release (ts_uhci_td* tds, unsigned int size)
{
    ts_uhci_td *itr;
    
    gdlf_fntrace ("ts_uhci_td_a_release (0x%p, %u)", tds, size);
    gdlf_aassert (size != 0, return);
    gdlf_aassert (tds != NULL, return);
    
    for (itr = tds + size - 1; itr >= tds; itr--)
        ts_uhci_td_release (itr);
    return;
}


ts_uhci_qh* ts_uhci_qh_init (void)
{
    ts_uhci_qh *qh;

    gdlf_fntrace ("ts_uhci_qh_init()");

    qh = usb_kmalloc_align (sizeof (ts_uhci_qh), 16, GFP_KERNEL);
    if (qh == NULL)
    {        
        gdlf_err ("No memory to initialize " 
                  "a Queue Head structure\n");
        usb_errno = -USBE_NO_MEMORY;
        return NULL;
    }
    
    ts_uhci_qh_clean (qh);
    return qh;
}


void ts_uhci_qh_release (ts_uhci_qh* qh)
{
    gdlf_fntrace ("ts_uhci_qh_release (0x%p)", qh);
    gdlf_aassert (qh != NULL, return);
    gdlf_aassert (qh->head.word == cpu_to_le32 (0x3), 
                  gdlf_debug ("QH is 0x%p\n", qh); 
                  ts_uhci_qh_printk (qh));
    gdlf_assert (qh->elt.word == cpu_to_le32 (0x3));
    return;
}


ts_uhci_qh* ts_uhci_qh_a_init (unsigned int size)
{
    ts_uhci_qh *qhs, *top;
    
    gdlf_fntrace ("ts_uhci_qh_a_init (%u)", size);
    gdlf_aassert (size != 0, return NULL);

    qhs = usb_kmalloc (sizeof (ts_uhci_qh) * size, GFP_KERNEL);
    if (qhs == NULL)
    {        
        gdlf_err ("No memory to initialize " 
                  "an array of Queue Head structures\n");
        usb_errno = -USBE_NO_MEMORY;
        return NULL;
    }
    
    for (top = qhs + size - 1; top >= qhs; top--)
    {
            /* Need braces, this is a macro! */
        ts_uhci_qh_clean (top);
    }
    return qhs;
}


void ts_uhci_qh_a_release (ts_uhci_qh* qhs, unsigned int size)
{
    ts_uhci_qh *itr;
    
    gdlf_fntrace ("ts_uhci_qh_a_release (0x%p, %u)", qhs, size);
    gdlf_aassert (size != 0, return);
    gdlf_aassert (qhs != NULL, return);
    
    for (itr = qhs + size; itr >= qhs; itr--)
        ts_uhci_qh_release (itr);
    return;
}


ts_uhci_tb* ts_uhci_tb_init (void)
{
    ts_uhci_tb *tb;
    
    gdlf_fntrace ("ts_uhci_tb_init()");

    tb = usb_kmalloc_align (sizeof (ts_uhci_tb), 16, GFP_KERNEL);
    if (tb == NULL)
    {        
        gdlf_err ("No memory to initialize " 
                  "a TDS Buffer structure\n");
        usb_errno = -USBE_NO_MEMORY;
        return NULL;
    }

    ts_uhci_tb_clean (tb);
    return tb;
}


void ts_uhci_tb_release (ts_uhci_tb* tb)
{
    gdlf_fntrace ("ts_uhci_tb_release (0x%p)", tb);
    gdlf_aassert (tb != NULL, return);
        
    if (tb->td)
    {
        ts_uhci_td_a_release (tb->td, tb->tds);
        usb_kfree (tb->td);
    }
        /*
         * no need to ts_snode_unref (tb->tr), it is an _unregistered_
         * reference 
         */ 
    return;
}


ts_uhci_tr_hci* ts_uhci_tr_hci_init (void)
{
    ts_uhci_tr_hci *tr_hci;
    
    gdlf_fntrace ("ts_uhci_tr_hci_init()");

    tr_hci = usb_kmalloc_align (sizeof (ts_uhci_tr_hci), 16, GFP_KERNEL);
    if (tr_hci == NULL)
    {        
        gdlf_err ("No memory to initialize " 
                  "a transaction HCI structure\n");
        usb_errno = -USBE_NO_MEMORY;
        return NULL;
    }

    ts_uhci_tr_hci_clean (tr_hci);
    return tr_hci;
}


void ts_uhci_tr_hci_release (ts_uhci_tr_hci* tr_hci)
{
    gdlf_fntrace ("ts_uhci_tr_hci_release (0x%p)", tr_hci);
    gdlf_aassert (tr_hci != NULL, return);
        
    ts_uhci_tb_release (&tr_hci->tb);
    return;
}


ts_uhci_tb_list* ts_uhci_tb_list_init (void)
{
    ts_uhci_tb_list *tb_list;
    
    gdlf_fntrace ("ts_uhci_tb_list_init()");

    tb_list = usb_kmalloc_align (sizeof (ts_uhci_tb_list), 16, GFP_KERNEL);
    if (tb_list == NULL)
    {        
        gdlf_err ("No memory to initialize " 
                  "a transaction blocks list structure\n");
        usb_errno = -USBE_NO_MEMORY;
        return NULL;
    }

    ts_uhci_tb_list_clean (tb_list);
    return tb_list;
}


void ts_uhci_tb_list_release (ts_uhci_tb_list* tb_list)
{
    gdlf_fntrace ("ts_uhci_tb_list_release (0x%p)", tb_list);
    gdlf_aassert (tb_list != NULL, return);
    gdlf_aassert (tb_list->last == NULL, return);
    ts_uhci_qh_release (&tb_list->first);
    return;
}


#if 0 /* deprecated */
ts_uhci_dev_hci* ts_uhci_dev_hci_init (void)
{
    ts_uhci_dev_hci *dev_hci;
    
    gdlf_fntrace ("ts_uhci_dev_hci_init()");

    dev_hci = usb_kmalloc_align (sizeof (ts_uhci_dev_hci), 16, GFP_KERNEL);
    if (dev_hci == NULL)
    {        
        gdlf_err ("No memory to initialize " 
                  "a transaction HCI structure\n");
        usb_errno = -USBE_NO_MEMORY;
        return NULL;
    }    

    ts_uhci_dev_hci_clean (dev_hci);
    return dev_hci;
}


void ts_uhci_dev_hci_release (ts_uhci_dev_hci* dev_hci)
{
    gdlf_fntrace ("ts_uhci_dev_hci_release (0x%p)", dev_hci);
    gdlf_aassert (dev_hci != NULL, return);
        
    if (dev_hci->bulk)
    {
        ts_uhci_tb_list_release (dev_hci->bulk);
        usb_kfree (dev_hci->bulk);
    }
    
    if (dev_hci->interrupt)
    {
        ts_uhci_tb_list_release (dev_hci->interrupt);
        usb_kfree (dev_hci->interrupt);
    }
    
    return;
}
#endif /* 0 - deprecated */


    /* Print out a description of a ts_uhci_host struct contents
     *
     * Parameters:  
     *  ts_uhci_host*   Pointer to the host structure.
     *
     * Returns:
     *  int             0 if ok, <0 USB error code (if NULL pointer given).
     *
     * Global:
     *  int usb_errno   Error code if != 0.
     */

#ifdef UHCI_DEBUG
void ts_uhci_host_printk (ts_uhci_host *host)
{
    gdlf_fntrace ("ts_uhci_host_printk (0x%p)", host);
    gdlf_aassert (host != NULL, return);

#ifndef CONFIG_PCI
    gdlf_debug ("ts_uhci_host at 0x%p\n"
                "  dtor:        0x%p\n"
                "  iobase:      0x%x\n"
                "  ioextent:    0x%x\n"
                "  irq:         %u\n"
                "  frame list:  0x%p\n"
                "  size:        %u\n"
                "  sof_timing:  %u\n",
                host,
                host->dtor.fn, host->iobase, host->ioextent, host->irq,
                host->fl, host->size, host->sof_timing);
#else /* CONFIG_PCI */
    gdlf_debug ("ts_uhci_host at 0x%p\n"
                "  dtor:        0x%p\n"
                "  iobase:      0x%x\n"
                "  ioextent:    0x%x\n"
                "  irq:         %u\n"
                "  pci:         0x%p\n"
                "  frame list:  0x%p\n"
                "  sof_timing:  %u\n",
                host,
                host->snode.dtor.fn, 
                host->iobase, host->ioextent, host->irq,
                host->pci,
                host->fl, host->sof_timing);
#endif /* CONFIG_PCI */
    return;
}
#endif


#undef UHCI_DEBUG_TOPS
#ifdef UHCI_DEBUG
#ifdef UHCI_DEBUG_TOPS
int sec = 0;
#define SEC_MAX 500
#endif


void ts_uhci_td_printk (ts_uhci_td *td)
{
    __u32 *ptr;

    gdlf_fntrace ("ts_uhci_td_printk (0x%p)", td);
    gdlf_aassert (td != NULL, return);
    
    ptr = (__u32*)td;
    gdlf_debug ("%08x %08x %08x %08x\n", ptr[0], ptr[1], ptr[2], ptr[3]);
    gdlf_debug ("  lp ptr=%x(0x%p) Vf=%d Q=%d T=%d\n",
                td->lp.bits.ptr << 4, 
                bus_to_virt (le32_to_cpu (td->lp.bits.ptr << 4)),
                td->lp.bits.vf, td->lp.bits.q, td->lp.bits.t);
    gdlf_debug ("  ctstat spd=%d c_err=%d ls=%d iso=%d ioc=%d "
                "active=%d stalled=%d databuf_err=%d babble=%d "    
                "nak=%d crcto_err=%d bitst_err=%d "
                "actlen=%x\n",
                td->ctstat.bits.spd,
                td->ctstat.bits.c_err,
                td->ctstat.bits.ls,
                td->ctstat.bits.iso,
                td->ctstat.bits.ioc,
                td->ctstat.bits.active,
                td->ctstat.bits.stalled,
                td->ctstat.bits.databuf_err,
                td->ctstat.bits.babble,
                td->ctstat.bits.nak,
                td->ctstat.bits.crcto_err,
                td->ctstat.bits.bitst_err,
                td->ctstat.bits.actlen);
    gdlf_debug ("  token maxlen=%x dt=%d endp=%d addr=%d pid=%x\n",
                td->token.bits.maxlen, td->token.bits.dt, 
                td->token.bits.endp, td->token.bits.addr, 
                td->token.bits.pid);
    gdlf_debug ("  buffer=%x(0x%p)\n",
                td->buffer, 
                bus_to_virt (le32_to_cpu (td->buffer)));
    return;
}


void ts_uhci_qh_printk (ts_uhci_qh *qh)
{
    __u32 *ptr;
    
    gdlf_fntrace ("ts_uhci_qh_printk (0x%p)", qh);
    gdlf_aassert (qh != NULL, return);

    ptr = (__u32*)qh;
    gdlf_debug ("%08x %08x\n", ptr[0], ptr[1]);
    gdlf_debug ("  head ptr=%x(0x%p) Q=%d T=%d\n",
                qh->head.bits.ptr << 4, 
                bus_to_virt (le32_to_cpu (qh->head.bits.ptr << 4)),
                qh->head.bits.q, qh->head.bits.t);
    gdlf_debug ("  elt ptr=%x(0x%p) Q=%d T=%d\n",
                qh->elt.bits.ptr << 4, 
                bus_to_virt (le32_to_cpu (qh->elt.bits.ptr << 4)),
                qh->elt.bits.q, qh->elt.bits.t);
    return;
}


void ts_uhci_tb_printk (ts_uhci_tb *tb)
{
    ts_uhci_td *itr, *top;

    gdlf_fntrace ("ts_uhci_tb_printk (0x%p)", tb);

#ifdef UHCI_DEBUG_TOPS
    if (sec++ > SEC_MAX)
    {
        gdlf_debug ("SEC_MAX reached\n");
        return;
    }    
#endif
    gdlf_aassert (tb != NULL, return);
    
    ts_uhci_qh_printk (&tb->qh);
    gdlf_debug ("  tr = 0x%p\n", tb->tr);
    gdlf_debug ("  td = 0x%p\n", tb->td);
    gdlf_debug ("  tds = %u\n", tb->tds);
    for (itr = tb->td, top = itr + tb->tds; itr < top; itr++)
        ts_uhci_td_printk (itr);
    return;
}


void ts_uhci_flp_printk (ts_uhci_flp *flp)
{
    gdlf_fntrace ("ts_uhci_flp_printk (0x%p)", flp);
#ifdef UHCI_DEBUG_TOPS
    if (sec++ > SEC_MAX)
    {
        gdlf_debug ("SEC_MAX reached\n");
        return;
    }    
#endif
    gdlf_aassert (flp != NULL, return);
    
    gdlf_debug ("%08x\n", flp->word);
    gdlf_debug ("  flp ptr=%x(0x%p) Q=%d T=%d\n",
                flp->bits.ptr << 4, 
                bus_to_virt (le32_to_cpu (flp->bits.ptr << 4)),
                flp->bits.q, flp->bits.t);
    return;
}


    /* 
     * NOTE: This function is identical to uhci_fl_search, so refer to
     *       it for slightly better docs
     */

void ts_uhci_flc_printk (ts_uhci_host *hci, int frnum, int all)
{
    void* ptr;
    ts_uhci_td *td;     /* td iterator */
    ts_uhci_qh *qh;     /* qh iterator */
    ts_uhci_flp *flp;
    char qcontext;      /* 0 - TD context, !0 - QH context */
    char runs;          /* times the search ran on control and bulk queues */
    
    gdlf_fntrace ("ts_uhci_flc_printk (0x%p, %d, %d)", hci, frnum, all);
    gdlf_aassert (hci != NULL, return);
    gdlf_aassert (hci->fl != NULL, return);
    gdlf_aassert (frnum < UHCI_FL_SIZE, return);

    flp = &hci->fl[frnum];
    ts_uhci_flp_printk (flp);
    gdlf_aassert (flp->bits.t != 1, return);

    td = NULL;          /* shut up gcc? */
    qh = NULL;
    if ((qcontext = flp->bits.q))   /* qh context or td context? */
        qh = ptr2kvoid (flp->bits.ptr << 4);
    else
        td = ptr2kvoid (flp->bits.ptr << 4);
    
        /* 
         *  Ok, let's check what we've got at this first frame chain;
         * If there are isochronous TDs, let's advance until we
         * find TDs in EXECUTING, DONE or ERROR status or a QH. When
         * we reach a QH, we just exit this loop and let the flow
         * continue handling it.
         */

    gdlf_debug ("Walking iso TDs\n");
    while (!qcontext)
    {
        gdlf_aassert (td != NULL, return);
        gdlf_aassert (td->lp.bits.t != 1, return);
        ptr = bus_to_virt (le32_to_cpu (td->lp.bits.ptr << 4));
        ts_uhci_td_printk (td);

            /* Now let's get next and see what is it */

        if (td->lp.bits.q)  /* qh context or td context? */
        {
            qh = ptr;
            break;
        }
        else
            td = ptr;
    }
            
        /*
         *  Now we are on QH context for sure; we may be in one of the
         * interrupt queues or at the LS Control queue. How to guess
         * it is done by checking the element pointer; if non
         * terminated and pointing to a QH, it is an interrupt list.
         *
         *  So, what we do at the very end is skip all the QHs with
         * terminated element and dig into the ones with .q bit
         * set. If we find one with the .q bit unset, we exit this
         * loop and continue (it means that qh is a TB, so that's
         * control/bulk transactions).
         */
    
    gdlf_debug ("Walking interrupt queues\n");
    while (qh != &hci->ls_ctrl->first)
    {
        ts_uhci_qh *itr;    /* queue iterator*/

        gdlf_debug ("Interrupt QH list 0x%p\n", qh);
        ts_uhci_qh_printk (qh);
        if (qh->elt.word == cpu_to_le32 (0x3))  /* empty one? check next */
        {
            qh = ptr2kvoid (qh->head.bits.ptr << 4);
            gdlf_aassert (qh != ptr2kvoid (qh->head.bits.ptr << 4), break);
            continue;
        }
        gdlf_aassert (qh->elt.bits.t != 1,      /* make it sure */
                      {
                          qh = ptr2kvoid (qh->head.bits.ptr << 4);
                          continue;
                      });

            /* Ok, walk this queue */

        gdlf_emerg ("Processing interrupt queue 0x%p\n", qh);
        itr = ptr2kvoid (qh->elt.bits.ptr << 4);
        gdlf_aassert (itr->elt.bits.q == 0, break);
        gdlf_aassert (((ts_uhci_tb*)itr)->tr != NULL, break);
        gdlf_aassert (((ts_uhci_tb*)itr)->tds != 0, break);
        gdlf_aassert (((ts_uhci_tb*)itr)->td != NULL, break);
        
        ts_uhci_tb_printk ((ts_uhci_tb*)itr);
        gdlf_aassert (itr != ptr2kvoid (itr->head.bits.ptr << 4), break);

        while (itr->head.bits.t != 1)
        {   
            itr = ptr2kvoid (itr->head.bits.ptr << 4);
                /* make sure this is a TB */

            gdlf_aassert (itr->elt.bits.q == 0, break);
            gdlf_aassert (((ts_uhci_tb*)itr)->tr != NULL, break);
            gdlf_aassert (((ts_uhci_tb*)itr)->tds != 0, break);
            gdlf_aassert (((ts_uhci_tb*)itr)->td != NULL, break);

            ts_uhci_tb_printk ((ts_uhci_tb*)itr);
            gdlf_aassert (itr != ptr2kvoid (itr->head.bits.ptr << 4), break);
        }
        gdlf_assert (itr->head.word == cpu_to_le32 (0x3));
        gdlf_aassert (qh != ptr2kvoid (qh->head.bits.ptr << 4), break);
        qh = ptr2kvoid (qh->head.bits.ptr << 4);
    }

        /* Now we _SHOULD_ be at the ls_ctrl list */

    gdlf_aassert (qh == &hci->ls_ctrl->first, return);
    if (!all)
        return;

        /*
         *  When running the LS Control, Control and Bulk queues, we
         * have to be careful with one thing: the last entry in bulk
         * queue points back to the control queue, and we could be
         * looping forever. Thus, each time we pass trough the Control
         * queue, we inc the 'runs' variable. If it reaches two, it'll
         * mean we have finished.
         */

    gdlf_debug ("Walking ctl/bulk queue\n");
    runs = 0;
    do
    {
        gdlf_debug ("qh is 0x%p\n", qh);
        if (qh->elt.word != cpu_to_le32 (0x3))  /* if not empty ... */
        {   
                /* Make sure this is a TB */

            gdlf_aassert (qh->elt.bits.q == 0, break);
            gdlf_aassert (((ts_uhci_tb*)qh)->tr != NULL, break);
            gdlf_aassert (((ts_uhci_tb*)qh)->tds != 0, break);
            gdlf_aassert (((ts_uhci_tb*)qh)->td != NULL, break);
            ts_uhci_tb_printk ((ts_uhci_tb*)qh);
        }
        else
            ts_uhci_qh_printk (qh);

#ifdef UHCI_DO_BWR
        gdlf_aassert (qh->head.bits.t != 1, break);
#warning FIXME: bandwidth reclamation slows system down!!
#else
        if (qh->head.bits.t == 1)
        {
            gdlf_assert (qh->head.word == cpu_to_le32 (0x3));
            break;
        }
#endif
        gdlf_aassert (qh != ptr2kvoid (qh->head.bits.ptr << 4), break);
        qh = ptr2kvoid (qh->head.bits.ptr << 4);
        if (qh == &hci->ctrl->first)
            runs++;
    } while (runs < 2);

    return;
}


void ts_uhci_fl_printk (ts_uhci_host *hci)
{
    int cnt;

    gdlf_fntrace ("ts_uhci_fl_printk (0x%p)", hci);
    gdlf_aassert (hci != NULL, return);
    gdlf_aassert (hci->fl != NULL, return);

#ifdef UHCI_DEBUG_TOPS
    if (sec++ > SEC_MAX)
    {
        gdlf_debug ("SEC_MAX reached\n");
        return;
    }    
#endif

#ifdef UHCI_DEBUG
    gdlf_debug ("LS Control QH is 0x%p\n", &hci->ls_ctrl->first); 
    gdlf_debug ("Control QH is 0x%p\n", &hci->ctrl->first);
    gdlf_debug ("Bulk QH is 0x%p\n", &hci->bulk->first);
#endif

    ts_uhci_flc_printk (hci, 0, 1);
#warning DEBUG: just printing some entries ... 1024 would be a hit
#if 0
#define FRAMES UHCI_FL_SIZE
#else
#define FRAMES 32
#endif
    for (cnt = 1; cnt < FRAMES; cnt++)
    {
        gdlf_debug ("Printing frame %d, 0x%p\n", cnt, hci->fl+cnt);
        ts_uhci_flc_printk (hci, cnt, 0);
    }
    return;
}

#endif /* UHCI_DEBUG */

/* Log:
 *
 * $Log: generic.c,v $
 * Revision 1.36  1998/11/14 17:45:42  inaky
 * Moved the include files to uusbd/include/uusbd, to ease integration
 * into the Linux kernel.
 *
 * Revision 1.35  1998/11/09 21:43:39  inaky
 * Removed the freeing of the tr_buffer, as now that's a mission of the
 * hub manager.
 *
 * Revision 1.34  1998/10/22 00:08:55  inaky
 * Added some clearifyin comments.
 *
 * Revision 1.33  1998/09/24 16:13:13  inaky
 * Added checks, some tricks to evitate assertions being triggered.
 *
 * Revision 1.32  1998/09/02 21:19:45  inaky
 * Moved uhci.h inclusion after gdlf.
 *
 * Revision 1.31  1998/08/30 13:59:30  inaky
 * Finished new list's linear topology.
 *
 * Revision 1.30  1998/08/29 00:10:33  inaky
 * Fixed assertions when cleaning host structure, endianess on qhs and
 * tds, and option to disable bandwidth reclamation.
 *
 * Revision 1.29  1998/08/27 22:21:54  inaky
 * Moved _free to _release.
 * Made the empty tb_lists in the UHCI host struct to have list->last as
 * NULL, so tb_list_release shuts up.
 * Added the iso and intr hash releasing. Fixed the _printk functions to
 * use the linear list scheme.
 *
 * Revision 1.28  1998/08/25 16:39:42  inaky
 * Cleaned up, removed some stale FIXMEs and severed assertions.
 *
 * Revision 1.27  1998/08/24 02:30:27  inaky
 * Added braces to ts_*_clean() calls inside for loops, as the for loop
 * was only executing the first macro's line.
 *
 * Revision 1.26  1998/08/19 23:56:26  inaky
 * Moved again the irc freeing code to ts_uhci_host_free(), thanks to the
 * addition of a host field in the UHCI host structure.
 *
 * Moved all the QH, TD and some related structures to do aligned
 * allocations.
 *
 * Revision 1.25  1998/08/03 13:41:08  inaky
 * Added some security checks to list transversing functions (compilation
 * time switchable).
 *
 * Revision 1.24  1998/07/30 22:59:54  inaky
 * I F-U-C-K-I-N-G F-O-U-N-D _IT_!! Two damn days hunting down this
 * stupid goddamn bug! Though I still don't understand what was wrong ...
 * :)
 *
 * Revision 1.23  1998/07/30 17:58:51  inaky
 * Fixed to have the lists in the host structure be allocated, to keep
 * them aligned at 16 bytes. Added _DIRTY_ temporary trick to have the
 * QHs aligned at 16 bytes. Fixed a lot of usb_kmalloc()s which had the
 * arguments swapped (!).
 *
 * Added some alignment checks where needed.
 *
 * Added functions to print frame list chains.
 *
 * Revision 1.22  1998/07/28 23:13:30  inaky
 * Moved the frame list size to be fixed.
 *
 * Revision 1.21  1998/07/23 14:43:00  inaky
 * Renamed the ts_uhci_buftds structure to ts_uhci_tb (transfer block).
 *
 * Added housekeeping functions for ts_uhci_tr_hci, ts_uhci_dev_hci,
 * and ts_uhci_tb_list.
 *
 * Revision 1.20  1998/07/16 14:17:04  inaky
 * Added code to handle the tr_buffer member of the ts_uhci_host.
 *
 * Revision 1.19  1998/06/21 20:04:05  inaky
 * Removed the structure cleans from the ts_*_free() methods, as that's
 * up to the caller if he wants to clean or not.
 *
 * Revision 1.18  1998/06/20 21:55:52  inaky
 * Minor tweaks to fix bugs.
 *
 * Revision 1.17  1998/05/01 18:31:27  inaky
 * Moved everything to use pci_dev according to new kernel PCI API. Also
 * split all the code in several files, as old uhci.c was a complete mess.
 *
 * Revision 1.16  1998/04/26 03:42:55  inaky
 * Removed the references to endp->pktsize in favour of endp->maxpktsize.
 *
 * Revision 1.15  1998/04/13 22:18:54  inaky
 * Moved to use the new Linux PCI code. Lot of work still needed.
 *
 * Revision 1.14  1998/01/11 15:44:47  inaky
 * Fixed docs
 *
 * Revision 1.13  1998/01/08 12:19:21  inaky
 * Added copyright strings to all important files
 *
 * Revision 1.12  1998/01/08 11:50:20  inaky
 * Fixed some bad docs :) thanks Alberto!
 *
 * Revision 1.11  1998/01/02 00:12:38  inaky
 * Removed some old code and fixed some bits
 *
 * Revision 1.10  1997/12/31 18:30:05  inaky
 * Finished adapting the UHCI module to the new HCD driver
 *
 * Revision 1.9  1997/12/29 00:26:42  inaky
 * Added function traces and fixed some craps
 *
 * Revision 1.8  1997/12/27 17:25:49  inaky
 * Finished moving to GDLF. Deprecated usb_printk and __LOCATION__
 *
 * Revision 1.7  1997/12/27 02:06:14  inaky
 * Moving everything to use the GDLF for logging and debug checkings.
 * Removed the global array of USB hosts and moved it to a linked
 * list. Added an address field to each USB host controller.
 * Removed lots of cruft that were accumulated and useless. Still some
 * missing, but may be useful someday.
 *
 * Revision 1.6  1997/12/08 02:42:55  inaky
 * Some cosmetic fixes, still missing a deep enhancement
 *
 * Revision 1.5  1997/12/06 00:19:52  inaky
 * Almost finished the usbd/host, usbd/cfg and a lot of meat from
 * usbd/device. Also added the driver definition. Got to polish it yet
 * ...
 *
 * Revision 1.4  1997/10/19 22:20:41  inaky
 * Modified some things, fixed some parameters to functions prototypes
 * and added a few FIXMEs
 *
 * Revision 1.3  1997/10/17 00:19:35  inaky
 * Moved the MOD_DEC_USE_COUNT stuff to a more appropiate place, the
 * function [ou]hci_hci_shutdown, that will take care of shutting down
 * the bus and the host, then clearing the data structures for HCI with
 * ts_uhci_host_free() and cleaning all the methods (thus, undoing the
 * work of uhci_init(). Then it'll do MOD_DEC_USE_COUNT on the uhci
 * module.
 *
 * Revision 1.2  1997/10/02 22:29:46  inaky
 * Finally managed to understand the workings of the MODVERSIONS (ok, not
 * exactly the working, but how to make it work).
 *
 * Now I can go to sleep happy :) -- I know, I'm really sick --
 *
 * Revision 1.1  1997/10/02 21:29:30  inaky
 * Remade all the directory structure to make it cleaner when the USB
 * class drivers come.
 *
 * Revision 1.3  1997/09/30 13:04:42  inaky
 * Moved the name of the package to uusbd: Universal USB Driver
 *
 * Revision 1.2  1997/09/25 00:12:30  inaky
 * At the very least I hunted down the bug which was causing all that
 * damned exceptions, all memory related. Well, I'm a cock-fingered. Ix
 * don't know how to code; I'm stupid; I'm dreadful; I'm ....
 *
 * Revision 1.1  1997/09/08 10:54:49  inaky
 * Modified the generic functions to gain some performance, also some other checks
 *
 * Revision 1.1  1997/02/22 22:25:44  inaky
 * *** empty log message ***
 *
 *
 */

