
/* USB Driver Stack v$Revision: 1.2 $ 
 *
 * (C) 1997 by Iaky Prez Gonzlez <inaky@peloncho.fis.ucm.es>
 *
 * Distributed under the terms of the GNU Public License Version 2
 * See files COPYRIGHT and COPYING at the root distribution directory
 *
 * $Id: piix3.h,v 1.2 1998/09/14 20:36:00 inaky Exp $
 *
 * Changelog at the bottom 
 *
 *
 *     USB host 82371SB PIIX3
 *     ----------------------
 *
 * This is the code to support the host controller embedded in the
 * 82371SB (PIIX3) IDE Xaccelerator. All the docs for this chipset can be
 * obtained from the Intel's website at http://www.intel.com. Search for
 * document (I guess it is) number 290550, though I think I'm pretty wrong and
 * you'd better go to Products/PCI chipsets, 430VX, and pick it from there. It
 * is a BIG .pdf file.
 *
 */

#ifndef _piix3_h_
#define _piix3_h_

    /* Masks for PCI registers of the PIIX3 (offset, size, default
     * value) 
     * ----------------------------------------------------------- 
     */

    /* Device identification register (dev id for PIIX3) */

#define PCIDI 0x2
#define PCIDI_DIN PCI_DEVICE_ID_INTEL_82371SB_2

    /* PCI Command Register bits (PCI_COMMAND, 16 bits, [0x7020]) */

#define PCICMD      PCI_COMMAND
#define PCICMD_BME  0x0004  /* RW - Bus master enable. 1 - allows the
                               device to behave as a USB host
                               controller bus master (and thus, it
                               must be set before serial transactions
                               start. 0 - disables the USB host from
                               generating PCI accesses */
#define PCICMD_IOSE 0x0001  /* RW - I/O space enable. Enable (1) or
                               disable (0) I/O acceses to the USB host
                               controller's I/O space.  The base I/O
                               register address must be programmed
                               before setting this bit.*/


    /* PCI Device Status Register bits (PCI_STATUS, 16 bits) */

#define PCIDS_MAS   0x2000  /* RWC - Master-Abort status. Set to 1
                               When the USB host generates a
                               master-abort [on the PCI bus, I
                               guess]. Software resets */
#define PCIDS_RTA   0x1000  /* RWC - Received target-abort status. Set
                               to 1 when the USB host is a master on
                               the PCI bus and receives a
                               target-abort. Software resets */
#define PCIDS_STA   0x0800  /* RwC - Signaled target-abort status. Set
                               when the USB host module function is
                               targeted with a transaction which it
                               terminates with a target
                               abort. Software resets. */
#define PCIDS_DEVT  0x0600  /* RO - DEVSEL# timing status. These 2
                               bits indicate the timing for DEVSEL#
                               assertions when performing a positive
                               decode. Since the PIIX3 always
                               generates the DEVSEL# with medium
                               timing, then DEVT = 01. This timing
                               doesn't include configuration
                               cycles. */
#define PCIDS_FBC   0x0080  /* RO - Fast back to back capable. Always
                               set, indicating to the PCI Master that
                               the USB host controller is capable of
                               accepting fast back-to-back
                               transactions as a target */



    /* PCI Master latency timer register (PCI_LATENCY_TIMER, 8 bits) */

#define PCMLT_MLTCV 0xf0    /* RW - Master latency counter
                               value. Number of PCI bus clock cycles
                               which may last a PCI transaction
                               initiateb by the USB controller (only
                               if PHLDA# is negated after the
                               transaction is initiated). */

    /* PCI I/O Space base address register (PCI_BASE_ADDRESS, 32 bits, [0]) */

#define PCIIOB_IRBA 0x0000fe0   /* RW - Index register base
                                   address. This is the base I/O
                                   address for the USB host controller */
#define PCIIOB_RTE  0x0000001   /* RO - Resource Type
                                   Indicator. Hardwired to 1,
                                   indicating that this register maps
                                   to I/O space */

    /* PCI Interrupt line register (PCI_INTERRUPT_LINE, 8 bits, [00]) */

#define PCIIL_IL    0xff    /* RO - Interrupt line being used (0-0xff :) */

    /* PCI Interrupt pin register (PCI_INTERRUPT_PIN, 8 bits, [0x04]) */

#define PCIIP_SBMIR 0x07    /* RO - PCI Interrupt pin to be used by
                               the USB host controller. The USB
                               interrupt is internally ORed to the
                               Interrupt controller with the PIRQD#
                               signal. */

    /* PCI Serial bus release number (PCI_USBREV, 8 bits, [0x00]) */

#define PCI_USBREV    0x60
#define PCIUSBREV_REV 0xff  /* RO - USB Specification compliance
                               revision number: 0x00 Pre-release 1.0,
                               0x01 Release 1.0 */
 
    /* PCI Miscellaneous status register (PCI_USBMSTAT, 16 bits, [0x0001]) */

#define PCI_USBMSTAT    0x6a
#define PCIMSTAT_USBCLK 0x0001  /* RW - USB Clock selection (1 = 48
                                   MHz, 0 = 24 MHz) */

    /*
     * PCI Legacy support registers (0xc0, 16 bits, [0x2000]) 
     */

#define PCILEGSUP               0xc0
#define PCILEGSUP_A20PTS        0x8000
#define PCILEGSUP_RESERVED      0x4000
#define PCILEGSUP_USBPIRQDEN    0x2000
#define PCILEGSUP_USBIRQS       0x1000
#define PCILEGSUP_TBY64W        0x0800
#define PCILEGSUP_TBY64R        0x0400
#define PCILEGSUP_TBY60W        0x0200
#define PCILEGSUP_TBY60R        0x0100
#define PCILEGSUP_SMIEPTE       0x0080
#define PCILEGSUP_PSS           0x0040
#define PCILEGSUP_A20PTEN       0x0020
#define PCILEGSUP_USBSMIEN      0x0010
#define PCILEGSUP_64WEN         0x0008
#define PCILEGSUP_64REN         0x0004
#define PCILEGSUP_60WEN         0x0002
#define PCILEGSUP_60REN         0x0001


#endif /* _piix3_h_ */

/* Log:
 *
 * $Log: piix3.h,v $
 * Revision 1.2  1998/09/14 20:36:00  inaky
 * Fixed the legacy support register address.
 *
 * Revision 1.1  1998/08/24 15:46:52  inaky
 * Fixed little cosmetic stuff, removed old files.
 *
 * Revision 1.6  1998/07/30 17:50:37  inaky
 * Added some missing definitions
 *
 * Revision 1.5  1998/01/08 12:19:08  inaky
 * Added copyright strings to all important files
 *
 * Revision 1.4  1997/12/27 02:07:01  inaky
 * Moving everything to use the GDLF for logging and debug checkings.
 * Removed the global array of USB hosts and moved it to a linked
 * list. Added an address field to each USB host controller.
 * Removed lots of cruft that were accumulated and useless. Still some
 * missing, but may be useful someday.
 *
 * Revision 1.3  1997/12/08 02:40:58  inaky
 * Fixed some stuff of the #includes, modified the makefile targets to be
 * more flexible, tried to finish the device configuration section.
 *
 * Revision 1.2  1997/04/06 21:14:32  inaky
 * Still with the definition of the access interface. More or less it is
 * taking an stable form.
 *
 * Revision 1.1  1997/02/22 22:25:44  inaky
 * *** empty log message ***
 *
 *
 */






















