
/* USB Driver Stack v$Revision: 1.24 $ 
 *
 * (C) 1997 by Iaky Prez Gonzlez <inaky@peloncho.fis.ucm.es>
 *
 * Distributed under the terms of the GNU Public License Version 2
 * See files COPYRIGHT and COPYING at the root distribution directory
 *
 * $Id: uhci.h,v 1.24 1998/11/14 17:45:49 inaky Exp $
 *
 * Changelog at the bottom 
 *
 *     Internal UHCI stuff
 *     -------------------
 */

    /*
     *  On little endian, gcc aligns first fields at the lowest
     * addresses (that's little endian :). Still have to see if on big
     * endian it does it the other way around or what happens ... :(
     */

#ifdef __BIG_ENDIAN
#error FIXME: Bitfields may break! It is not checked!
#endif

#ifndef _uhci_h_
#define _uhci_h_

#include <linux/config.h>
#include <linux/timer.h>

#include <uusbd/usb.h>
#include <uusbd/usb-pci.h>
#include <uusbd/usb-hub.h>
#include <uusbd/usb-misc.h>


    /* 
     * Control of the UHCI module 
     * --------------------------
     */

#define UHCI_DEBUG              /* Want debug stuff on UHCI */
#if 0
#undef UHCI_DEBUG_TIMER         /* Want debug stuff on UHCI (each seconds) */
#else
#define UHCI_DEBUG_TIMER 3
#undef UHCI_SBYS                /* Want step by step */
#undef UHCI_DEBUG_TIMER_FL     /* do fl search on debug timer */
#endif
#define UHCI_CTL_STR    USB_CTL_STR "-uhci"
#define UHCI_NAME       "UHCI"
#define UHCI_RETRIES    3       /* Number of retries the hw will do */
#define UHCI_ACTLEN     0x7fe   /* Length used as marker for inserted state */
#if 1
#define UHCI_CLEANUP_T  10      /* Cleanup tasks fires each X seconds */
#else
#define UHCI_CLEANUP_T  120     /* Cleanup task fires each X seconds */
#endif
#define UHCI_HUB_T      5       /* Poll root hub each 5 seconds */

    /* 
     * WARNING: bandwidth reclamation, I dunno why, slows my system
     *          down a damn lot. use at your own risk ... 
     *
     * FIXME: Ok, on empty list, it stupidly loops around and fills
     *        the bus with accesses. So the sollution is to terminate
     *        the list when no transactions are active and loop it
     *        when some one is active.
     */

#undef UHCI_DO_BWR              /* define to do bandwidth reclamation */

#if !defined (UHCI_DEBUG_TIMER) && defined (UHCI_SBYS)
#error UHCI_SBYS requires UHC_DEBUG_TIMER
#endif

#if !defined (UHCI_DEBUG_TIMER) && defined (UHCI_DEBUG_TIMER_FL)
#error UHCI_SBYS requires UHC_DEBUG_TIMER
#endif

    /* 
     * Global variables of the UHCI module
     * -----------------------------------
     */

extern volatile int uhci_errno;  /* error code; changed only on error */


    /* 
     * UHCI registers addresses and masks (addr relative to base register) 
     * ------------------------------------------------------------------- 
     */

#define UHCI_FL_SIZE  1024      /* Size of the UHCI frame list */
#define UHCI_IOEXTENT 0x14      /* IO ports used by an UHCI controller */

    /* USBCMD USB command register RW (0x00, 16 bits, [0x0000]) */

#define USBCMD          0x00
#define USBCMD_MAXP     0x0080
#define USBCMD_CF       0x0040
#define USBCMD_SWDBG    0x0020
#define USBCMD_FGR      0x0010
#define USBCMD_EGSM     0x0008
#define USBCMD_GRESET   0x0004
#define USBCMD_HCRESET  0x0002
#define USBCMD_RS       0x0001

    /* USBSTS USB status register RWC (0x02, 16 bits, [0x0000]) */

#define USBSTS          0x02
#define USBSTS_HCH      0x0020
#define USBSTS_HCPE     0x0010
#define USBSTS_PCIBE    0x0008
#define USBSTS_RR       0x0004
#define USBSTS_USBEI    0x0002
#define USBSTS_USBINT   0x0001

    /* USBINTR USB interrupt enable register RW (0x04, 16 bits, [0x0000]) */

#define USBINTR         0x04
#define USBINTR_SPIEN   0x0008
#define USBINTR_IOCEN   0x0004
#define USBINTR_REN     0x0002
#define USBINTR_TOCRCEN 0x0001

    /* FRNUM USB Frame number register RW (0x06, 16 bits, [0x000]) */

#define USBFRNUM        0x06
#define USBFRNUM_M      0x03ff
 

    /* FLBASEADD USB frame list base address register RW (0x08, 32 bits, [undefined]) */

#define USBFLBASEADD    0x08
#define USBFLBASEADD_M  0xfffff000

    /* SOFMR USB Start of frame (SOF) modify register RW (0x0c, 8 bits, [0x40]) */

#define USBSOF          0x0c
#define USBSOF_M        0x7f

    /* PORTSC1 USB Port 1 status and control register RW (0x10 16 bits, [0x0080]) */
    /* PORTSC2 USB Port 2 status and control register RW (0x12 16 bits, [0x0080]) */

#define USBPORTSC1      0x010
#define USBPORTSC2      0x012
#define USBPORTSC_SUSP  0x1000  /* RW port 0 - not, 1 - in suspend state */
#define USBPORTSC_PR    0x0200  /* RW port 0 - not, 1 - is on reset */
#define USBPORTSC_LSDA  0x0100  /* RO device attached is 0 - full, 1 - low speed*/
#define USBPORTSC_RD    0x0040  /* RO resume detected/driven 0 - no, 1 - yes */
#define USBPORTSC_LS_DN 0x0020  /* RO D+ line logical level */
#define USBPORTSC_LS_DP 0x0010  /* RO D- line logical level */
#define USBPORTSC_PEDC  0x0008  /* RWC Port enable/disable status 0 - not, 1 - changed*/
#define USBPORTSC_PED   0x0004  /* RW Port 0 - disable, 1 - enable*/
#define USBPORTSC_CSC   0x0002  /* RWC connect status 0 - hasn't, 1 - has changed */
#define USBPORTSC_CCS   0x0001  /* RO device is 0 - detached, 1 - attached to the port */

    /* Defaults */

#define USB_UHCI_SOF_DEFAULT 0x40

    /* Macros */

#define UHCI(hci) ((ts_uhci_host*)(hci))


    /* Types of behaviours when searching the frame list */

#define UHCI_FL_FAST    0   /* repair first and exit */
#define UHCI_FL_ALL     1   /* check all the frame chain */
#define UHCI_FL_INI     2   /* check only iso and interrupt chains */


    /* UHCI datatypes 
     * -------------- 
     *
     * FIXME: I've read on the C++ ANSI Standard that bitfields vary wildly
     *        from machine to machine (ok, it is not C, but is backwards
     *        compatible :) in terms of alignment, sides ... etc. I'll
     *        assume GNU CC is a nice kid and will stack bitfields from LSB
     *        to MSB in inverse declaration order 
     *
     * FIXME: Check this out, I think I wrote it for Big Endian
     *        ... something has to be done for little endian machines
     *        (appart from byte swapping :)
     */


    /* Frame list pointer */

typedef union
{
    struct
    {
        unsigned int t:1;       /* 1 Empty frame (invalid ptr), 0 valid frame */
        unsigned int q:1;       /* 1 Points to a Queue Head, 0 Points to a Transfer Descriptor */
        unsigned int reserved:2;/* Reserved */
        unsigned int ptr:27;    /* Address of the first QH/TD to be processed in the frame (16 byte aligned) */
    } bits;
    __u32 word;
} ts_uhci_flp;

#ifdef __KERNEL__
#define ts_uhci_flp_clean(_flp)                 \
    (_flp)->word = cpu_to_le32 (0x1)

extern ts_uhci_flp* ts_uhci_flp_init (void);
extern void ts_uhci_flp_release (ts_uhci_flp*);
extern ts_uhci_flp* ts_uhci_flp_a_init (ts_uhci_flp *flps, unsigned int);
extern void ts_uhci_flp_a_release (ts_uhci_flp*, unsigned int);
#endif


    /* 
     * Transfer descriptor 
     * 
     * R = read by the UHCI controller
     * W = writable by the UHCI controller 
     */

typedef struct
{
    union
    {
        struct
        {
            unsigned t:1;           /* R - 1 Empty frame (invalid ptr), 0 valid frame */
            unsigned q:1;           /* R - 1 Points to a Queue Head, 0 Points to a Transfer Descriptor */
            unsigned vf:1;          /* R - 1 Process in depth, 0 process in breadth */
            unsigned reserved:1;    /*     Reserved  */
            unsigned ptr:28;        /* R - Address of the next QH/TD to be processed in the frame (16 byte aligned) */
        } bits;
        __u32 word;
    } lp;

    volatile union
    { 
        struct
        {
            unsigned actlen:11;     /* W - Actual number of bytes transfered (n-1, in fact). cref maxlen */
            unsigned reserved1:6;   /*     Reserved */
            unsigned bitst_err:1;   /* W - Bitstuff error (1) */
            unsigned crcto_err:1;   /* W - CRC (receive) / Timeout (transmit) error (1) */
            unsigned nak:1;         /* W - NAK received (1) - NAK from SETUP transactions sets Time Out Errors as well */
            unsigned babble:1;      /* W - Babble detected during transaction (1) - STALLED is set too and the current frame is "lost" */
            unsigned databuf_err:1; /* W - Overrun(rx)/underrun(tx) detected (actlen != maxlen). CRC/Timeout forced. TD is invalidated */
            unsigned stalled:1;     /* W - Babble / error count == 0 / STALL (Timeout reported to). Active bit is set */
            unsigned active:1;      /* W - 1 to enable this TD. Set to 0 on completion/stall of the transaction */
            unsigned ioc:1;         /* ? - 1 issue an interrupt on completion (even if active==0) at end of frame for this TD */
            unsigned iso:1;         /* ? - 1 isochronous transfer (host'll always set active=0, regardless of the result of transaction) */
            unsigned ls:1;          /* ? - 1 target device is a low speed one */
            unsigned c_err:2;       /* W - Error retry count allowed for this TD (0 = no limit) after being marked inactive (active = 0) */
            unsigned spd:1;         /* W - If 1 and the packet is input, queued and completes with actlen < maxlen -> TD inactive, USBSTS_INT is set to 1 */
            unsigned reserved2:2;   /*     Reserved */
        } bits;
        __u32 word;
    } ctstat;

    union
    {
        struct
        {
            unsigned pid:8;         /* R - Packet IDentification (IN 0x69, OUT 0xe1 'n SETUP 0x2d allowed) */
            unsigned addr:7;        /* R - Device's address (0-127) */
            unsigned endp:4;        /* R - Device's endpoint (0-16) */
            unsigned dt:1;          /* R - Data toggle */
            unsigned reserved0:1;   /*     Reserved */
            unsigned maxlen:11;     /* R - Maximum Length (0 = 1 byte, 0x4ff = 1280 bytes, 0x7ff = 0 bytes, 0x500-0x7fe invalid */
        } bits;
        __u32 word;
    } token;

    __u32   buffer;             /* R - Buffer pointer */

} ts_uhci_td;

#ifdef __KERNEL__
#define ts_uhci_td_clean(td)                    \
    (td)->buffer = 0;                           \
    (td)->token.word = 0;                       \
    (td)->ctstat.word = 0;                      \
    (td)->lp.word = cpu_to_le32 (0x1);

extern ts_uhci_td* ts_uhci_td_init (void);
extern void ts_uhci_td_release (ts_uhci_td*);
extern ts_uhci_td* ts_uhci_td_a_init (unsigned int);
extern void ts_uhci_td_a_release (ts_uhci_td*, unsigned int);
#endif


    /* Queue Head Link Pointer */

typedef struct
{
    union
    {
        struct              /* Pointer to the first data object in the queue */
        {
            unsigned t:1;       /* R - 1 No more entries (invalid ptr), 0 valid pointer */
            unsigned q:1;       /* R - 1 Points to a Queue Head, 0 Points to a Transfer Descriptor */
            unsigned reserved:2;
            unsigned ptr:28;    /* R - Address of next data object to be processed */  
        } bits;
        __u32 word;
    } head;

    volatile union
    {
        struct              /* Pointer to the next object to be processed after this queue is done */
        {
            unsigned t:1;       /* R - 1 Last QH (invalid ptr), 0 valid pointer */
            unsigned q:1;       /* R - 1 Points to a Queue Head, 0 Points to a Transfer Descriptor */
            unsigned reserved:2;
            unsigned ptr:28;    /* R - Address of next data object to be processed */  
        } bits;
        __u32 word;
    } elt;
} ts_uhci_qh;

#ifdef __KERNEL__
#define ts_uhci_qh_clean(_qh)                   \
do                                              \
{                                               \
    (_qh)->head.word = cpu_to_le32 (0x1);       \
    (_qh)->elt.word = cpu_to_le32 (0x1);        \
} while (0)
extern ts_uhci_qh* ts_uhci_qh_init (void);
extern void ts_uhci_qh_release (ts_uhci_qh*);
extern ts_uhci_qh* ts_uhci_qh_a_init (unsigned int);
extern void ts_uhci_qh_a_release (ts_uhci_qh*, unsigned int);
#endif


    /*
     * UHCI Transaction Block
     * ----------------------
     *
     *  This keeps all the info needed for fast access to the UHCI
     * transfer information. Note the qh memeber MUST be the first, so
     * that the pointer to the TBs QH is the pointer to the TB (when
     * walking the transfer list, we get QH's addresses, it's nice
     * to get that fast TB's address).
     */

typedef struct s_uhci_tb ts_uhci_tb;

struct s_uhci_tb
{
        /* FIXME: locks? */
    ts_uhci_qh  qh;     /* queue head (unused on isochronous) */
    ts_usb_tr   *tr;    /* transaction structure we belong to */
    ts_uhci_td  *td;    /* array of tds (NULL = invalid) */
    __u16       tds;    /* number of tds (0 = invalid) */
};

#ifdef __KERNEL__
#define ts_uhci_tb_clean(tb)                                        \
    ts_uhci_qh_clean (&(tb)->qh);                                   \
    (tb)->tr = NULL;                                                \
    (tb)->td = NULL;                                                \
    (tb)->tds = 0;                                                  \


extern ts_uhci_tb* ts_uhci_tb_init (void);
extern void ts_uhci_tb_release (ts_uhci_tb*);    
#endif

#if defined (__KERNEL__) && defined (UHCI_DEBUG)
void ts_uhci_td_printk (ts_uhci_td*);
void ts_uhci_tb_printk (ts_uhci_tb*);
void ts_uhci_qh_printk (ts_uhci_qh*);
void ts_uhci_flp_printk (ts_uhci_flp*);
#endif


    /*
     * Structure for managing a Device list of TBs 
     * -------------------------------------------
     *
     *  This device contains a list of TB's started with a QH. It also
     * contains a rwlock for messing with the data and a time-stamp of
     * the last time a transaction was INSERTED. With that data, on
     * each insertion, the 'period' is recalculated.
     *
     *  When NOT inserted into any list, the list.head is all zeroes;
     * when the list is empty, the list.element.terminate bit is set.
     *
     *  The 'list' must be the first one so the tb_list's address is
     * the same as the qh it uses to get linked.
     */

typedef struct s_uhci_tb_list ts_uhci_tb_list;
struct s_uhci_tb_list
{
    ts_uhci_qh  first;  /* list of TEs queued here - MUST BE FIRST! */
    ts_uhci_qh  *last;  /* last element queued (&first if none) */
    rwlock_t    lock;
};

#ifdef __KERNEL__
#define ts_uhci_tb_list_clean(tb_list)              \
do                                                  \
{                                                   \
    ts_uhci_qh_clean (&(tb_list)->first);           \
    (tb_list)->last = &(tb_list)->first;            \
    __USB_SMP ((tb_list)->lock = RW_LOCK_UNLOCKED); \
} while (0)
extern ts_uhci_tb_list* ts_uhci_tb_list_init (void);
extern void ts_uhci_tb_list_release (ts_uhci_tb_list*);
#endif /* __KERNEL__ */


    /*
     * Sructure to keep transactions's HCI data
     * ----------------------------------------
     *
     */

typedef struct s_uhci_tr_hci ts_uhci_tr_hci;
struct s_uhci_tr_hci
{
    ts_uhci_tb tb;
};

#ifdef __KERNEL__
extern void uhci_tr_timer (ts_uhci_tb*);
#define ts_uhci_tr_hci_clean(tr_hci)            \
    ts_uhci_tb_clean (&(tr_hci)->tb);

extern ts_uhci_tr_hci* ts_uhci_tr_hci_init (void);
extern void ts_uhci_tr_hci_release (ts_uhci_tr_hci*);
#endif /* __KERNEL__ */


    /*
     * Isochronous hash
     * ----------------
     *
     *  This makes it easy to guess to which TB an isochronous TD
     * belongs to.
     */

typedef struct s_uhci_iso_hash ts_uhci_iso_hash;
struct s_uhci_iso_hash
{
    ts_uhci_tb   **hash; /* array of pointers to tbs ... */
    rwlock_t     lock;
    atomic_t     usecnt;
    size_t       used;   /* entries filled in */
    size_t       slots;  /* entries allocated */
};

#ifdef __KERNEL__
#define ts_uhci_iso_hash_clean(hash)                \
do                                                  \
{                                                   \
    memset ((hash), 0, sizeof (*hash));             \
    atomic_set (&(hash)->usecnt, 0);                \
    __USB_SMP ((hash)->lock = RW_LOCK_UNLOCKED);    \
} while (0);

static inline
ts_uhci_iso_hash* ts_uhci_iso_hash_init (void)
{
    ts_uhci_iso_hash *hash;    
    hash = usb_kmalloc (sizeof (ts_uhci_iso_hash), GFP_KERNEL);
    if (hash == NULL)
        return NULL;
    ts_uhci_iso_hash_clean (hash);
    return hash;
}

static inline
void ts_uhci_iso_hash_release (ts_uhci_iso_hash* hash)
{
    gdlf_aassert (hash != NULL, return);
    if (hash->hash)
    {
#ifdef GDLF_ASSERTIONS
        {
            int cnt;
            for (cnt = 0; cnt < hash->slots; cnt++)
                if (hash->hash[cnt] != NULL)
                    gdlf_err ("iso 0x%p has non-null tb %d to 0x%p\n", 
                              hash, cnt, hash->hash[cnt]);
        }
#endif
        usb_kfree (hash->hash);
    }
    return;
}
#endif /* __KERNEL__ */


    /*
     * Interrupt lists & hash
     * ----------------------
     *
     *  See intr-hash.c for info.
     */

typedef struct s_uhci_intr_hash ts_uhci_intr_hash;
struct s_uhci_intr_hash
{
    rwlock_t            lock;       /* array lock */
    atomic_t            usecnt;     /* being actually used */
    ts_uhci_tb_list     **slot;     /* array of pointers to list head structures */
    ts_uhci_tb_list     *top;       /* pointer to top [higher interval] list head */
    __u8                size;
};

#ifdef __KERNEL__
#define ts_uhci_intr_hash_clean(hash)               \
do                                                  \
{                                                   \
    memset ((hash), 0, sizeof (*hash));             \
    atomic_set (&(hash)->usecnt, 0);                \
    __USB_SMP ((hash)->lock = RW_LOCK_UNLOCKED);    \
} while(0);

static inline
ts_uhci_intr_hash* ts_uhci_intr_hash_init (void)
{
    ts_uhci_intr_hash *hash;    

    gdlf_fntrace ("ts_uhci_intr_hash_init (void)");

    hash = usb_kmalloc (sizeof (ts_uhci_intr_hash), GFP_KERNEL);
    if (hash == NULL)
        return NULL;
    ts_uhci_intr_hash_clean (hash);
    return hash;
}

static inline
void ts_uhci_intr_hash_release (ts_uhci_intr_hash* hash)
{
    ts_uhci_tb_list **itr, **top;
    
    gdlf_fntrace ("ts_uhci_intr_hash_release (0x%p)", hash);
    gdlf_aassert (hash != NULL, return);

    if (hash->slot)
    {
        for (itr = hash->slot, top = itr + hash->size; itr < top; itr++)
            if (*itr)
            {
                ts_uhci_tb_list_release (*itr);
                usb_kfree (*itr);
            }
        usb_kfree (hash->slot);
    }
    return;
}
#endif /* __KERNEL__ */


    /* 
     * UHCI host controller
     * --------------------
     *
     * DESTRUCTOR: uhci_host_destroy()
     *
     *  The 'frbot' field is used when searching the frame list for
     * changes. When an interrupt is called, it searches from the
     * current frame down to the one after 'frbot', and then it set's
     * 'frbot' to the current frame number, before exiting. This will
     * cause problems if we stay a _full_ frame list run (1024 seconds)
     * without updating it, as 'frbot' should be always be behind the
     * current frame number [if not, it would mean some frame list
     * entries haven't been explored].
     */

typedef struct
{
    ts_snode        snode;      /* destructor, usecount and node */
                                
    unsigned int    iobase;     /* IO base for accesing */
    unsigned int    ioextent;   /* IO addresses used from iobase */
    unsigned int    irq;        /* IRQ line */
    ts_usb_host     *host;      /* USB host pointer. RO only, no reference counting here */
    
#ifdef CONFIG_PCI               
    struct pci_dev  *pci;       /* PCI device structure */
#endif

    unsigned char   sof_timing; /* Value of the SOF timing register */

        /* Frame list */

    atomic_t        sem;        /* semaphore for IRQ handling */
    ts_uhci_flp     *fl;        /* Frame list */
        /* Frame list size is always UHCI_FL_SIZE */
    unsigned        frbot;      /* last frame revised */
    unsigned        errors;     /* hard errors (HCPE and PCIBE) */
    
        /* Need to be pointers, as we need them aligned to 16 bytes */

    ts_uhci_tb_list     *ls_ctrl;   /* Low Speed Control QH list */
    ts_uhci_tb_list     *ctrl;      /* Control QH list */
    ts_uhci_tb_list     *bulk;      /* Bulk QH list */
    ts_uhci_iso_hash    iso;        /* Iso hash */
    ts_uhci_intr_hash   intr;       /* interrupt lists and hash */
    
#ifdef UHCI_DEBUG_TIMER
    struct timer_list timer;
#endif
} ts_uhci_host;

#ifdef __KERNEL__
#define ts_uhci_host_clean(hci)                 \
    memset (hci, 0, sizeof (ts_uhci_host));     \
    ts_snode_clean (&(hci)->snode);             \
    atomic_set (&(hci)->sem, 0);                \
    (hci)->sof_timing = USB_UHCI_SOF_DEFAULT;

ts_uhci_host* ts_uhci_host_init (void);
void ts_uhci_host_release (ts_uhci_host*);
#ifdef UHCI_DEBUG
void ts_uhci_host_printk (ts_uhci_host*);
void ts_uhci_fl_printk (ts_uhci_host*);
#endif
#endif /* __KERNEL__ */


    /* 
     * UHCI functions for the kernel 
     *
     *  In fact, the kernel only needs uhci_init(), but I need to
     * declare them somewhere and this looks the best :)
     */


#ifdef __KERNEL__

    /* vhub.c */

extern ssize_t uhci_hub_descriptor_get (ts_usb_hub*, __u8, void*, 
                                        size_t);
extern int uhci_hub_descriptor_fill (ts_usb_hub*);
extern __s32 uhci_hub_status_get (ts_usb_hub*);
extern __s8 uhci_hub_bus_status_get (ts_usb_hub*, __u8);
extern int uhci_hub_feature (ts_usb_hub*, e_usb_hub_featsel, e_usb_req);
extern int uhci_port_feature (ts_usb_hub*, __u8,
                              e_usb_port_featsel, e_usb_req);
extern __s32 uhci_port_status_get (ts_usb_hub*, __u8);
extern int uhci_port_reset (ts_usb_hub*, __u8);
extern int uhci_hub_free (ts_usb_hub*);
extern const ts_usb_hub_ops uhci_hub_operations;


    /* driver.c */

extern ts_usb_hci* uhci;        /* pointer to UHCI's HCI struct */
extern int uhci_errno;          /* UHCI errno code */

extern int uhci_host_destroy (ts_uhci_host*);
#ifdef CONFIG_PCI
extern int uhci_pci_init (ts_usb_host*, struct pci_dev*);
extern int uhci_pci_enumerate (void);
#endif
extern int uhci_host_init (ts_usb_host*);
extern int uhci_init (void);
extern int uhci_reference (ts_snode*, int);
extern int uhci_release (void);
extern int uhci_shutdown (void);

    /* intr-hash.c */

extern void intr_hash_cleanup (ts_uhci_host*, unsigned long);
extern ts_uhci_tb_list* intr_hash_list (ts_uhci_host*, __u8);

    /* iso-hash.c */

extern void iso_hash_cleanup (ts_uhci_iso_hash*, unsigned long);
extern int iso_hash_add (ts_uhci_iso_hash*, ts_uhci_tb*);
extern void iso_hash_del (ts_uhci_iso_hash*, ts_uhci_tb*);
extern ts_uhci_tb* get_iso_tb_by_td (ts_uhci_iso_hash*, ts_uhci_td*);


    /* operations.c */

#ifdef UHCI_DEBUG
extern int uhci_dumpreg (ts_uhci_host*);
#endif
extern __u16 uhci_get_frnum (ts_uhci_host*);
extern int uhci_hci_busreset (ts_usb_host*);
extern int uhci_hci_reset (ts_usb_host*);
extern int uhci_hci_setsof (ts_usb_host*, unsigned);
extern int uhci_hci_shutdown (ts_uhci_host*);
extern int uhci_host_reset (ts_uhci_host*);
extern int uhci_maxp (ts_uhci_host*, int);
extern int uhci_runstop (ts_uhci_host*, int);
extern int uhci_set_frnum (ts_uhci_host*, unsigned int);


    /* scheduler.c */

extern void qh_list_append (ts_uhci_tb_list*, ts_uhci_qh*);
extern void qh_ilist_append (ts_uhci_tb_list*, ts_uhci_qh*);
extern ts_uhci_qh* qh_list_prev (ts_uhci_tb_list*, ts_uhci_qh*);
extern void qh_list_delete (ts_uhci_tb_list*, ts_uhci_qh*);
extern void qh_ilist_delete (ts_uhci_tb_list*, ts_uhci_qh*);
extern int uhci_tb_fill (ts_uhci_tb*);
extern int uhci_tb_reset (ts_uhci_tb*, e_usb_tr_type);
extern void uhci_tb_activate (ts_uhci_tb*);
extern int uhci_tr_abort (ts_usb_tr*);
extern int uhci_tr_insert (ts_usb_tr*);
extern int uhci_tr_remove (ts_usb_tr*);
extern int uhci_tr_reset (ts_usb_tr*);
extern int uhci_tr_run (ts_usb_tr*);
extern int uhci_tr_setdn (ts_usb_tr*);
extern int uhci_tr_setup (ts_usb_tr*);


    /* irq-handler.c */

extern void move_to_error (ts_uhci_tb*, int, int);
extern void update_statistics (ts_usb_tr*);
extern void uhci_irqhandler (int, ts_usb_host*, struct pt_regs*);
extern void uhci_fl_search (ts_uhci_host*, unsigned, __u8, int);


    /*
     * Inlined functions for UHCI 
     * --------------------------
     */

    /* Send a global reset on the USB bus (write a 1 to USBCMD_GRESET,
     * and then, 10 ms after, set it to 0). Also reset the host.
     *
     * Parameters:  
     *
     *  ts_uhci_host*   Pointer to the host controller structure of
     *                  the host.
     *
     * Returns:
     *
     *  int             0 if ok, errno code < 0 on error.
     *
     * Global:
     *
     *  int usb_errno   On error, error code.  
     */

static inline
int uhci_bus_reset (ts_uhci_host* hci)
{
    gdlf_fntrace ("uhci_bus_reset (0x%p)", hci);
    gdlf_aassert (hci != NULL, return -EINVAL);
    
       /* do the reset, wait 10 ms and reset bits */

    write_lock (&hci->snode.lock);
    outw (USBCMD_GRESET, hci->iobase + USBCMD);
    usb_waitms (15);        /* 10 ms gives trouble */
    outw (0, hci->iobase + USBCMD);
    write_unlock (&hci->snode.lock);
    return 0;
}


    /* Set/reset the configure flag on the controller. 1 means we are
     * still configurating it.
     *
     * Parameters:  
     *
     *  ts_uhci_host*   Pointer to the host controller structure of
     *                  the host.
     *
     *  int             Action to take
     *                  -1  get actual value
     *                  0   reset
     *                  1   set
     *
     * Returns:
     *
     *  int             The state of the bit before changing it (or
     *                  actual if action is -1); < 0 errno code on
     *                  error. 
     *
     * Global:
     *
     *  int usb_errno   On error, error code.  
     */

static inline
int uhci_configure (ts_uhci_host* hci, int action)
{
    int errcode;
    
    gdlf_fntrace ("uhci_configure (0x%p)", hci);

    gdlf_apassert (hci != NULL,
                   { 
                       uhci_errno = usb_errno = -USBE_HCI_NULL; 
                       return -EINVAL; 
                   },
                   "NULL pointer to UHCI host controller structure\n");    
    
    write_lock (&hci->snode.lock);
    errcode = (usb_bits16 (hci->iobase + USBCMD, action << 6, 
                           action == -1? 0 : USBCMD_CF) 
               & USBCMD_CF) >> 6;
    write_unlock (&hci->snode.lock);
    return errcode;
}


    /* Put the USB bus into the global suspend mode. Before modifying
     * anything, we got to make sure that the controller is stopped by
     * setting the USBCMD_RS to zero, waiting until USBSTS_HCH is 1
     * (controller halted) and then we can work.
     *
     * Parameters:
     *
     *  ts_uhci_host*   Pointer to the host controller structure of
     *                  the host.
     *
     * Returns:
     *
     *  int                     0 ok, -1 error
     */

static inline
int uhci_global_suspend (ts_uhci_host* hci)
{
    int errcode;
    
    gdlf_fntrace ("uhci_global_suspend (0x%p)", hci);

    gdlf_apassert (hci != NULL,
                   { 
                       uhci_errno = usb_errno = -USBE_HCI_NULL; 
                       return -EINVAL; 
                   },
                   "NULL pointer to UHCI host controller structure\n");    
    
    errcode = uhci_runstop (hci, 0);       /* halt the controller */
    if (errcode < 0)
        return errcode;

    write_lock (&hci->snode.lock);
    usb_bits16 (hci->iobase + USBCMD, 0xffff, USBCMD_EGSM);
    write_unlock (&hci->snode.lock);
    return 0;
}


    /* Do a global resume on the whole bus.
     *
     * Parameters:
     *
     *  ts_uhci_host*   Pointer to the host controller structure of
     *                  the host.
     *
     * Returns:
     *
     *  int             0 if ok, errno code < 0 on error.
     */

static inline
int uhci_global_resume (ts_uhci_host* hci)
{
    int errcode;
    
    gdlf_fntrace ("uhci_global_resume (0x%p)", hci);

    gdlf_aassert (hci != NULL, return -EINVAL);

    write_lock (&hci->snode.lock);
    usb_bits16 (hci->iobase + USBCMD, 0xffff, USBCMD_FGR);
    usb_waitms (20);    
    usb_bits16 (hci->iobase + USBCMD, 0x0000, USBCMD_FGR);
    write_unlock (&hci->snode.lock);

    errcode = uhci_runstop (hci, 1);    /* restart the controller */
    return errcode >= 0? 0 : errcode;
}


    /* Set the software debug on the controller. The debug bit may
     * only be set with the controller halted (USBSTS_HCH = 1). Thus,
     * the controller is halted prior to setting the flag.
     *
     * Parameters:
     *
     *  ts_uhci_host*   Pointer to the host controller structure of
     *                  the host.
     *
     *  int             Action to take
     *                  -1  get actual value
     *                  0   reset software debug
     *                  1   set software debug
     *
     * Returns:
     *
     *  int             The state of the bit before changing it (or
     *                  actual if action is -1); < 0 errno code on
     *                  error.
     *
     * Global:
     *
     *  int usb_errno   On error, error code.  
     */

static inline
int uhci_swdebug (ts_uhci_host* hci, int action)
{
    int errcode;

    gdlf_fntrace ("uhci_swdebug (0x%p, %d)", hci, action);

    gdlf_apassert (hci != NULL,
                   { 
                       uhci_errno = usb_errno = -USBE_HCI_NULL; 
                       return -EINVAL; 
                   },
                   "NULL pointer to UHCI host controller structure\n");    
    
    if (action != -1                    /* If trying to set the bit, */
        && (errcode = 
            uhci_runstop (hci, 0)) < 0) /* and cannot stop it ... */
        return errcode;                 /* ... return error */

    write_lock (&hci->snode.lock);
    errcode = (usb_bits16 (hci->iobase + USBCMD, action << 5, 
                           action == -1? 0 : USBCMD_SWDBG) 
               & USBCMD_SWDBG) >> 5;
    write_unlock (&hci->snode.lock);
    uhci_runstop (hci, 1);
    return errcode;
}


    /* Get or set the base pointer for the Frame List. This pointer's
     * 4Kbyte aligned, so a check is performed to ensure it. Hmmm
     *  ... we gotta do address translation (for portability), I guess
     * that we should use virt_to_bus() to tell the controller the
     * exact addess. Intel doesn't say nothing in the specs about the
     * PIIX3 (for example) translating the addresses, so I'll assume
     * it doesn't.
     *
     * So, should we get it from where? the struct or the controller?
     * Using the controller to make it more solid.
     *
     * Parameters:
     *
     *  ts_uhci_host*   Pointer to the host controller structure of
     *                  the host.
     *
     *  unsigned*       [usb_uhci_get_fl()] Where to store the size of
     *                  the Frame List. If NULL, it won't be
     *                  reported.
     *
     *  ts_uhci_flp*    [usb_uhci_set_fl()] The address of the new
     *                  frame list. 
     *
     *  unsigned        [usb_uhci_set_fl()] The new size of the frame
     *                  list; won't set the size if it is 0.
     *
     * Returns:
     *
     *  __u32           [usb_uhci_get_fl()] The actual frame
     *                  number.
     *
     *  unsigned *psize [usb_uhci_get_fl()] The size of the frame
     *                  list (if size != NULL).
     *
     *  int             [usb_uhci_set_fl()] 0 on success, on error, <
     *                  0 errno code.
     * Global:
     *
     *  int usb_errno   On error, error code.  
     */

static inline
__u32 uhci_get_fl (ts_uhci_host *hci, unsigned *psize)
{
    int fl;
    
    gdlf_fntrace ("uhci_get_fl (0x%p, 0x%p)", hci, psize);

    gdlf_apassert (hci != NULL,
                   {    
                       uhci_errno = usb_errno = -USBE_HCI_NULL; 
                       return -EINVAL; 
                   },
                   "NULL pointer to UHCI host controller structure\n");    
    
    if (psize != NULL)          /* gotta report size? ... */
        *psize = UHCI_FL_SIZE;  /* ... done */

    read_lock (&hci->snode.lock);
    fl = (inl (hci->iobase + USBFLBASEADD) & USBFLBASEADD_M);
    read_unlock (&hci->snode.lock);
    return fl;
}


static inline
int uhci_set_fl (ts_uhci_host *hci, ts_uhci_flp* fl)
{
    gdlf_fntrace ("uhci_set_fl (0x%p, 0x%p)", hci, fl);

    gdlf_apassert (hci != NULL,
                   { 
                       uhci_errno = usb_errno = -USBE_HCI_NULL; 
                       return -EINVAL; 
                   },
                   "NULL pointer to UHCI host controller structure\n");    
    
#ifdef UHCI_DEBUG
    if ((__u32)fl & 0x00000fff)     /* check for correct alignment (4 KB) */
    {
        uhci_errno = usb_errno = -USBE_UHCI_FL_BAD_ALIGN;
        gdlf_err ("Tried to set the frame list pointer to "
                    "a mis-aligned (to 4Kbyte) %p\n",
                    fl);
        return -EINVAL;
    }
#endif

    write_lock (&hci->snode.lock);

    outl (le32_to_cpu (virt_to_bus (fl)), hci->iobase + USBFLBASEADD);
    hci->fl = fl;

    write_unlock (&hci->snode.lock);
    
    return 0;
}


    /* Read/Clear status bits for a UHCI host controller.
     *
     * Parameters:  
     *
     *  ts_uhci_host*   Pointer to the host controller structure of
     *                  the host.
     *
     *  __u16           The mask of bits to set (USBSTS_*).
     *
     *  int             1 to clear the bits after reading, 0 to
     *                  just read'em.
     *
     * Returns:     
     *
     *  long            The value of the bits readen, < 0 errno code
     *                  on error.
     *
     * Global:
     *
     *  int usb_errno   On error, error code.  
     */

static inline
long uhci_status (ts_uhci_host *hci, __u16 mask, int clear)
{
    long val;
    
    gdlf_fntrace ("uhci_status (0x%p, 0x%x, %d)", hci, mask, clear);
    gdlf_aassert (hci != NULL, return -EINVAL);
    gdlf_aassert (clear == 0 || clear == 1, return -EINVAL);

        /* 
         * get value(s), got to clear? clear (writing 1, it's an RWC)
         */

    write_lock (&hci->snode.lock);
    val = inw (hci->iobase + USBSTS) & mask;
    if (clear == 1)
        outw_mask (val, 0xffff, mask, hci->iobase + USBSTS);
    write_unlock (&hci->snode.lock);
    return val;
}


    /* Set the interrupt settings for the controller
     *
     * Parameters:  
     *
     *  ts_uhci_host*   Pointer to the host controller structure of
     *                  the host.
     *
     *  int             0 to clear the bits after reading, 1 to
     *                  set'em. -1 to read'em.
     *
     *  __u16           The mask of bits to set (USBINTR_*).
     *
     * Returns:
     *     
     *  long            The value of the bits readen (the value prior
     *                  to setting if set); < 0 errno code on error.
     *
     * Global:
     *
     *  int usb_errno   On error, error code.  
     */

static inline
long uhci_intr (ts_uhci_host *hci, int action, __u16 mask)
{
    int errcode;
    
    gdlf_fntrace ("uhci_intr (0x%p, %d, %x)", hci, action, mask);

    gdlf_aassert (hci != NULL, return -EINVAL); 
    gdlf_aassert (action == 0 || action == 1 || action == -1, 
                  return -EINVAL);

    write_lock (&hci->snode.lock);
    errcode = usb_bits16 (hci->iobase + USBINTR, 
                          action == 1? 0xffff : 0x000, 
                          action == -1? 0 : mask);
    write_unlock (&hci->snode.lock);
    return errcode;
}


    /* Set/get the SOF timing register
     *
     * Parameters:  
     *
     *  ts_uhci_host*   Pointer to the host controller structure of
     *                  the host.
     *
     *  unsigned char   The new SOF timing value
     *                  [usb_uhci_set_sof()]. 
     *
     * Returns:     
     *
     *  long            The value of the bits readen (the value prior
     *                  to setting when set). < 0 errno code on
     *                  error.
     *
     * Global:
     *
     *  int usb_errno   On error, error code.  
     */

#warning FIXME: adopt a single standard to do this between [UO]HCI
static inline
long uhci_get_sof (ts_uhci_host *hci)
{
    long errcode;
    
    gdlf_fntrace ("uhci_get_sof (0x%p)", hci);

    gdlf_apassert (hci != NULL,
                   {
                       uhci_errno = usb_errno = -USBE_HCI_NULL; 
                       return -EINVAL;
                   },
                   "NULL pointer to UHCI host controller structure\n");    
    
    read_lock (&hci->snode.lock);
    errcode = inb (hci->iobase + USBSOF) & USBSOF_M;
    read_unlock (&hci->snode.lock);
    return errcode;
}

static inline
long uhci_set_sof (ts_uhci_host *hci, unsigned char sof)
{
    long errcode;
    
    gdlf_fntrace ("uhci_set_sof (0x%p, %u)", hci, sof);

    gdlf_apassert (hci != NULL,
                   {
                       uhci_errno = usb_errno = -USBE_HCI_NULL; 
                       return -EINVAL; 
                   },
                   "NULL pointer to UHCI host controller structure\n");    

    write_lock (&hci->snode.lock);
    errcode = usb_bits8 (hci->iobase + USBSOF, sof, USBSOF_M) & USBSOF_M;
    write_unlock (&hci->snode.lock);
    return errcode;
}

#endif /* __KERNEL__ */

#endif /* _uhci_h_ */

/* Log:
 *
 * $Log: uhci.h,v $
 * Revision 1.24  1998/11/14 17:45:49  inaky
 * Moved the include files to uusbd/include/uusbd, to ease integration
 * into the Linux kernel.
 *
 * Revision 1.23  1998/11/09 21:45:24  inaky
 * Removed the tr_buffer. Now that's at tr.data and it's management is
 * responsibility of the hub manager.
 *
 * Revision 1.22  1998/10/31 02:21:22  inaky
 * Removed the device setup methods [unused].
 *
 * Revision 1.21  1998/10/14 21:54:00  inaky
 * Debug stuff.
 *
 * Revision 1.20  1998/09/24 16:20:45  inaky
 * Moved 'hub_port' to 'port', added some declarations, made it wait 15
 * ms after asserting reset.
 *
 * Revision 1.19  1998/09/02 21:22:59  inaky
 * Removed GDLF, now th inclusion has to come after declaring GDLF. Moved
 * the ptr2kvoid and kvoid2ptr macros to usb-misc.h.
 *
 * Revision 1.18  1998/08/30 14:02:04  inaky
 * Fixed remaining structures to use memset() when cleaning.
 *
 * Revision 1.17  1998/08/29 00:12:31  inaky
 * Fixed endianess on qhs and tds, and option to disable bandwidth
 * reclamation and severed assertions.
 *
 * Revision 1.16  1998/08/27 22:24:38  inaky
 * Removed traces. Fixed _clean() macros, finished iso and intr hash
 * lists.
 *
 * Revision 1.15  1998/08/26 18:13:23  inaky
 * Fixed iso hash tables to be host controller bound, and added a timer
 * to clean them up periodically, together with the interrupt hash
 * tables.
 *
 * Revision 1.14  1998/08/25 16:46:03  inaky
 * Removed unneeded FIXMEs.
 *
 * Revision 1.13  1998/08/24 02:32:39  inaky
 * Did the little debugging needed to make the functions work as they're
 * expected to.
 *
 * Removed auto-stopping and restarting stuff from uhci_*()
 * functions. Now is responsability of the caller.
 *
 * Revision 1.12  1998/08/20 00:01:23  inaky
 * Added timers for transactions, to keep timeouts.
 *
 * Revision 1.11  1998/08/03 13:53:37  inaky
 * Added a per-transaction timer and fixed some bits here and there.
 *
 * Revision 1.10  1998/07/30 18:01:34  inaky
 * Tweaked some fields, they were swapped in QHs and TDs.
 *
 * Revision 1.9  1998/07/28 23:17:17  inaky
 * Moved the frame list size to be fixed; added constants, the iso-hash
 * declarations and a function to do a global resume.
 *
 * Revision 1.8  1998/07/24 15:40:50  inaky
 * Removed the PAUSED state, it was quite useless ...
 *
 * Revision 1.7  1998/07/24 15:40:37  inaky
 * Added new constants, fixed other's names, added the frbot field to
 * ts_uhci_host, QH list handling functions and use of direct in[bwl],
 * out[bwl] for reading/writing to ports.
 *
 * Revision 1.6  1998/07/23 14:47:33  inaky
 * Added warnings, moved ../usb-uhci.h to uhci.h and removed memory
 * address xlation functions, as we don't need them now.
 *
 * Revision 1.5  1998/07/16 14:19:04  inaky
 * Moved from the old read/write interface to the new, request-based
 * interface.
 *
 * Added some memory functions which will be moved to usb-mm.h.
 *
 * Revision 1.4  1998/07/14 14:58:49  inaky
 * Fixed locking acknowledges of the whole uhci subsystem.
 *
 * Revision 1.3  1998/07/12 22:59:23  inaky
 * Added the vhub declarations.
 *
 * Revision 1.2  1998/05/01 19:22:06  inaky
 * Added decl of the irq handler so it can be seen by driver.c; also
 * prettyprinted the rest of declarations.
 *
 * Revision 1.1  1998/05/01 18:31:29  inaky
 * Moved everything to use pci_dev according to new kernel PCI API. Also
 * split all the code in several files, as old uhci.c was a complete mess.
 *
 */
